dnl Copyright (C) 2004-2005 by Carnegie Mellon University.
dnl
dnl @OPENSOURCE_HEADER_START@
dnl
dnl Use of the SILK system and related source code is subject to the terms
dnl of the following licenses:
dnl
dnl GNU Public License (GPL) Rights pursuant to Version 2, June 1991
dnl Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
dnl
dnl NO WARRANTY
dnl
dnl ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER
dnl PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY
dnl PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN
dnl "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY
dnl KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT
dnl LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE,
dnl MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE
dnl OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT,
dnl SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY
dnl TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF
dnl WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES.
dnl LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF
dnl CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON
dnl CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE
dnl DELIVERABLES UNDER THIS LICENSE.
dnl
dnl Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie
dnl Mellon University, its trustees, officers, employees, and agents from
dnl all claims or demands made against them (and any related losses,
dnl expenses, or attorney's fees) arising out of, or relating to Licensee's
dnl and/or its sub licensees' negligent use or willful misuse of or
dnl negligent conduct or willful misconduct regarding the Software,
dnl facilities, or other rights or assistance granted by Carnegie Mellon
dnl University under this License, including, but not limited to, any
dnl claims of product liability, personal injury, death, damage to
dnl property, or violation of any laws or regulations.
dnl
dnl Carnegie Mellon University Software Engineering Institute authored
dnl documents are sponsored by the U.S. Department of Defense under
dnl Contract F19628-00-C-0003. Carnegie Mellon University retains
dnl copyrights in all material produced under this contract. The U.S.
dnl Government retains a non-exclusive, royalty-free license to publish or
dnl reproduce these documents, or allow others to do so, for U.S.
dnl Government purposes only pursuant to the copyright license under the
dnl contract clause at 252.227.7013.
dnl
dnl @OPENSOURCE_HEADER_END@

dnl RCSIDENT("$SiLK: /autoconf/silkconfig.m4,v 1.25 2005/11/08 22:31:57 thomasm Exp $")


# ---------------------------------------------------------------------------
# SILK_AC_INIT - initialize SiLK autoconf magic
#
# all configure.in files should start with an invocation of SILK_AC_INIT
#
AC_DEFUN([SILK_AC_INIT],[
    SPLINT_FLAGS="+posixlib +unixlib -weak"

    PACKAGE=silk
    PACKAGENAME=silk
    SILK_DATA_ROOTDIR_DEFAULT=/data
    silk_check_cluster=yes

])# SILK_AC_INIT


AC_DEFUN([SILK_AC_MESSAGE],[
    AC_MSG_NOTICE([${PACKAGE} $1])
])


# ---------------------------------------------------------------------------
# SILK_AC_OPT_DATA_ROOTDIR
#
#   Let the user specify the base directory where the packed data
#   should be stored and from where it will be read.
#
#   Output variable: SILK_DATA_ROOTDIR
#
AC_DEFUN([SILK_AC_OPT_DATA_ROOTDIR],[
    AC_ARG_ENABLE([data-rootdir],[AC_HELP_STRING([--enable-data-rootdir=dir],
             [location of packed data files (default=/data)])[]dnl
        ],[
        if test "x$enableval" != "xyes"
        then
            SILK_DATA_ROOTDIR="$enableval"
        fi
        ])

    if test "x${SILK_DATA_ROOTDIR}" = "x"
    then
        SILK_DATA_ROOTDIR="${SILK_DATA_ROOTDIR_DEFAULT}"
    fi

    AC_DEFINE_UNQUOTED(SILK_DATA_ROOTDIR,"$SILK_DATA_ROOTDIR",
        [Path to root directory of packed data files])
    AC_MSG_NOTICE([(${PACKAGE}) Using ${SILK_DATA_ROOTDIR} as packed file data directory])
])# SILK_AC_OPT_DATA_ROOTDIR




# ---------------------------------------------------------------------------
# SILK_AC_CHECK_VARIOUS_DECLS
#
# Check for a bunch of random declarations
#
AC_DEFUN([SILK_AC_CHECK_VARIOUS_DECLS],[
    # The following big block is use to test for these r-values:
    #     EPROTO in <errno.h>
    #     GLOB_ABORTED in <glob.h>
    #     GLOB_NOMATCH in <glob.h>
    #     INADDR_NONE in <sys/types.h> or <netinet/in.h>
    #     UINT*_MAX in <limits.h> or <stdint.h> or <inttypes.h>
    #     ULLONG_MAX in <limits.h> or <stdint.h> or <inttypes.h>
    #     PRIu32, PRIu64 in <limits.h> or <stdint.h> or <inttypes.h>
    #
    AC_CHECK_DECLS([EPROTO, GLOB_ABORTED, GLOB_NOMATCH, INADDR_NONE, UINT8_MAX, UINT16_MAX, UINT32_MAX, UINT64_MAX, ULLONG_MAX, PRIu32, PRIu64], , ,[
#include <stdio.h>
#if HAVE_SYS_TYPES_H
# include <sys/types.h>
#endif
#if HAVE_SYS_STAT_H
# include <sys/stat.h>
#endif
#if STDC_HEADERS
# include <stdlib.h>
# include <stddef.h>
#else
# if HAVE_STDLIB_H
#  include <stdlib.h>
# endif
#endif
#if HAVE_STRING_H
# if !STDC_HEADERS && HAVE_MEMORY_H
#  include <memory.h>
# endif
# include <string.h>
#endif
#if HAVE_STRINGS_H
# include <strings.h>
#endif
#if HAVE_INTTYPES_H
# include <inttypes.h>
#else
# if HAVE_STDINT_H
#  include <stdint.h>
# endif
#endif
#if HAVE_UNISTD_H
# include <unistd.h>
#endif
#if HAVE_ERRNO_H
# include <errno.h>
#endif
#if HAVE_GLOB_H
# include <glob.h>
#endif
#if HAVE_LIMITS_H
# include <limits.h>
#endif
#if HAVE_NETINET_IN_H
# include <netinet/in.h>
#endif
    ])
])# SILK_AC_CHECK_VARIOUS_DECLS


# ---------------------------------------------------------------------------
# SILK_AC_COMP_ATTRIBUTE_USED
#
# Check whether the compile understands the __attribute__((__used__))
#
AC_DEFUN([SILK_AC_COMP_ATTRIBUTE_USED],
    [AC_MSG_CHECKING([whether the compiler understands __attribute__((__used__))])

    # Don't use AC_LANG_WERROR since it applies to *ALL* configure
    # tests and there is no way to disable it once enabled, nor a way
    # to enable it only for this test.

    sk_save_CFLAGS="$CFLAGS"
    CFLAGS="-Werror $CFLAGS"

    AC_COMPILE_IFELSE(
        [AC_LANG_PROGRAM([],[
             static char *x __attribute__((__used__)) = "used";
             ])
        ],[
            AC_DEFINE(HAVE_COMP_ATTRIBUTE_USED, 1,
                [Define to 1 if your compiler understands __attribute__((__used__)).])
            AC_MSG_RESULT([yes])],
        [AC_MSG_RESULT([no])])

    CFLAGS="$sk_save_CFLAGS"
])# SILK_AC_COMP_ATTRIBUTE_USED


# ---------------------------------------------------------------------------
# SILK_AC_NEED_REENTRANT
#
# Check whether -D_REENTRANT is required for certain functions (Solaris)
#
AC_DEFUN([SILK_AC_NEED_REENTRANT],[
    AC_CHECK_DECL(gmtime_r, ,[
        AC_MSG_CHECKING([whether gmtime_r is declared with -D_REENTRANT])
        AC_COMPILE_IFELSE([
            AC_LANG_PROGRAM([
#define _REENTRANT 1
#include <stdio.h>
#if HAVE_SYS_TYPES_H
# include <sys/types.h>
#endif
#if     TIME_WITH_SYS_TIME
#  include <sys/time.h>
#  include <time.h>
#else
#  if   HAVE_SYS_TIME_H
#    include <sys/time.h>
#  else
#    include <time.h>
#  endif
#endif
],[
#ifndef gmtime_r
    char *p = (char *) gmtime_r;
#endif
])],[
            AC_DEFINE(SK_NEED_REENTRANT, 1,
                [Define to 1 if your system requires _REENTRANT to be defined for multi-threaded programs])
            AC_MSG_RESULT([yes])],
            [AC_MSG_RESULT([no])])
    ],[
#include <stdio.h>
#if HAVE_SYS_TYPES_H
# include <sys/types.h>
#endif
#if     TIME_WITH_SYS_TIME
#  include <sys/time.h>
#  include <time.h>
#else
#  if   HAVE_SYS_TIME_H
#    include <sys/time.h>
#  else
#    include <time.h>
#  endif
#endif
    ])
])#SILK_AC_NEED_REENTRANT


# ---------------------------------------------------------------------------
# SILK_AC_HEADER_DEFINES_SYMBOL(hdr, sym, var)
#
# Check for symbol SYM in header HDR, defining VAR if it exists
#
# NOTE:  It is probably better to use AC_CHECK_DECLS() for this
#
AC_DEFUN([SILK_AC_HEADER_DEFINES_SYMBOL],[
    sk_acfg_hdr="$1"
    sk_acfg_sym="$2"
    sk_acfg_var="$3"

    AC_MSG_CHECKING([whether <$sk_acfg_hdr> defines $sk_acfg_sym])
    AC_COMPILE_IFELSE([
       AC_LANG_PROGRAM([#include <$sk_acfg_hdr>],
           [return $sk_acfg_sym ;])],[
       AC_DEFINE_UNQUOTED(${sk_acfg_var},[1],
           [Define to 1 if <${sk_acfg_hdr}> defines ${sk_acfg_sym}])
       AC_MSG_RESULT([yes])],
       [AC_MSG_RESULT([no])])
])# SILK_AC_HEADER_DEFINES_SYMBOL


# ---------------------------------------------------------------------------
# SILK_AC_HEADER_DEFINES_VARIABLE(hdr, varname, type, var)
#
# Check for variable VARNAME as reducable to type TYPE in header HDR,
# defining VAR if it exists
#
AC_DEFUN([SILK_AC_HEADER_DEFINES_VARIABLE],[
    sk_acfg_hdr="$1"
    sk_acfg_varname="$2"
    sk_acfg_type="$3"
    sk_acfg_var="$4"

    AC_MSG_CHECKING([whether <$sk_acfg_hdr> defines $sk_acfg_varname as $sk_acfg_type])
    AC_RUN_IFELSE([
        AC_LANG_PROGRAM([#include <$sk_acfg_hdr>],[
            $sk_acfg_type somevar = $sk_acfg_varname;
            return (sizeof($sk_acfg_type) !=
                    sizeof($sk_acfg_varname));])],[
        AC_DEFINE_UNQUOTED(${sk_acfg_var},[1],
            [Define to 1 if <${sk_acfg_hdr}> defines ${sk_acfg_sym} as ${sk_acfg_type}])
        AC_MSG_RESULT([yes])],
        [AC_MSG_RESULT([no])],
        [AC_MSG_RESULT([no])])
])# SILK_AC_HEADER_DEFINES_SYMBOL


# ---------------------------------------------------------------------------
# SILK_AC_STD_SUBSTS
#
# Do final substitutions
#
AC_DEFUN([SILK_AC_STD_SUBSTS],[
    AC_SUBST(SPLINT_FLAGS)
    if test -n "$ac_cv_path_MAILX"
    then
       AC_DEFINE_UNQUOTED([SK_MAILER],"$ac_cv_path_MAILX",[Path to mail program used for sending alerts])
    fi
])


# ---------------------------------------------------------------------------
# SILK_AC_LIB_ZLIB
#
# Determine how to use zlib
#
AC_DEFUN(SILK_AC_LIB_ZLIB,[
    AC_SUBST(INCLUDE_ZLIB)
    AC_SUBST(LIB_Z)
    AC_ARG_WITH([zlib],[AC_HELP_STRING([--with-zlib=dir],
            [find zlib.h in DIR/include, libz in DIR/lib])[]dnl
        ],[
            if test "x$withval" != "xyes"
            then
                zlib_dir="$withval"
            fi
    ])

    sk_zlib_found=0
    if test "x$zlib_dir" != "xno"
    then
        # Cache current values
        sk_save_LDFLAGS="$LDFLAGS"
        sk_save_CFLAGS="$CFLAGS"
        sk_save_CPPFLAGS="$CPPFLAGS"

        if test "x$zlib_dir" = "x"
        then
            LIB_Z="-lz"
            INCLUDE_ZLIB=
        else
            # When explicit directory given, ignore previous LDFLAGS,
            # CFLAGS, CPPFLAGS values.
            LDFLAGS="-L$zlib_dir/lib"
            LIB_Z="$LDFLAGS -lz"

            INCLUDE_ZLIB="-I$zlib_dir/include"
            CFLAGS="$INCLUDE_ZLIB"
            CPPFLAGS="$INCLUDE_ZLIB"
        fi

        AC_CHECK_LIB([z], [gzopen],[
            sk_zlib_found=1])

        AC_CHECK_HEADER([zlib.h],[
            AC_DEFINE(HAVE_ZLIB_H, 1,
                [Define to 1 if you have the <zlib.h> header file.])
            sk_zlib_found="1$sk_zlib_found"])

        # Restore cached values
        LDFLAGS="$sk_save_LDFLAGS"
        CFLAGS="$sk_save_CFLAGS"
        CPPFLAGS="$sk_save_CPPFLAGS"
    fi

    if test "$sk_zlib_found" -lt 11
    then
        AC_MSG_NOTICE([(${PACKAGE}) Cannot build collection/packing tools without zlib])
        sk_zlib_found=0
        LIB_Z=
        INCLUDE_ZLIB=
    else
        sk_zlib_found=1
    fi
])# SILK_AC_LIB_ZLIB


# ---------------------------------------------------------------------------
# SILK_AC_LIB_PCAP
#
# Determine how to use pcap
#
AC_DEFUN(SILK_AC_LIB_PCAP,[
    AC_SUBST(INCLUDE_PCAP)
    AC_SUBST(LIB_PCAP)
    AC_ARG_WITH([pcap],[AC_HELP_STRING([--with-pcap=dir],
            [find pcap.h in DIR/include, libpcap in DIR/lib])[]dnl
        ],[
            if test "x$withval" != "xyes"
            then
                pcap_dir="$withval"
            fi
    ])

    sk_pcap_found=0
    if test "x$pcap_dir" != "xno"
    then
        # Cache current values
        sk_save_LDFLAGS="$LDFLAGS"
        sk_save_CFLAGS="$CFLAGS"
        sk_save_CPPFLAGS="$CPPFLAGS"

        if test "x$pcap_dir" = "x"
        then
            LIB_PCAP="-lpcap"
            INCLUDE_PCAP=
        else
            # When explicit directory given, ignore previous LDFLAGS,
            # CFLAGS, CPPFLAGS values.
            LDFLAGS="-L$pcap_dir/lib"
            LIB_PCAP="$LDFLAGS -lpcap"

            INCLUDE_PCAP="-I$pcap_dir/include"
            CFLAGS="$INCLUDE_PCAP"
            CPPFLAGS="$INCLUDE_PCAP"
        fi

        AC_CHECK_LIB([pcap], [pcap_open_dead],[
            sk_pcap_found=1])

        AC_CHECK_HEADER([pcap.h],[
            AC_DEFINE(HAVE_PCAP_H, 1,
                [Define to 1 if you have the <pcap.h> header file.])
            sk_pcap_found="1$sk_pcap_found"])

        # Restore cached values
        LDFLAGS="$sk_save_LDFLAGS"
        CFLAGS="$sk_save_CFLAGS"
        CPPFLAGS="$sk_save_CPPFLAGS"
    fi

    if test "$sk_pcap_found" -lt 11
    then
        AC_MSG_NOTICE([(${PACKAGE}) Cannot build rwp* tools without pcap])
        sk_pcap_found=0
        LIB_PCAP=
        INCLUDE_PCAP=
    else
        sk_pcap_found=1
    fi
])# SILK_AC_LIB_PCAP


# ---------------------------------------------------------------------------
# SILK_AC_LIB_PCRE
#
# Determine how to use pcre
#
AC_DEFUN(SILK_AC_LIB_PCRE,[
    AC_SUBST(INCLUDE_PCRE)
    AC_SUBST(LIB_PCRE)
    AC_ARG_WITH([pcre],[AC_HELP_STRING([--with-pcre=dir],
            [find pcre.h in DIR/include, libpcre in DIR/lib])[]dnl
        ],[
            if test "x$withval" != "xyes"
            then
                pcre_dir="$withval"
            fi
    ])

    sk_pcre_found=0
    if test "x$pcre_dir" != "xno"
    then
        # Cache current values
        sk_save_LDFLAGS="$LDFLAGS"
        sk_save_CFLAGS="$CFLAGS"
        sk_save_CPPFLAGS="$CPPFLAGS"

        if test "x$pcre_dir" = "x"
        then
            LIB_PCRE="-lpcre"
            INCLUDE_PCRE=
        else
            # When explicit directory given, ignore previous LDFLAGS,
            # CFLAGS, CPPFLAGS values.
            LDFLAGS="-L$pcre_dir/lib"
            LIB_PCRE="$LDFLAGS -lpcre"

            INCLUDE_PCRE="-I$pcre_dir/include"
            CFLAGS="$INCLUDE_PCRE"
            CPPFLAGS="$INCLUDE_PCRE"
        fi

        AC_CHECK_LIB([pcre], [pcre_compile],[
            sk_pcre_found=1])

        # Look for the header twice, once as pcre.h; if that fails,
        # try pcre/pcre.h.

        AC_CHECK_HEADER([pcre.h],[
            AC_DEFINE(HAVE_PCRE_H, 1,
                [Define to 1 if you have the <pcre.h> header file.])
            sk_pcre_found="1$sk_pcre_found"],[
            AC_CHECK_HEADER([pcre/pcre.h],[
                AC_DEFINE(HAVE_PCRE_PCRE_H, 1,
                    [Define to 1 if you have the <pcre/pcre.h> header file.])
                sk_pcre_found="$sk_pcre_found"1])])

        # Restore cached values
        LDFLAGS="$sk_save_LDFLAGS"
        CFLAGS="$sk_save_CFLAGS"
        CPPFLAGS="$sk_save_CPPFLAGS"
    fi

    if test "$sk_pcre_found" -lt 11
    then
        sk_pcre_found=0
        LIB_PCRE=
        INCLUDE_PCRE=
    else
        sk_pcre_found=1
    fi
])# SILK_AC_LIB_PCRE


# ---------------------------------------------------------------------------
# SILK_AC_LIB_PTHREAD
#
# Determine how to use pthreads
#
AC_DEFUN(SILK_AC_LIB_PTHREAD,[
    AC_SUBST(LIB_PTHREAD)

    # libexc is required for OSF/1 or TruUNIX or Tru64 or whatever
    # DEC/Compaq/HP is calling it now

    AC_CHECK_LIB([exc], [exc_resume], [LIB_PTHREAD="-lexc $LIB_PTHREAD"])
    AC_CHECK_LIB([pthread], [pthread_key_delete],
        [LIB_PTHREAD="-lpthread $LIB_PTHREAD"], ,[$LIB_PTHREAD])

    if test "X" = "X$LIB_PTHREAD"
    then
        # FreeBSD just needs -pthread on link line
        AC_MSG_CHECKING([whether -pthread is required for pthread support])
        sk_save_LIBS="$LIBS"
        LIBS="$LIBS -pthread"
        AC_LINK_IFELSE([
            AC_LANG_PROGRAM([
#if HAVE_PTHREAD_H
#include <pthread.h>
#endif
                ],[
pthread_key_t x;
pthread_key_delete(x)])],[
            LIB_PTHREAD="-pthread"
            AC_MSG_RESULT([yes])],
            [AC_MSG_RESULT([no])])

        LIBS="$sk_save_LIBS"
    fi
])# SILK_AC_LIB_PTHREAD


# ---------------------------------------------------------------------------
# SILK_AC_FIND_SOURCE_MAKEFILES
#
# Find all the subdirectories to build
#
AC_DEFUN(SILK_AC_FIND_SOURCE_MAKEFILES,[
    SK_LIBRARY_MAKEFILES=''
    AC_SUBST(SK_LIBRARY_MAKEFILES)
    SILK_AC_LIBRARY_MAKEFILES(sk_makes)
    SILK_AC_ADD_MAKEFILES_TO_VAR([$sk_makes], SK_LIBRARY_MAKEFILES)

    SK_ANALYSIS_MAKEFILES=''
    AC_SUBST(SK_ANALYSIS_MAKEFILES)
    SILK_AC_ANALYSIS_MAKEFILES(sk_makes)
    SILK_AC_ADD_MAKEFILES_TO_VAR([$sk_makes], SK_ANALYSIS_MAKEFILES)

    SK_PACKING_MAKEFILES=''
    AC_SUBST(SK_PACKING_MAKEFILES)
    SILK_AC_PACKING_MAKEFILES(sk_makes)
    SILK_AC_ADD_MAKEFILES_TO_VAR([$sk_makes], SK_PACKING_MAKEFILES)

    SK_EXTRA_MAKEFILES=''
    AC_SUBST(SK_EXTRA_MAKEFILES)
    SILK_AC_EXTRA_MAKEFILES(sk_makes)
    SILK_AC_ADD_MAKEFILES_TO_VAR([$sk_makes], SK_EXTRA_MAKEFILES)
])# SILK_AC_FIND_SOURCE_MAKEFILES



# ---------------------------------------------------------------------------
# SILK_AC_ADD_MAKEFILES_TO_VAR(makelist, var)
#
# For each path in makelist, if it is a file under the $srcdir, add it
# to the variable 'var'
#
AC_DEFUN(SILK_AC_ADD_MAKEFILES_TO_VAR,[
    sk_make_list=''
    for i in $1 ; do
        if test -f "$srcdir/$i"
        then
            sk_make_list="$sk_make_list$i "
        fi
    done
    $2="$$2$sk_make_list"
])



# ---------------------------------------------------------------------------
# SILK_AC_LIBRARY_MAKEFILES
#
# Return a list of all potential library makefiles
#
AC_DEFUN(SILK_AC_LIBRARY_MAKEFILES,
 [$1='
src/util/utils/Makefile
src/rw/librw/Makefile
'])



# ---------------------------------------------------------------------------
# SILK_AC_ANALYSIS_MAKEFILES
#
# Return a list of all potential analysis-tools makefiles
#
AC_DEFUN(SILK_AC_ANALYSIS_MAKEFILES,
 [$1='
src/ds/hashlib/Makefile
src/ds/heaplib/Makefile
src/ds/iptree/Makefile
src/util/iochecks/Makefile
src/util/dynlib/Makefile
src/util/ccfilter_new/Makefile
src/util/stats/Makefile
src/util/num2dot/Makefile
src/rw/rwaddrcount/Makefile
src/rw/rwappend/Makefile
src/rw/rwcat/Makefile
src/rw/rwcount/Makefile
src/rw/rwcut/Makefile
src/rw/rwdynlibs/Makefile
src/rw/rwfilter/Makefile
src/rw/rwgroup/Makefile
src/rw/rwinfo/Makefile
src/rw/rwmatch/Makefile
src/rw/rwrandomizeip/Makefile
src/rw/rwset/Makefile
src/rw/rwsort/Makefile
src/rw/rwstats/Makefile
src/rw/rwswapbytes/Makefile
src/rw/rwtotal/Makefile
src/rw/rwuniq/Makefile
src/rw/rwutils/Makefile
'
if test "0$sk_pcap_found" -gt 0
then
    $1="$$1 src/rw/rwpmatch/Makefile"
fi
])# SILK_AC_ANALYSIS_MAKEFILES



# ---------------------------------------------------------------------------
# SILK_AC_PACKING_MAKEFILES
#
# Return a list of all potential packing-tool makefiles
#
AC_DEFUN(SILK_AC_PACKING_MAKEFILES,
 [
if test "0$sk_zlib_found" -gt 0
then
    $1='
src/util/log/Makefile
src/rw/flowsource/Makefile
src/rw/rwpack/Makefile
'
fi
])# SILK_AC_PACKING_MAKEFILES



# ---------------------------------------------------------------------------
# SILK_AC_EXTRA_MAKEFILES
#
# Return a list of all other (extra/experimental) makefiles
#
AC_DEFUN(SILK_AC_EXTRA_MAKEFILES,
 [$1=''
])# SILK_AC_EXTRA_MAKEFILES
