#####################################################-*- makefile -*-##
# CERT SiLK Analysis Suite Global Makefile
# $SiLK: /scripts/build/build.mk,v 1.32 2005/10/27 19:23:41 thomasm Exp $
#######################################################################

ifndef BUILDROOT
include $(SUITEROOT)/scripts/build/skconfig.mk
endif

include $(SUITEROOT)/scripts/build/target.mk

# option to make used when calling onesself recursively without
# changing directories.
MAKE_RECURSIVE_OPT += -f $(SUITEROOT)/scripts/build/recurse.mk


#######################################################################
# Functions to be invoked by $(call xxx,args)
#######################################################################

# $(call mkobjs,SUFFIX,NAMES)
#     For each name in NAMES, this function removes any existing
#     suffix, prepends "$(O)" to the name and appends SUFFIX to it.
#     SUFFIX should not be blank.
# Ex: $(call mkobjs,.g,foo.o bar.o baz.o)
#         ==> $(O)foo.g $(O)bar.g $(O)baz.g
# Ex: $(call mkobjs,.o,foo.o bar.o baz.o)
#         ==> $(O)foo.o $(O)bar.o $(O)baz.o

mkobjs = $(foreach name,$(2),$(O)$(basename $(name))$(strip $(1)))


# $(call mklibs,LABEL,DEPLIBS)
#     For each lib in DEPLIBS, where lib looks like
#     "util/fglob/libfglob.a", this function inserts "$(O)" before the
#     library name and inserts LABEL before the ".a" extension.  LABEL
#     may be blank.
# Ex: $(call mklibs,_g,util/fglob/libfglob.a ds/hashlib/libhash.a)
#         ==> util/fglob/$(O)libfglob_g.a ds/hashlib/$(O)libhash_g.a)
# Ex: $(call mklibs,,util/fglob/libfglob.a ds/hashlib/libhash.a)
#         ==> util/fglob/$(O)libfglob.a ds/hashlib/$(O)libhash.a)

mklibs = $(foreach lib,$(2),$(BUILDROOT)/src/$(dir $(lib))$(O)$(basename $(notdir $(lib)))$(strip $(1)).a)


#######################################################################
# First pass through makefile, recurse with the correct object list
# for the target
#######################################################################

ifneq ($(strip $(OBJS) $(foreach app, $(APPNAMES), $(OBJS_$(app)))),)
$(APPLS_LOCAL_RELEASE): %: always_rebuild
	@WITHDEPS_OBJ="$(call mkobjs,.o,$(OBJS) $(OBJS_$(*)))" WITHDEPS_LIB="$(call mklibs,,$(DEPLIBS_$(*)) $(DEPLIBS))" WITHDEP_LDFLAGS="$(LDFLAGS_$(*))" WITHDEP_CFLAGS="$(CFLAGS_$(*))" APP_RELEASE="release" APPNAMES="$@" $(MAKE) $(MAKE_RECURSIVE_OPT) $@

$(APPLS_LOCAL_DEBUG): %_g: always_rebuild
	@WITHDEPS_OBJ="$(call mkobjs,.g,$(OBJS) $(OBJS_$(*)))" WITHDEPS_LIB="$(call mklibs,_g,$(DEPLIBS_$(*)) $(DEPLIBS))" WITHDEP_LDFLAGS="$(LDFLAGS_$(*))" WITHDEP_CFLAGS="$(CFLAGS_$(*))" APP_RELEASE="debug" APPNAMES="$@" $(MAKE) $(MAKE_RECURSIVE_OPT) $@

$(APPLS_LOCAL_PROFILE): %_p: always_rebuild
	@WITHDEPS_OBJ="$(call mkobjs,.P,$(OBJS) $(OBJS_$(*)))" WITHDEPS_LIB="$(call mklibs,_p,$(DEPLIBS_$(*)) $(DEPLIBS))" WITHDEP_LDFLAGS="$(LDFLAGS_$(*))" WITHDEP_CFLAGS="$(CFLAGS_$(*))" APP_RELEASE="profile" APPNAMES="$@" $(MAKE) $(MAKE_RECURSIVE_OPT) $@
endif


ifneq ($(strip $(TESTOBJS) $(foreach app, $(TESTAPPS), $(TESTOBJS_$(app)))),)
$(TAPPS_LOCAL_RELEASE): %: always_rebuild
	@WITHDEPS_OBJ="$(call mkobjs,.o,$(TESTOBJS) $(TESTOBJS_$(*)))" WITHDEPS_LIB="$(call mklibs,,$(DEPLIBS_$(*)) $(DEPLIBS))" WITHDEP_LDFLAGS="$(LDFLAGS_$(*))" WITHDEP_CFLAGS="$(CFLAGS_$(*))" APP_RELEASE="release" TESTAPPS="$@" $(MAKE) $(MAKE_RECURSIVE_OPT) $@

$(TAPPS_LOCAL_DEBUG): %_g: always_rebuild
	@WITHDEPS_OBJ="$(call mkobjs,.g,$(TESTOBJS) $(TESTOBJS_$(*)))" WITHDEPS_LIB="$(call mklibs,_g,$(DEPLIBS_$(*)) $(DEPLIBS))" WITHDEP_LDFLAGS="$(LDFLAGS_$(*))" WITHDEP_CFLAGS="$(CFLAGS_$(*))" APP_RELEASE="debug" TESTAPPS="$@" $(MAKE) $(MAKE_RECURSIVE_OPT) $@

$(TAPPS_LOCAL_PROFILE): %_p: always_rebuild
	@WITHDEPS_OBJ="$(call mkobjs,.P,$(TESTOBJS) $(TESTOBJS_$(*)))" WITHDEPS_LIB="$(call mklibs,_p,$(DEPLIBS_$(*)) $(DEPLIBS))" WITHDEP_LDFLAGS="$(LDFLAGS_$(*))" WITHDEP_CFLAGS="$(CFLAGS_$(*))" APP_RELEASE="profile" TESTAPPS="$@" $(MAKE) $(MAKE_RECURSIVE_OPT) $@
endif


ifneq ($(strip $(LIBOBJS) $(foreach lib, $(LIBRARIES), $(LIBOBJS_$(lib)))),)
$(LIBRS_LOCAL_RELEASE): $(O)lib%.a: always_rebuild
	@WITHDEPS_OBJ="$(call mkobjs,.o,$(LIBOBJS) $(LIBOBJS_$(*)))" WITHDEP_CFLAGS="$(CFLAGS_$(*))" LIBRARIES="$(*)" $(MAKE) $(MAKE_RECURSIVE_OPT) $@

$(LIBRS_LOCAL_DEBUG): $(O)lib%_g.a: always_rebuild
	@WITHDEPS_OBJ="$(call mkobjs,.g,$(LIBOBJS) $(LIBOBJS_$(*)))" WITHDEP_CFLAGS="$(CFLAGS_$(*))" LIBRARIES="$(*)" $(MAKE) $(MAKE_RECURSIVE_OPT) $@

$(LIBRS_LOCAL_PROFILE): $(O)lib%_p.a: always_rebuild
	@WITHDEPS_OBJ="$(call mkobjs,.P,$(LIBOBJS) $(LIBOBJS_$(*)))" WITHDEP_CFLAGS="$(CFLAGS_$(*))" LIBRARIES="$(*)" $(MAKE) $(MAKE_RECURSIVE_OPT) $@
endif


ifneq ($(strip $(DLOBJS) $(foreach lib, $(DYNALIBS), $(DLOBJS_$(lib)))),)
$(DYNAS_LOCAL_RELEASE): $(O)lib%.$(SHLIB_EXT): always_rebuild
	@WITHDEPS_OBJ="$(call mkobjs,.o,$(DLOBJS) $(DLOBJS_$(*)))" WITHDEPS_LIB="$(call mklibs,,$(DEPLIBS_$(*)) $(DEPLIBS))" WITHDEP_LDFLAGS="$(LDFLAGS_$(*))" WITHDEP_CFLAGS="$(CFLAGS_$(*)) $(SHLIB_CCFLAGS)" APP_RELEASE="release" DYNALIBS="$(*)" $(MAKE) $(MAKE_RECURSIVE_OPT) $@

$(DYNAS_LOCAL_DEBUG): $(O)lib%_g.$(SHLIB_EXT): always_rebuild
	@WITHDEPS_OBJ="$(call mkobjs,.g,$(DLOBJS) $(DLOBJS_$(*)))" WITHDEPS_LIB="$(call mklibs,_g,$(DEPLIBS_$(*)) $(DEPLIBS))" WITHDEP_LDFLAGS="$(LDFLAGS_$(*))" WITHDEP_CFLAGS="$(CFLAGS_$(*)) $(SHLIB_CCFLAGS)" APP_RELEASE="debug" DYNALIBS="$(*)" $(MAKE) $(MAKE_RECURSIVE_OPT) $@

$(DYNAS_LOCAL_PROFILE): $(O)lib%_p.$(SHLIB_EXT): always_rebuild
	@WITHDEPS_OBJ="$(call mkobjs,.P,$(DLOBJS) $(DLOBJS_$(*)))" WITHDEPS_LIB="$(call mklibs,_p,$(DEPLIBS_$(*)) $(DEPLIBS))" WITHDEP_LDFLAGS="$(LDFLAGS_$(*))" WITHDEP_CFLAGS="$(CFLAGS_$(*)) $(SHLIB_CCFLAGS)" APP_RELEASE="profile" DYNALIBS="$(*)" $(MAKE) $(MAKE_RECURSIVE_OPT) $@
endif

