#####################################################-*- makefile -*-##
# CERT SiLK Analysis Suite Global Makefile
# $SiLK: /scripts/build/target.mk,v 1.3 2005/07/01 15:39:20 dugganm Exp $
#######################################################################


#######################################################################
# Variables listings possible targets
#######################################################################

# Create variables for debugging and profiling

APPLS_LOCAL_RELEASE = $(APPNAMES)
APPLS_LOCAL_DEBUG   = $(APPNAMES:%=%_g)
APPLS_LOCAL_PROFILE = $(APPNAMES:%=%_p)
APPLS_LOCAL_ALL = $(foreach x,RELEASE DEBUG PROFILE,$(APPLS_LOCAL_$(x)))

TAPPS_LOCAL_RELEASE = $(TESTAPPS)
TAPPS_LOCAL_DEBUG   = $(TESTAPPS:%=%_g)
TAPPS_LOCAL_PROFILE = $(TESTAPPS:%=%_p)
TAPPS_LOCAL_ALL = $(foreach x,RELEASE DEBUG PROFILE,$(TAPPS_LOCAL_$(x)))

LIBRS_LOCAL_RELEASE = $(foreach lib,$(LIBRARIES),$(O)lib$(lib).a)
LIBRS_LOCAL_DEBUG   = $(foreach lib,$(LIBRARIES),$(O)lib$(lib)_g.a)
LIBRS_LOCAL_PROFILE = $(foreach lib,$(LIBRARIES),$(O)lib$(lib)_p.a)
LIBRS_LOCAL_ALL = $(foreach x,RELEASE DEBUG PROFILE,$(LIBRS_LOCAL_$(x)))

DYNAS_LOCAL_RELEASE = $(foreach lib,$(DYNALIBS),$(O)lib$(lib).$(SHLIB_EXT))
DYNAS_LOCAL_DEBUG   = $(foreach lib,$(DYNALIBS),$(O)lib$(lib)_g.$(SHLIB_EXT))
DYNAS_LOCAL_PROFILE = $(foreach lib,$(DYNALIBS),$(O)lib$(lib)_p.$(SHLIB_EXT))
DYNAS_LOCAL_ALL = $(foreach x,RELEASE DEBUG PROFILE,$(DYNAS_LOCAL_$(x)))


APPLS_RELEASE = $(addprefix $(B),$(APPLS_LOCAL_RELEASE))
APPLS_DEBUG   = $(addprefix $(B),$(APPLS_LOCAL_DEBUG))
APPLS_PROFILE = $(addprefix $(B),$(APPLS_LOCAL_PROFILE))
APPLS_ALL = $(foreach x,RELEASE DEBUG PROFILE,$(APPLS_$(x)))

TAPPS_RELEASE = $(addprefix $(B),$(TAPPS_LOCAL_RELEASE))
TAPPS_DEBUG   = $(addprefix $(B),$(TAPPS_LOCAL_DEBUG))
TAPPS_PROFILE = $(addprefix $(B),$(TAPPS_LOCAL_PROFILE))
TAPPS_ALL = $(foreach x,RELEASE DEBUG PROFILE,$(TAPPS_$(x)))

LIBRS_RELEASE = $(LIBRS_LOCAL_RELEASE:$(O)%=$(L)%)
LIBRS_DEBUG   = $(LIBRS_LOCAL_DEBUG:$(O)%=$(L)%)
LIBRS_PROFILE = $(LIBRS_LOCAL_PROFILE:$(O)%=$(L)%)
LIBRS_ALL = $(foreach x,RELEASE DEBUG PROFILE,$(LIBRS_$(x)))

DYNAS_RELEASE = $(DYNAS_LOCAL_RELEASE:$(O)%=$(L)%)
DYNAS_DEBUG   = $(DYNAS_LOCAL_DEBUG:$(O)%=$(L)%)
DYNAS_PROFILE = $(DYNAS_LOCAL_PROFILE:$(O)%=$(L)%)
DYNAS_ALL = $(foreach x,RELEASE DEBUG PROFILE,$(DYNAS_$(x)))


#######################################################################
# Targets
#######################################################################

appnames-release:: $(APPLS_RELEASE)
appnames-debug::   $(APPLS_DEBUG)
appnames-profile:: $(APPLS_PROFILE)

testapps-release:: $(TAPPS_RELEASE)
testapps-debug::   $(TAPPS_DEBUG)
testapps-profile:: $(TAPPS_PROFILE)

libraries-release:: $(LIBRS_RELEASE)
libraries-debug::   $(LIBRS_DEBUG)
libraries-profile:: $(LIBRS_PROFILE)

dynalibs-release:: $(DYNAS_RELEASE)
dynalibs-debug::   $(DYNAS_DEBUG)
dynalibs-profile:: $(DYNAS_PROFILE)

local-release:: $(LIBRS_LOCAL_RELEASE) $(APPLS_LOCAL_RELEASE) $(DYNAS_LOCAL_RELEASE) $(TAPPS_LOCAL_RELEASE)
local-debug:: $(LIBRS_LOCAL_DEBUG) $(APPLS_LOCAL_DEBUG) $(DYNAS_LOCAL_DEBUG) $(TAPPS_LOCAL_DEBUG)
local-profile:: $(LIBRS_LOCAL_PROFILE) $(APPLS_LOCAL_PROFILE) $(DYNAS_LOCAL_PROFILE) $(TAPPS_LOCAL_PROFILE)


$(APPLS_ALL): $(B)%: %
	$(TEST) -d $(B) || $(MKDIR) -p $(B)
	$(CP) $^ $@.tmp
	-$(RM) -f $@
	$(MV) $@.tmp $@

$(TAPPS_ALL): $(B)%: %
	$(TEST) -d $(B) || $(MKDIR) -p $(B)
	$(CP) $^ $@.tmp
	-$(RM) -f $@
	$(MV) $@.tmp $@

$(LIBRS_ALL): $(L)%: $(O)%
	$(TEST) -d $(L) || $(MKDIR) -p $(L)
	$(CP) $^ $@.tmp
	-$(RM) -f $@
	$(MV) $@.tmp $@
	$(RANLIB) $@

$(DYNAS_ALL): $(L)%: $(O)%
	$(TEST) -d $(L) || $(MKDIR) -p $(L)
	$(CP) $^ $@.tmp
	$(RM) -f $@
	$(MV) $@.tmp $@


install-appnames::  $(APPLS_LOCAL_RELEASE)
	@for i in $^ /dev/null ; do \
	  if $(TEST) -f "$$i" ; then \
	    echo $(INSTALL_PROGRAM) $$i $(bindir)/$$i ; \
	    $(INSTALL_PROGRAM) $$i $(bindir)/$$i ; \
	  fi ; \
	done

install-libraries:: 
	@for i in $(^F) /dev/null ; do \
	  if $(TEST) -f "$(O)$$i" ; then \
	    echo $(INSTALL_PROGRAM) $(O)$$i $(libdir)/$$i ; \
	    $(INSTALL_DATA) $(O)$$i $(libdir)/$$i ; \
	    $(RANLIB) $(libdir)/$$i ; \
	  fi ; \
	done

install-dynalibs::  $(DYNAS_LOCAL_RELEASE)
	@for i in $(^F) /dev/null ; do \
	  if $(TEST) -f "$(O)$$i" ; then \
	    echo $(INSTALL_PROGRAM) $(O)$$i $(libdir)/$$i ; \
	    $(INSTALL_PROGRAM) $(O)$$i $(libdir)/$$i ; \
	  fi ; \
	done

# Additional rule so "make libXXX.a" does something useful
$(subst $(O), ,$(LIBRS_LOCAL_ALL)): %.a: $(O)%.a
