
/*-------------------------------------------------------------------------
 *  filebuf.h : file buffer to keep actual disk reads to large chunks
 *
 *  Copyright (C) 2004 by Accelerated Servers,
 *  			  Todd K McAnally <todd@acceleratedservers.com>
 *  			  Charles Galpin  <charles@acceleratedservers.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $SiLK: filebuf.h,v 1.3 2004/03/11 15:40:53 tmcanally Exp $
 *-----------------------------------------------------------------------*/

#ifndef _FILEBUF_H
#define _FILEBUF_H

#include <sys/types.h>
#include <inttypes.h>
#include <unistd.h>
#include <string.h>


/*
 * bufStruct - the file buffer 
 *
 *   fp - file id 
 *   buffer - dynamically allocated buffer to buffer data from file
 *   buffer_size - the size of the allocated buffer
 *   chars_in_buffer - number of characters read into buffer
 *   next_char_in_buffer - index (in buffer) of the first character buffered
 *     but not read
 *
 */

typedef enum b_file_type { b_unix = 1, b_pvfs, b_pvfs_local } b_file_type;

typedef struct bufStruct {
  int fp;
  char *buffer;
  int buffer_size;
  int chars_in_buffer;
  int next_char_in_buffer;  
  int chars_left_to_read;  
  char eof;
  b_file_type b_type;
  int local_offset;
  int local_stripe_size;
  int local_jump;
  int node_base;
} bufStruct;
typedef bufStruct * bufStructPtr;

bufStructPtr open_buffered(const char *filename, int bufsize, b_file_type b_type);
int read_buffered(bufStructPtr bstruct, uint8_t * buf, int readsize);
int close_buffered(bufStructPtr bstruct);
off_t seek_buffered(bufStructPtr bstruct, off_t offset, int where);
long get_buffered_fileSTime();

#ifndef MIN
#define MIN(x,y) (x>y?y:x)
#endif
#endif /* _FILEBUF_H */

