/*
** Copyright (C) 2001-2005 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
** 
** Use of the SILK system and related source code is subject to the terms 
** of the following licenses:
** 
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
** 
** NO WARRANTY
** 
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
** DELIVERABLES UNDER THIS LICENSE.
** 
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
** Mellon University, its trustees, officers, employees, and agents from 
** all claims or demands made against them (and any related losses, 
** expenses, or attorney's fees) arising out of, or relating to Licensee's 
** and/or its sub licensees' negligent use or willful misuse of or 
** negligent conduct or willful misconduct regarding the Software, 
** facilities, or other rights or assistance granted by Carnegie Mellon 
** University under this License, including, but not limited to, any 
** claims of product liability, personal injury, death, damage to 
** property, or violation of any laws or regulations.
** 
** Carnegie Mellon University Software Engineering Institute authored 
** documents are sponsored by the U.S. Department of Defense under 
** Contract F19628-00-C-0003. Carnegie Mellon University retains 
** copyrights in all material produced under this contract. The U.S. 
** Government retains a non-exclusive, royalty-free license to publish or 
** reproduce these documents, or allow others to do so, for U.S. 
** Government purposes only pursuant to the copyright license under the 
** contract clause at 252.227.7013.
** 
** @OPENSOURCE_HEADER_END@
*/
#ifndef _RWPACK_H
#define _RWPACK_H

#include "silk.h"

RCSIDENTVAR(rcsID_RWPACK_H, "$SiLK: /src/include/rwpack.h,v 1.57 2005/09/16 21:54:40 thomasm Exp $");

#include <stdarg.h>

#include "utils.h"
#include "filebuf.h"
#include "v5pdu.h"
#include "skstringmap.h"


/* period definitions*/
#define PREVPREV 0
#define PREV  1
#define CUR   2
#define NEXT 3
/* num periods should be NEXT + 1 */
#define NUMPERIODS 4

#define MAX_PKTS 1048576        /* 2^20 */
#define PKTS_DIVISOR 64         /* gives a max of 67,100,864 pkts */
#define BPP_BITS        6
#define BPP_PRECN       64 /* 2^BPP_BITS */
#define BPP_PRECN_DIV_2       32 /* 2^BPP_BITS/2 */
/*
** max elapsed time is the offset from the record end time and
** with the 30 minute flush can be at most 30*60 = 1800
*/
#define MAX_ELAPSED_TIME        2048 /* 2*11 */
/*
** max end time is offset from the file max end time and can be at
** most 5*60 = 300
*/
#define MAX_START_TIME          4096 /* 2^12 */
#define MAX_IP_NUMS 2           /*the max # of ip's passed to the packer*/

/*
 *  Limit of SNMP interface index value
 */
#define SK_SNMP_INDEX_LIMIT   65536

/* legacy exit error code */
#define GENERAL_ERROR 1


#define  CONVERTTIME(r,t)                                               \
    (r).t = (rtr_unix_secs + ((r).t/1000)                               \
             + (((rtr_unix_msecs + ((r).t %1000)) >= 1000) ? 1 : 0));


#define SK_MAX_RECORD_SIZE 64
/*
 *    The maximum size of a packed record.  All of the rw*Rec_V* below
 *    should fit in uint8_t array of this size.
 */


#define SILK_INVALID_FLOW_TYPE  0xFF
/*
 *    Invalid flow_type value.  A flow-type is a class/type pair.
 */


typedef uint16_t sensorID_t;
/*
 *    Type to hold a sensor ID.  Usually, a sensor is a router or
 *    other flow collector.
 */


#define SENSOR_INVALID_SID ((sensorID_t)0xFFFF)
/*
 *    The value for an invalid or unrecognized sensor.
 */


#define SENSOR_MAX_SENSOR_NAME_LEN 24
/*
 *    The maximum length of a sensor name, not including the final
 *    NUL.
 */



typedef struct rwFileHeaderV0 {
    genericHeader   gHdr;
    uint32_t        fileSTime;
} rwFileHeaderV0;
typedef rwFileHeaderV0 *rwFileHeaderV0Ptr;
/*
 *    This is the header used in all "packed data" files which store
 *    time as an offset from the "fileSTime" value in the header.
 *    This includes the FT_RWROUTED, FT_RWNOTROUTED, FT_RWSPLIT, and
 *    FT_RWWWW types.
 */


typedef struct streamInfo {
    char *fName;
#ifdef  USE_FILDES
    int outFD;
#else
    FILE *outF;
#endif
    uint32_t count;
    uint32_t sTime;
} streamInfo;


/* FIXME. Ultimately, we'll want to split out the static type
 * definition information from the rwpack state, and move the static
 * configuration information to another file. */
typedef struct fileTypeInfo {
    char *className;
    char *dirPrefix;
    char *filePrefix;             /* file prefix */
    /* FIXME: We think the following field supersedes filePrefix */
    char *pathPrefix;
    int32_t filesPerDay;
    int32_t packType;             /* one of RW_XXXX from silk.h */
    streamInfo sInfo[NUMPERIODS];
    uint32_t maxETime;            /* utc end of the next file period */
} fileTypeInfo;

typedef struct engineData {
    uint32_t next_seq;
    uint32_t missingRecCount;
    uint32_t pktCount;
    uint32_t recCount;
    uint32_t badRecCount;
    uint32_t prev_sysUptime;
} engineData;
typedef engineData * engineDataPtr;

typedef struct routerData {
    engineDataPtr eData[65536];
    uint16_t version;
    /* sysUptime: from prev pdu or disk file if not-initialized */
    uint32_t pktCount;
    uint32_t outofSeqCount;
    uint32_t badPktCount;
    uint32_t badRecCount;
    uint32_t misRecCount;
    uint16_t numEngines;
} routerData;


/* struct to hold a single command line invocation; an array of these
 * is stored in the filter header */
typedef struct filterInfoV1 {
    /* Number of bytes in this information set */
    uint16_t byteCount;
    /* The command line */
    char *info;
} filterInfoV1;
typedef filterInfoV1 * filterInfoV1Ptr;

#define RWFILTER_HEADER_MAX_CMD_LINE_COUNT 256

/* struct to hold the filter header itself for FT_RWFILTER. */
typedef struct filterHeaderV1 {
    genericHeader   gHdr;
    /* Number of filters in this file */
    uint32_t        filterCount;
    /* Capacity of the following array */
    uint32_t        filterCapacity;
    /* array of filterInfo structs */
    filterInfoV1   *fiArray;
    /* length of all filter information structs */
    uint32_t        totFilterLen;
} filterHeaderV1;
typedef filterHeaderV1 * filterHeaderV1Ptr;



/*
**  This is the generic record returned from ANY packed type.
*/
typedef struct rwGenericRec_V3 {
    ipUnion     sIP;         /*  0- 3  Source IP      */
    ipUnion     dIP;         /*  4- 7  Destination IP */

    uint16_t    sPort;       /*  8- 9  Source port */
    uint16_t    dPort;       /* 10-11  Destination port */

    ipUnion     nhIP;        /* 12-15  Router Next Hop IP */
    uint16_t    input;       /* 16-17  Router incoming SNMP interface */
    uint16_t    output;      /* 18-19  Router outgoing SNMP interface */

    uint32_t    sTime;       /* 20-23  Start time of flow-epoch secs */
    uint32_t    elapsed;     /* 24-27  Duration of flow */
    uint32_t    pkts;        /* 28-31  Count of packets */
    uint32_t    bytes;       /* 32-35  Count of bytes */

    uint8_t     proto;       /* 36     IP protocol */
    uint8_t     flow_type;   /* 37     Class & Type info */
    sensorID_t  sID;         /* 38-39  Sensor ID */

    uint8_t     flags;       /* 40     OR of all flags (Netflow flags) */
    uint8_t     init_flags;  /* 41     TCP flags in first packet
                              *        or blank for "legacy" data */
    uint8_t     rest_flags;  /* 42     TCP flags on non-initial packet
                              *        or blank for "legacy" data */
    uint8_t     tcp_state;   /* 43     TCP state machine information (below)*/

    uint32_t    bpp;         /* 44-47  Bytes-per-Packet--eliminate roundoff */

    uint16_t    sTime_msec;  /* 48-49  Start time fraction (in millisec) */
    uint16_t    elapsed_msec;/* 50-51  Elapsed time fraction (in millisec) */
    uint16_t    application; /* 52-53  Indication of type of traffic */
} rwGenericRec_V3;

#define rwRec rwGenericRec_V3
typedef rwRec * rwRecPtr;


/*
**  Values for tcp_state value in rwGeneric and packed formats
*/

/* No additional TCP-state machine information is available */
#define SK_TCPSTATE_NO_INFO               0x00

/* Additional TCP-state machine information is available.  This must
 * be set for every TCP flow where the init_flags and rest_flags
 * fields are set.  */
#define SK_TCPSTATE_EXPANDED              0x01

/* Flow received packets following the FIN packet that were not ACK or
 * RST packets. */
#define SK_TCPSTATE_FIN_FOLLOWED_NOT_ACK  0x08

/* Flow ends prematurely due to a timeout by the collector. */
#define SK_TCPSTATE_TIMEOUT_KILLED        0x20

/* Flow is a continuation of a previous flow that was killed
 * prematurely due to a timeout by the collector. */
#define SK_TCPSTATE_TIMEOUT_STARTED       0x40


typedef enum {
    /* Used internally to mark stream as closed.  Currently,
     * re-opening a stream is not supported. */
    SK_RWIO_CLOSED = 0,

    /* Open an existing file for read. */
    SK_RWIO_READ = 1,

    /* Open a new file for output; file must not exist unless output
     * is to a named pipe. */
    SK_RWIO_WRITE = 2,

    /* Open an existing file for appending.  Must be a regular file
     * that is not compressed. */
    SK_RWIO_APPEND = 3

} rwio_mode_t;




/* struct for rw file io */
typedef struct rwIOStruct {
    /*
     * The header: genericHeader plus whatever is specific to the file
     * type. Responsibility of the opener to know what this is
     */
    void * hdr;
    /* Length of header */
    uint32_t hdrLen;
    /* Function to read a record */
    int (*rwReadFn)(struct rwIOStruct *, rwRecPtr);
    /* Function to write a record */
    int (*rwWriteFn)(struct rwIOStruct *, const rwRec *);
    /* Function to go forward 'skip_count' records */
    int (*rwSkipFn)(struct rwIOStruct *, int skip_count);
    /* Function to byte-swap a record in place */
    void (*rwSwapFn)(uint8_t *);
    /* Function to close the file associated with this structure */
    int (*closeFn)(FILE *);
    /* Function to write a record. */
    int (*writeBodyRecordsFn)(struct rwIOStruct *, const rwRec *);
    /* The open file */
    FILE *FD;
    /* Path to the file */
    char *fPath;
    /* The stream to copy the input to (support for --all-dest switch) */
    struct rwIOStruct *copyInputFD;
    /* The errno from the last system call that failed */
    int errnum;
    /* An object to hold the parameter that caused the last error */
    union {
        uint32_t     num;
        const char  *str;
        const rwRec *rec;
    } errobj;
    /* The fixed length of records of this type */
    uint16_t recLen;
    /* Sensor ID of the file.  Not used for FT_RWGENERIC or
     * FT_RWFILTER files. */
    sensorID_t sID;
    /* Whether stream is read, write, append. */
    rwio_mode_t io_mode;
    /* Whether the bytes need to be swapped on read/write */
    int8_t swapFlag;
    /* Have we reached the end of this file */
    int8_t eofFlag;
    /* The flow-type (class+type information) of the file as defined
     * in silk_site_*.h.  Not used for FT_RWGENERIC or FT_RWFILTER
     * files. */
    uint8_t flow_type;

    /* Flag to indicate pvfs filesystem in use, and the
     * file descriptor if pvfsFlag == 1. */
    int pvfsFlag;
    int bufFlag;
    int node;
    bufStructPtr bufFD;
} rwIOStruct;
typedef rwIOStruct * rwIOStructPtr;


/* function declarations for handling PDUs */
void flowPktTimeSetup(v5Header *hdr);
extern uint32_t rtr_unix_secs;
extern uint32_t rtr_unix_msecs;

/* function declarations from packutils-common.c */
int parseIPinOptions(const char *data, uint32_t *ipArray);
int parseIndexinOptions(char *data, uint8_t array[1], uint8_t n);





uint32_t rwGetFileOutputFormatCount(void);
/*
 *    Return the number of output file formats that are defined.
 *    Output formats are defined in silk_files.h and their identifiers
 *    begin with "FT_".
 */


const char *rwGetFileOutputFormatName(uint32_t file_output_format);
/*
 *    Given an output file format identifier, return a C-string
 *    representing that value.  Returns "UNKNOWN" for an unrecognized
 *    value.  For example, rwGetFileOutputFormatName(FT_RWFILTER)
 *    returns "FT_RWFILTER".
 *
 *    The caller should not modifier nor free the return value.
 */






/* rwreader functions */


/*
 *  rwIOSPtr = rwOpenFile(filename, copy_stream);
 *
 *    Open the given file 'filename' and based on the header
 *    information setup everything we need to read through the file
 *    always returning rwRec.
 *
 *    If 'copy_stream' is not-NULL, it should be an open rwIOStruct*
 *    stream to copy the input to.
 *
 *    Return a pointer to a new struct to contain all the information
 *    needed to traverse the file.  Return NULL on error.
 *
 *    If 'filename' is "stdin", then read binary input stream from
 *    stdin.
*/
rwIOStructPtr rwOpenFile(const char *fPath, rwIOStruct *copyInput);



/*
 * rwCloseFile:
 *      close everthing associated with this read struct
 * Input: rwIOStructPtr
 * Output: None
 *
*/
int rwCloseFile (rwIOStructPtr);



/* ****  sID  stuff  **** */

const char *sensorIdToName(sensorID_t id);
/*
 *    Return the name of the sensor, or "UNKN?" if the sensor id is
 *    out of range.  Return value is a character string constant.  It
 *    should not be changed nor free()'d.
 */


sensorID_t sensorNameToId(const char *name);
/*
 *    Return the id of the sensor with the specified name, or
 *    SENSOR_INVALID_SID if the name is not found.
 */


int sensorCheckCounts(void);
/*
 *    Return 0 if declared number of sensors matches the number of
 *    sensors defined; otherwise print a warning and return the number
 *    of sensors actually defined.
 */


int sensorNameIsLegal(const char *name);
/*
 *    Return 0 if the C-string 'name' is of proper form for a sensor
 *    name; i.e., 'name' contains no illegal characters and isn't too
 *    long or too short; return non-zero otherwise.  To see if 'name'
 *    represents an actual sensor, use sensorNameToId().
 */


/* macros for accessing header fields */
#define rwIsCompressed(a)       (((a)->closeFn == pclose) ? 1 : 0)
#define rwRead(a, b)            (a)->rwReadFn((a), (b))
#define rwWrite(a, b)           (a)->rwWriteFn((a), (b))
#define rwSkip(a, b)            (a)->rwSkipFn((a), (b))
#define rwGetIsBigEndian(a)     (((genericHeader *)(a)->hdr)->isBigEndian)
#define rwGetFileType(a)        (((genericHeader *)(a)->hdr)->type)
#define rwGetFileVersion(a)     (((genericHeader *)(a)->hdr)->version)
#define rwGetFileSTime(a)       (((rwFileHeaderV0 *)(a)->hdr)->fileSTime)
#define rwGetFILE(a)            ((a)->FD)
#define rwGetFileName(a)        ((a)->fPath)
#define rwGetHeader(a)          ((a)->hdr)
#define rwGetFileByteorder(a)   \
    (rwGetIsBigEndian(a) ? SILK_ENDIAN_BIG : SILK_ENDIAN_LITTLE)


size_t librwGetRecordSize(genericHeaderPtr gHdrPtr);
int librwComputeHeaderPadding(genericHeaderPtr gHdrPtr);

/* Number of records in a striped block on the clusters */
#define SK_STRIPE_REC_COUNT 4096

/* Striping sizes to use on the clusters */
#define librwGetStripeSize(gHdrPtr) \
    (SK_STRIPE_REC_COUNT * librwGetRecordSize(gHdrPtr))


/* ***  Printing rwRec as ASCII  *** */

/* Number of fields we can print; should be one more than the last ID
 * in rwrec_printable_fields_t */
#define  RWREC_PRINTABLE_FIELD_COUNT  24

/* Maximum width of the name of a field */
#define RWREC_PRINTABLE_MAX_NAME_LEN 16

/* An identifier for each field */
typedef enum {
    RWREC_FIELD_SIP,
    RWREC_FIELD_DIP,
    RWREC_FIELD_SPORT,
    RWREC_FIELD_DPORT,
    RWREC_FIELD_PROTO,
    RWREC_FIELD_PKTS,
    RWREC_FIELD_BYTES,
    RWREC_FIELD_FLAGS,
    RWREC_FIELD_STIME,
    RWREC_FIELD_ELAPSED,
    RWREC_FIELD_ETIME,
    RWREC_FIELD_SID,
    RWREC_FIELD_INPUT,
    RWREC_FIELD_OUTPUT,
    RWREC_FIELD_NHIP,
    RWREC_FIELD_INIT_FLAGS,
    RWREC_FIELD_REST_FLAGS,
    RWREC_FIELD_TCP_STATE,
    RWREC_FIELD_APPLICATION,
    RWREC_FIELD_FTYPE_CLASS,
    RWREC_FIELD_FTYPE_TYPE,
    RWREC_FIELD_STIME_MSEC,
    RWREC_FIELD_ETIME_MSEC,
    RWREC_FIELD_ELAPSED_MSEC
} rwrec_printable_fields_t;

/* A type for printing records as ASCII.  Created via
 * rwAsciiStreamCreate() */
typedef struct _rwAsciiStream rwAsciiStream_t;

extern int rwAsciiStreamCreate(rwAsciiStream_t **stream);
/*
 *  Create a new output stream for printing rwRec records in a human
 *  readable form. Store the newly allocated rwAsciiStream_t in the
 *  memory pointed to by **stream.  Return 0 on success or non-zero if
 *  allocation fails.
 *
 *  The caller may immediately call rwAsciiPrintRec() to print records
 *  to the stream, or the caller may use the functions listed below to
 *  change the defaults.  Calling an rwAsciiSet*() function once
 *  titles or records have been printed may result in strange output.
 *
 *  The defaults are (function that modifies this behavior):
 *  -- Output is sent to the standard output (rwAsciiSetOutputHandle)
 *  -- All fields are printed (rwAsciiSetFields)
 *  -- Column titles are printed before first record (rwAsciiSetNoTitles)
 *  -- Fields are printed in a columnar format (rwAsciiSetNoColumns)
 *  -- Delimiter between columns is '|' 
 *  -- A newline is printed after final column (rwAsciiSetNoNewline)
 *  -- IPs are printed in dotted quad (rwAsciiSetIntegerIps,
 *     rwAsciiSetZeroPadIps)
 *  -- Times are printed in human readable form (rwAsciiSetEpochTime)
 *  -- Sensor names are printed (rwAsciiSetIntegerSensors)
 *  -- No special handling of ICMP (rwAsciiSetIcmpTypeCode)
 */

extern void rwAsciiStreamDestroy(rwAsciiStream_t **stream);
/*
 *  Free all memory associated with the stream.  It is the caller's
 *  responsibility to fflush() the underlying file pointer.
 */

extern void rwAsciiPrintRec(
    rwAsciiStream_t    *stream,
    const rwRec        *rwrec);
/*
 *  Print 'rwrec' in a human-readable form to 'stream'.  Will print
 *  the column titles when the stream is configured to have titles and
 *  the titles have not yet been printed.
 */

extern void rwAsciiPrintTitles(rwAsciiStream_t *stream);
/*
 *  Print the column titles when the stream is configured to have
 *  titles and they have not been printed.
 */

extern void rwAsciiSetOutputHandle(rwAsciiStream_t *stream, FILE *fh);
/*
 *  Configure the 'stream' to print the output to 'fh'.  If 'fh' is
 *  NULL, stdout is used.
 */

extern int rwAsciiSetFields(
    rwAsciiStream_t    *stream,
    const uint32_t     *field_list,
    uint32_t            field_count);
/*
 *  Configure the 'stream' to print the fields listed in 'field_list',
 *  which is an array of 'field_count' elements.  If field_list is
 *  NULL or field_count is 0, returns non-zero.  Any previously
 *  configured field_list will be lost.
 */

extern void rwAsciiSetNoTitles(rwAsciiStream_t *stream);
/*
 *  Configure the 'stream' not to print titles before the first record
 *  of output.
 */

extern void rwAsciiSetDelimiter(rwAsciiStream_t *stream, char delimiter);
/*
 *  Configure the 'stream' to put the character 'delimiter' between
 *  columns.  Does not effect the columnar setting.
 */

extern void rwAsciiSetNoColumns(rwAsciiStream_t *stream);
/*
 *  Configure the 'stream' not to use fixed size columns.  This avoids
 *  extra whitespace, but makes the output difficult for humans to
 *  read.
 */

extern void rwAsciiSetNoNewline(rwAsciiStream_t *stream);
/*
 *  Configure the 'stream' not to print a newline after the final field.
 */

extern void rwAsciiSetIntegerIps(rwAsciiStream_t *stream);
/*
 *  Configure the 'stream' to print IP addresses as integers (0)
 *  instead of as dotted quad (0.0.0.0).  This function will override
 *  rwAsciiSetZeroPadIps().
 */

extern void rwAsciiSetZeroPadIps(rwAsciiStream_t *stream);
/*
 *  Configure the 'stream' to print each octet of an IP with extra
 *  0's; e.g., print "10.1.2.3" as "010.001.002.003"; useful for
 *  sorting with text-based sortes.  rwAsciiSetIntegerIps() will
 *  override this function.
 */

extern void rwAsciiSetEpochTime(rwAsciiStream_t *stream);
/*
 *  Configure the 'stream' to print times as seconds since the UNIX
 *  epoch.  Usually times are printed as YYYY/MM/DD:hh:mm:ss.
 */

extern void rwAsciiSetIntegerSensors(rwAsciiStream_t *stream);
/*
 *  Configure the 'stream' to print sensors as integers.  Usually the
 *  name of the sensor is printed.
 */

extern void rwAsciiSetIcmpTypeCode(rwAsciiStream_t *stream);
/*
 *  Configure the 'stream' to use a slightly different output for ICMP
 *  records (proto==1); when active and an ICMP record is given
 *  (proto==1), the sPort and dPort columns will hold the ICMP type
 *  and code, respectively.
 */

extern void rwAsciiGetFieldName(
    char                       *buf, 
    size_t                      buf_len,
    rwrec_printable_fields_t    field_id);
/*
 *  Put the first 'buf_len'-1 characters of the name of the
 *  field/column denoted by 'field_id' into the buffer 'buf'.  The
 *  caller should ensure that 'buf' is 'buf_len' characters long.
 *
 *  A 'buf' of at least RWREC_PRINTABLE_MAX_NAME_LEN characters will
 *  be large enough to hold the entire field name.
 */

extern int rwAsciiFlush(rwAsciiStream_t *stream);
/*
 *  Call flush() on the I/O object that 'stream' wraps.
 */

typedef enum {
    RW_ASCII_DUPES_KEEP = 0,
    RW_ASCII_DUPES_REMOVE_SILENT,
    RW_ASCII_DUPES_REMOVE_WARN,
    RW_ASCII_DUPES_ERROR
} rwascii_dupes_t;

void rwAsciiFieldMapPrintUsage(FILE *fh, sk_stringmap_t *field_map);

int rwAsciiFieldMapAddDefaultFields(sk_stringmap_t **field_map);

int rwAsciiFieldMapParseFields(
    uint32_t              **field_list,
    uint32_t               *field_count,
    const char             *input,
    const sk_stringmap_t   *field_map,
    rwascii_dupes_t         handle_dupes);

extern int rwAsciiParseFieldList(
    rwrec_printable_fields_t  **field_ids,
    uint32_t                   *field_count,
    const char                 *field_string);
/* NOT IMPLEMENTED FULLY */






int fdfilterWriteHeaderV1(rwIOStruct *rwIOS, int fildes);
int fdfilterReadHeaderV1(rwIOStruct *rwIOS, int fildes);



/* FIXME: PUT THESE SOMEWHERE */
typedef uint8_t fileVersion_t;

#define SK_FILE_VERSION_ANY ((fileVersion_t)255)


typedef enum {

    /* This indicates no error */
    LIBRW_OK = 0,


    /* The following set of errors affect only the current record;
     * they occur when trying to write a record to a stream. */

    /* The record's start time is less than the file's start time */
    LIBRW_ERR_STIME_UNDRFLO = 17,

    /* The record's start time at least an hour greater than the
     * file's start time */ 
    LIBRW_ERR_STIME_OVRFLO,

    /* The record's elapsed time is greater than space allocated for
     * duration in this file format */
    LIBRW_ERR_ELPSD_OVRFLO,

    /* The record contains more than the number of packets allowed in
     * this file format */
    LIBRW_ERR_PKTS_OVRFLO,

    /* The record contains a 0 value in the packets field. */
    LIBRW_ERR_PKTS_ZERO,

    /* The records contains an SNMP value too large to fit into the
     * space allocated in this file format. */
    LIBRW_ERR_SNMP_OVRFLO,

    /* The records contains a SensorID too large to fit into the space
     * allocated in this file format. */
    LIBRW_ERR_SENSORID_OVRFLO,

    /* The record's IP protocol is not supported by the file's format;
     * for example, trying to store a non-TCP record in a FT_RWWWW
     * file. */
    LIBRW_ERR_PROTO_MISMATCH,

    /* ERRORS < 32 are not fatal.
     * Use the LIBRW_ERROR_IS_FATAL() macro below. */


    /* The following set of errors can occur at any time and generally
     * indicate a fatal condition. */

    /* The call to write() or fwrite() failed */
    LIBRW_ERR_WRITE = 33,

    /* Caller passed an unexpected NULL value into a function; may
     * also be returned when function receives the empty string. */
    LIBRW_ERR_NULL_AUGUMENT,


    /* The following set of errors are general errors that occur when
     * opening a SiLK file for read, write, or append. */

    /* Cannot allocate memory; this should only occur during the
     * initial opening of a file and processing of its header. */
    LIBRW_ERR_ALLOC = 65,

    /* The file has a type that does not support this operation. */
    LIBRW_ERR_UNSUPPORT_TYPE,

    /* The file as a version that this version of librw does not know
     * how to handle */
    LIBRW_ERR_UNSUPPORT_VERSION,

    /* Caller is attempting to read from stdout, write to stdin,
     * append to a stream that is not a regular file, process binary
     * data on a terminal, or read from a temporary file. */
    LIBRW_ERR_UNSUPPORT_STREAM,

    /* The caller cannot change attributes on the rwIOStream;
     * typically this means the caller is attempting to change either
     * a stream opened for write to which the header has already been
     * written or a stream opened for read or append. */
    LIBRW_ERR_ATTRIBUTE_FIXED,

    /* Caller is attempting to call open on a previously opened
     * stream. */
    LIBRW_ERR_STREAM_OPEN,

    /* Caller is attempting to lock, read-from, or write-to an
     * unopened stream, i.e., a stream that has not yet been
     * opened. */
    LIBRW_ERR_STREAM_NOT_OPEN,

    /* Caller is attempting to operate on a stream that has been
     * closed.  Re-opening of streams is not supported. */
    LIBRW_ERR_STREAM_CLOSED,

    /* Call to open(), fopen(), or popen() failed */
    LIBRW_ERR_OPEN,

    /* Cannot open file descriptor as a file pointer */
    LIBRW_ERR_FDOPEN,

    /* Call to mkstemp() failed. */
    LIBRW_ERR_MKSTEMP,
    
    /* Unable to read data from a stream. */
    LIBRW_ERR_READ,


    /* The following set of errors occur when opening a file or stream
     * for writing. */

    /* Cannot create a file for writing because a file already exists
     * at that location */
    LIBRW_ERR_WRITE_FILE_EXISTS,

    /* Cannot create the directory path necessary to create the file
     * for writing */
    LIBRW_ERR_MKDIRPATH,

    /* The following set of errors occur when opening an existing a
     * file for appending. */

    /* File does not exist */
    LIBRW_ERR_OPEN_NOEXIST,

    /* Cannot seek to end of file */
    LIBRW_ERR_FSEEK,

    /* File is compressed---cannot append to a compressed file */
    LIBRW_ERR_UNSUPPORT_COMPRESS,

    /* File does not appear to be a SiLK file */
    LIBRW_ERR_BAD_MAGIC,

    /* Attempting to open a file as a type different from the one that
     * was used to write the file. */
    LIBRW_ERR_MISMATCH_TYPE,

    /* Attempting to open a file as a version different from the one
     * that was used to write the file. */
    LIBRW_ERR_MISMATCH_VERSION,

    /* Attempting to open a file in a byte-order different from the
     * one that was used to write the file. */
    LIBRW_ERR_MISMATCH_ENDIAN,

    /* Attempting to open a file with a start-time different from the
     * one that was used to write the file. */
    LIBRW_ERR_MISMATCH_STARTIME,


    /* The following set of errors occur when opening a file for
     * writing or for appending. */

    /* Cannot get an exclusive write lock on the file */
    LIBRW_ERR_WLOCK,

    /* Cannot get a shared read lock on the file */
    LIBRW_ERR_RLOCK

} librwErrorCodes_t;


/*
 *  LIBRW_ERROR_IS_FATAL(err);
 *
 *    Evaluates to a true value if error is a fatal error, false otherwise.
 */
#define LIBRW_ERROR_IS_FATAL(err) ((err) >= 32)



int rwioCreate(
    rwIOStruct    **rwIOS,
    const char     *pathname,
    rwio_mode_t     read_write_append);

int rwioSetFileType(
    rwIOStruct     *rwIOS,
    uint8_t         file_type);

int rwioSetFileVersion(
    rwIOStruct     *rwIOS,
    fileVersion_t   file_version);

int rwioSetFileByteorder(
    rwIOStruct     *rwIOS,
    silk_endian_t   byte_order);

int rwioSetErrorFunction(
    rwIOStruct     *rwIOS,
    sk_msg_fn_t     errFn);

int rwioSetFileSTime(
    rwIOStruct     *rwIOS,
    uint32_t        start_time,
    int             use_floor_hour);

int rwioAppendToHistoryFromFile(
    rwIOStruct         *rwIOS,
    const rwIOStruct   *source_rwIOS);

int rwioAppendToHistoryArgv(
    rwIOStruct         *rwIOS,
    int                 argc,
    char * const       *argv);

int rwioOpen(
    rwIOStruct     *rwIOS);

int rwioFDOpen(
    rwIOStruct     *rwIOS,
    int             file_desc);

int rwioMakeTemp(
    rwIOStruct     *rwIOS);

int rwioLockFile(
    rwIOStruct    *rwIOS);

int rwioMakeDirectory(
    rwIOStruct     *rwIOS);

int rwioWriteHeader(
    rwIOStruct     *rwIOS);

int rwioClose(
    rwIOStruct     *rwIOS);

int rwioUnbindFile(
    FILE          **out_filep,
    rwIOStruct     *rwIOS);

int rwioDestroy(
    rwIOStruct    **rwIOS);

void rwioPrintLastErr(
    rwIOStruct         *rwIOS,
    librwErrorCodes_t   error_code,
    sk_msg_fn_t         errfn);



#endif /* _RWPACK_H */
