/*
** Copyright (C) 2003-2004 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
**
** Use of the SILK system and related source code is subject to the terms
** of the following licenses:
**
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
**
** NO WARRANTY
**
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE,
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT,
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES.
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE
** DELIVERABLES UNDER THIS LICENSE.
**
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie
** Mellon University, its trustees, officers, employees, and agents from
** all claims or demands made against them (and any related losses,
** expenses, or attorney's fees) arising out of, or relating to Licensee's
** and/or its sub licensees' negligent use or willful misuse of or
** negligent conduct or willful misconduct regarding the Software,
** facilities, or other rights or assistance granted by Carnegie Mellon
** University under this License, including, but not limited to, any
** claims of product liability, personal injury, death, damage to
** property, or violation of any laws or regulations.
**
** Carnegie Mellon University Software Engineering Institute authored
** documents are sponsored by the U.S. Department of Defense under
** Contract F19628-00-C-0003. Carnegie Mellon University retains
** copyrights in all material produced under this contract. The U.S.
** Government retains a non-exclusive, royalty-free license to publish or
** reproduce these documents, or allow others to do so, for U.S.
** Government purposes only pursuant to the copyright license under the
** contract clause at 252.227.7013.
**
** @OPENSOURCE_HEADER_END@
*/

#ifndef _SILK_SITE_H
#error "Never include silk_site_generic.h directly,include silk_site.h instead"
#endif

RCSIDENTVAR(rcsID_SILK_SITE_GENERIC_H, "$SiLK: /src/include/silk_site_generic.h,v 1.20 2005/11/21 15:07:28 thomasm Exp $");


/* Indexes into the outFInfo array */
typedef enum {
    RW_IN,
    RW_OUT,
    RW_IN_WEB,
    RW_OUT_WEB,
    RW_IN_NULL,
    RW_OUT_NULL
} flowtype_t;
#define NUM_FLOW_TYPES 6


#ifdef DECLARE_SITE_VARIABLES

/*
**  The following macro is used to fill in the outFInfo[] array below.
**  This macro fills in the fileTypeInfo struct, defined in rwpack.h
**  and repeated here:
**
**  className, dirPrefix, filePrefix, pathPrefix, filesPerDay,
**  packType
**  array[4] of streamInfo each consisting of:
**      char *fName, FILE *outf, uint32_t count, uint32_t sTime;
**  maxETime: end of NEXT hour
*/
#define OUT_F_INFO_ENTRY(dirPrefix,filePrefix,packType)                 \
    {"all", (dirPrefix), (filePrefix), (dirPrefix "/"), 1, (packType),  \
     {{NULL,NULL,0,0},{NULL,NULL,0,0},{NULL,NULL,0,0},{NULL,NULL,0,0}}, 0}

fileTypeInfo outFInfo[NUM_FLOW_TYPES] = {
    /* dirPrefix, filePrefilx, packType */
    OUT_F_INFO_ENTRY("in",       "in",       FT_RWSPLIT),
    OUT_F_INFO_ENTRY("out",      "out",      FT_RWSPLIT),
    OUT_F_INFO_ENTRY("inweb",    "iw",       FT_RWWWW),
    OUT_F_INFO_ENTRY("outweb",   "ow",       FT_RWWWW),
    OUT_F_INFO_ENTRY("innull",   "innull",   FT_RWSPLIT),
    OUT_F_INFO_ENTRY("outnull",  "outnull",  FT_RWSPLIT),
};

/* compute number of file types */
uint32_t numFileTypes = sizeof(outFInfo)/sizeof(fileTypeInfo);
#endif /* DECLARE_SITE_VARIABLES */


/************** Classes */
#define NUM_CLASSES 1

#define DEFAULT_CLASS 0

#ifdef DECLARE_CLASSINFO_VARIABLES
/*
 * Types associated with classes
 *    {"className", NUM_TYPES (number from 1 to 12), NUM_DEFAULTS (1 to 4),
 *     {TYPE0,TYPE1,...,TYPE11} (types: array[12] of flowtype_t)
 *     {DEF_TYPE0,...,DEF_TYPE3} (default types: array[4] of flowtype_t)}
 */
static classInfoStruct classInfo[NUM_CLASSES] = {
    {"all", NUM_FLOW_TYPES, 2,
     {RW_IN, RW_OUT, RW_IN_WEB, RW_OUT_WEB, RW_IN_NULL, RW_OUT_NULL,
      0, 0, 0, 0, 0, 0},
     {RW_IN, RW_IN_WEB, 0, 0}}, /* 0 */
};
#endif /* DECLARE_CLASSINFO_VARIABLES */



/************ Sensors */

/*
 * To customize for a site change SENSOR_COUNT to reflect all sensors
 * used at the site, and add the appropriate entries to sensorInfo.
 *
 * NOTE: All sensor names should begin with a capital letter..  The
 * sensor name may be 1-SENSOR_MAX_SENSOR_NAME_LEN characters long
 * (that macro is defined in rwpask.h) and should only contain
 * alpha-numerics; it must not contain underscore.
 */

/*
 * THESE ARE SAMPLE VALUES ONLY.  CHANGE THE STRINGS IN THE sensorInfo
 * STRUCT TO MATCH WHAT WAS USED IN THE "SENSOR_NAME" FIELD FOR EACH
 * "rwfpd" YOU HAVE DEPLOYED.  CHANGE THE VALUE OF "SENSOR_COUNT" AS
 * WELL.
 */
#define SENSOR_COUNT    15
#ifdef DECLARE_SENSORID_VARIABLES
uint32_t numSensors=SENSOR_COUNT;

/*  sensorinfo_t is defined in silk_site.h
 *
 *      typedef struct {
 *          char *sensorName;
 *          uint8_t numClasses;
 *          uint8_t classNumbers[MAX_CLASSES_PER_SENSOR];
 *      } sensorinfo_t;
 *
 *  Since there is a single class, use this macro to set the sensor name.
 */
#define SENSOR_INFO_ENTRY(sname)                        \
    {(sname), 1, {DEFAULT_CLASS, 0, 0, 0, 0, 0, 0, 0} }

sensorinfo_t sensorInfo[SENSOR_COUNT+1] = {
    /*   0 */ SENSOR_INFO_ENTRY("S0"),
    /*   1 */ SENSOR_INFO_ENTRY("S1"),
    /*   2 */ SENSOR_INFO_ENTRY("S2"),
    /*   3 */ SENSOR_INFO_ENTRY("S3"),
    /*   4 */ SENSOR_INFO_ENTRY("S4"),
    /*   5 */ SENSOR_INFO_ENTRY("S5"),
    /*   6 */ SENSOR_INFO_ENTRY("S6"),
    /*   7 */ SENSOR_INFO_ENTRY("S7"),
    /*   8 */ SENSOR_INFO_ENTRY("S8"),
    /*   9 */ SENSOR_INFO_ENTRY("S9"),
    /*  10 */ SENSOR_INFO_ENTRY("S10"),
    /*  11 */ SENSOR_INFO_ENTRY("S11"),
    /*  12 */ SENSOR_INFO_ENTRY("S12"),
    /*  13 */ SENSOR_INFO_ENTRY("S13"),
    /*  14 */ SENSOR_INFO_ENTRY("S14"),

#if 0
    /*  15 */ SENSOR_INFO_ENTRY("S15"),
    /*  16 */ SENSOR_INFO_ENTRY("S16"),
    /*  17 */ SENSOR_INFO_ENTRY("S17"),
    /*  18 */ SENSOR_INFO_ENTRY("S18"),
    /*  19 */ SENSOR_INFO_ENTRY("S19"),
    /*  20 */ SENSOR_INFO_ENTRY("S20"),
    /*  21 */ SENSOR_INFO_ENTRY("S21"),
    /*  22 */ SENSOR_INFO_ENTRY("S22"),
    /*  23 */ SENSOR_INFO_ENTRY("S23"),
    /*  24 */ SENSOR_INFO_ENTRY("S24"),
    /*  25 */ SENSOR_INFO_ENTRY("S25"),
    /*  26 */ SENSOR_INFO_ENTRY("S26"),
    /*  27 */ SENSOR_INFO_ENTRY("S27"),
    /*  28 */ SENSOR_INFO_ENTRY("S28"),
    /*  29 */ SENSOR_INFO_ENTRY("S29"),
    /*  30 */ SENSOR_INFO_ENTRY("S30"),
    /*  31 */ SENSOR_INFO_ENTRY("S31"),
    /*  32 */ SENSOR_INFO_ENTRY("S32"),
    /*  33 */ SENSOR_INFO_ENTRY("S33"),
    /*  34 */ SENSOR_INFO_ENTRY("S34"),
    /*  35 */ SENSOR_INFO_ENTRY("S35"),
    /*  36 */ SENSOR_INFO_ENTRY("S36"),
    /*  37 */ SENSOR_INFO_ENTRY("S37"),
    /*  38 */ SENSOR_INFO_ENTRY("S38"),
    /*  39 */ SENSOR_INFO_ENTRY("S39"),
    /*  40 */ SENSOR_INFO_ENTRY("S40"),
    /*  41 */ SENSOR_INFO_ENTRY("S41"),
    /*  42 */ SENSOR_INFO_ENTRY("S42"),
    /*  43 */ SENSOR_INFO_ENTRY("S43"),
    /*  44 */ SENSOR_INFO_ENTRY("S44"),
    /*  45 */ SENSOR_INFO_ENTRY("S45"),
    /*  46 */ SENSOR_INFO_ENTRY("S46"),
    /*  47 */ SENSOR_INFO_ENTRY("S47"),
    /*  48 */ SENSOR_INFO_ENTRY("S48"),
    /*  49 */ SENSOR_INFO_ENTRY("S49"),
    /*  50 */ SENSOR_INFO_ENTRY("S50"),
    /*  51 */ SENSOR_INFO_ENTRY("S51"),
    /*  52 */ SENSOR_INFO_ENTRY("S52"),
    /*  53 */ SENSOR_INFO_ENTRY("S53"),
    /*  54 */ SENSOR_INFO_ENTRY("S54"),
    /*  55 */ SENSOR_INFO_ENTRY("S55"),
    /*  56 */ SENSOR_INFO_ENTRY("S56"),
    /*  57 */ SENSOR_INFO_ENTRY("S57"),
    /*  58 */ SENSOR_INFO_ENTRY("S58"),
    /*  59 */ SENSOR_INFO_ENTRY("S59"),
    /*  60 */ SENSOR_INFO_ENTRY("S60"),
    /*  61 */ SENSOR_INFO_ENTRY("S61"),
    /*  62 */ SENSOR_INFO_ENTRY("S62"),
    /*  63 */ SENSOR_INFO_ENTRY("S63"),
    /*  64 */ SENSOR_INFO_ENTRY("S64"),
    /*  65 */ SENSOR_INFO_ENTRY("S65"),
    /*  66 */ SENSOR_INFO_ENTRY("S66"),
    /*  67 */ SENSOR_INFO_ENTRY("S67"),
    /*  68 */ SENSOR_INFO_ENTRY("S68"),
    /*  69 */ SENSOR_INFO_ENTRY("S69"),
    /*  70 */ SENSOR_INFO_ENTRY("S70"),
    /*  71 */ SENSOR_INFO_ENTRY("S71"),
    /*  72 */ SENSOR_INFO_ENTRY("S72"),
    /*  73 */ SENSOR_INFO_ENTRY("S73"),
    /*  74 */ SENSOR_INFO_ENTRY("S74"),
    /*  75 */ SENSOR_INFO_ENTRY("S75"),
    /*  76 */ SENSOR_INFO_ENTRY("S76"),
    /*  77 */ SENSOR_INFO_ENTRY("S77"),
    /*  78 */ SENSOR_INFO_ENTRY("S78"),
    /*  79 */ SENSOR_INFO_ENTRY("S79"),
    /*  80 */ SENSOR_INFO_ENTRY("S80"),
    /*  81 */ SENSOR_INFO_ENTRY("S81"),
    /*  82 */ SENSOR_INFO_ENTRY("S82"),
    /*  83 */ SENSOR_INFO_ENTRY("S83"),
    /*  84 */ SENSOR_INFO_ENTRY("S84"),
    /*  85 */ SENSOR_INFO_ENTRY("S85"),
    /*  86 */ SENSOR_INFO_ENTRY("S86"),
    /*  87 */ SENSOR_INFO_ENTRY("S87"),
    /*  88 */ SENSOR_INFO_ENTRY("S88"),
    /*  89 */ SENSOR_INFO_ENTRY("S89"),
    /*  90 */ SENSOR_INFO_ENTRY("S90"),
    /*  91 */ SENSOR_INFO_ENTRY("S91"),
    /*  92 */ SENSOR_INFO_ENTRY("S92"),
    /*  93 */ SENSOR_INFO_ENTRY("S93"),
    /*  94 */ SENSOR_INFO_ENTRY("S94"),
    /*  95 */ SENSOR_INFO_ENTRY("S95"),
    /*  96 */ SENSOR_INFO_ENTRY("S96"),
    /*  97 */ SENSOR_INFO_ENTRY("S97"),
    /*  98 */ SENSOR_INFO_ENTRY("S98"),
    /*  99 */ SENSOR_INFO_ENTRY("S99"),
    /* 100 */ SENSOR_INFO_ENTRY("S100"),
    /* 101 */ SENSOR_INFO_ENTRY("S101"),
    /* 102 */ SENSOR_INFO_ENTRY("S102"),
    /* 103 */ SENSOR_INFO_ENTRY("S103"),
    /* 104 */ SENSOR_INFO_ENTRY("S104"),
    /* 105 */ SENSOR_INFO_ENTRY("S105"),
    /* 106 */ SENSOR_INFO_ENTRY("S106"),
    /* 107 */ SENSOR_INFO_ENTRY("S107"),
    /* 108 */ SENSOR_INFO_ENTRY("S108"),
    /* 109 */ SENSOR_INFO_ENTRY("S109"),
    /* 110 */ SENSOR_INFO_ENTRY("S110"),
    /* 111 */ SENSOR_INFO_ENTRY("S111"),
    /* 112 */ SENSOR_INFO_ENTRY("S112"),
    /* 113 */ SENSOR_INFO_ENTRY("S113"),
    /* 114 */ SENSOR_INFO_ENTRY("S114"),
    /* 115 */ SENSOR_INFO_ENTRY("S115"),
    /* 116 */ SENSOR_INFO_ENTRY("S116"),
    /* 117 */ SENSOR_INFO_ENTRY("S117"),
    /* 118 */ SENSOR_INFO_ENTRY("S118"),
    /* 119 */ SENSOR_INFO_ENTRY("S119"),
    /* 120 */ SENSOR_INFO_ENTRY("S120"),
    /* 121 */ SENSOR_INFO_ENTRY("S121"),
    /* 122 */ SENSOR_INFO_ENTRY("S122"),
    /* 123 */ SENSOR_INFO_ENTRY("S123"),
    /* 124 */ SENSOR_INFO_ENTRY("S124"),
    /* 125 */ SENSOR_INFO_ENTRY("S125"),
    /* 126 */ SENSOR_INFO_ENTRY("S126"),
    /* 127 */ SENSOR_INFO_ENTRY("S127"),
    /* 128 */ SENSOR_INFO_ENTRY("S128"),
    /* 129 */ SENSOR_INFO_ENTRY("S129"),
    /* 130 */ SENSOR_INFO_ENTRY("S130"),
    /* 131 */ SENSOR_INFO_ENTRY("S131"),
    /* 132 */ SENSOR_INFO_ENTRY("S132"),
    /* 133 */ SENSOR_INFO_ENTRY("S133"),
    /* 134 */ SENSOR_INFO_ENTRY("S134"),
    /* 135 */ SENSOR_INFO_ENTRY("S135"),
    /* 136 */ SENSOR_INFO_ENTRY("S136"),
    /* 137 */ SENSOR_INFO_ENTRY("S137"),
    /* 138 */ SENSOR_INFO_ENTRY("S138"),
    /* 139 */ SENSOR_INFO_ENTRY("S139"),
    /* 140 */ SENSOR_INFO_ENTRY("S140"),
    /* 141 */ SENSOR_INFO_ENTRY("S141"),
    /* 142 */ SENSOR_INFO_ENTRY("S142"),
    /* 143 */ SENSOR_INFO_ENTRY("S143"),
    /* 144 */ SENSOR_INFO_ENTRY("S144"),
    /* 145 */ SENSOR_INFO_ENTRY("S145"),
    /* 146 */ SENSOR_INFO_ENTRY("S146"),
    /* 147 */ SENSOR_INFO_ENTRY("S147"),
    /* 148 */ SENSOR_INFO_ENTRY("S148"),
    /* 149 */ SENSOR_INFO_ENTRY("S149"),
    /* 150 */ SENSOR_INFO_ENTRY("S150"),
    /* 151 */ SENSOR_INFO_ENTRY("S151"),
    /* 152 */ SENSOR_INFO_ENTRY("S152"),
    /* 153 */ SENSOR_INFO_ENTRY("S153"),
    /* 154 */ SENSOR_INFO_ENTRY("S154"),
    /* 155 */ SENSOR_INFO_ENTRY("S155"),
    /* 156 */ SENSOR_INFO_ENTRY("S156"),
    /* 157 */ SENSOR_INFO_ENTRY("S157"),
    /* 158 */ SENSOR_INFO_ENTRY("S158"),
    /* 159 */ SENSOR_INFO_ENTRY("S159"),
    /* 160 */ SENSOR_INFO_ENTRY("S160"),
    /* 161 */ SENSOR_INFO_ENTRY("S161"),
    /* 162 */ SENSOR_INFO_ENTRY("S162"),
    /* 163 */ SENSOR_INFO_ENTRY("S163"),
    /* 164 */ SENSOR_INFO_ENTRY("S164"),
    /* 165 */ SENSOR_INFO_ENTRY("S165"),
    /* 166 */ SENSOR_INFO_ENTRY("S166"),
    /* 167 */ SENSOR_INFO_ENTRY("S167"),
    /* 168 */ SENSOR_INFO_ENTRY("S168"),
    /* 169 */ SENSOR_INFO_ENTRY("S169"),
    /* 170 */ SENSOR_INFO_ENTRY("S170"),
    /* 171 */ SENSOR_INFO_ENTRY("S171"),
    /* 172 */ SENSOR_INFO_ENTRY("S172"),
    /* 173 */ SENSOR_INFO_ENTRY("S173"),
    /* 174 */ SENSOR_INFO_ENTRY("S174"),
    /* 175 */ SENSOR_INFO_ENTRY("S175"),
    /* 176 */ SENSOR_INFO_ENTRY("S176"),
    /* 177 */ SENSOR_INFO_ENTRY("S177"),
    /* 178 */ SENSOR_INFO_ENTRY("S178"),
    /* 179 */ SENSOR_INFO_ENTRY("S179"),
    /* 180 */ SENSOR_INFO_ENTRY("S180"),
    /* 181 */ SENSOR_INFO_ENTRY("S181"),
    /* 182 */ SENSOR_INFO_ENTRY("S182"),
    /* 183 */ SENSOR_INFO_ENTRY("S183"),
    /* 184 */ SENSOR_INFO_ENTRY("S184"),
    /* 185 */ SENSOR_INFO_ENTRY("S185"),
    /* 186 */ SENSOR_INFO_ENTRY("S186"),
    /* 187 */ SENSOR_INFO_ENTRY("S187"),
    /* 188 */ SENSOR_INFO_ENTRY("S188"),
    /* 189 */ SENSOR_INFO_ENTRY("S189"),
    /* 190 */ SENSOR_INFO_ENTRY("S190"),
    /* 191 */ SENSOR_INFO_ENTRY("S191"),
    /* 192 */ SENSOR_INFO_ENTRY("S192"),
    /* 193 */ SENSOR_INFO_ENTRY("S193"),
    /* 194 */ SENSOR_INFO_ENTRY("S194"),
    /* 195 */ SENSOR_INFO_ENTRY("S195"),
    /* 196 */ SENSOR_INFO_ENTRY("S196"),
    /* 197 */ SENSOR_INFO_ENTRY("S197"),
    /* 198 */ SENSOR_INFO_ENTRY("S198"),
    /* 199 */ SENSOR_INFO_ENTRY("S199"),
    /* 200 */ SENSOR_INFO_ENTRY("S200"),
    /* 201 */ SENSOR_INFO_ENTRY("S201"),
    /* 202 */ SENSOR_INFO_ENTRY("S202"),
    /* 203 */ SENSOR_INFO_ENTRY("S203"),
    /* 204 */ SENSOR_INFO_ENTRY("S204"),
    /* 205 */ SENSOR_INFO_ENTRY("S205"),
    /* 206 */ SENSOR_INFO_ENTRY("S206"),
    /* 207 */ SENSOR_INFO_ENTRY("S207"),
    /* 208 */ SENSOR_INFO_ENTRY("S208"),
    /* 209 */ SENSOR_INFO_ENTRY("S209"),
    /* 210 */ SENSOR_INFO_ENTRY("S210"),
    /* 211 */ SENSOR_INFO_ENTRY("S211"),
    /* 212 */ SENSOR_INFO_ENTRY("S212"),
    /* 213 */ SENSOR_INFO_ENTRY("S213"),
    /* 214 */ SENSOR_INFO_ENTRY("S214"),
    /* 215 */ SENSOR_INFO_ENTRY("S215"),
    /* 216 */ SENSOR_INFO_ENTRY("S216"),
    /* 217 */ SENSOR_INFO_ENTRY("S217"),
    /* 218 */ SENSOR_INFO_ENTRY("S218"),
    /* 219 */ SENSOR_INFO_ENTRY("S219"),
    /* 220 */ SENSOR_INFO_ENTRY("S220"),
    /* 221 */ SENSOR_INFO_ENTRY("S221"),
    /* 222 */ SENSOR_INFO_ENTRY("S222"),
    /* 223 */ SENSOR_INFO_ENTRY("S223"),
    /* 224 */ SENSOR_INFO_ENTRY("S224"),
    /* 225 */ SENSOR_INFO_ENTRY("S225"),
    /* 226 */ SENSOR_INFO_ENTRY("S226"),
    /* 227 */ SENSOR_INFO_ENTRY("S227"),
    /* 228 */ SENSOR_INFO_ENTRY("S228"),
    /* 229 */ SENSOR_INFO_ENTRY("S229"),
    /* 230 */ SENSOR_INFO_ENTRY("S230"),
    /* 231 */ SENSOR_INFO_ENTRY("S231"),
    /* 232 */ SENSOR_INFO_ENTRY("S232"),
    /* 233 */ SENSOR_INFO_ENTRY("S233"),
    /* 234 */ SENSOR_INFO_ENTRY("S234"),
    /* 235 */ SENSOR_INFO_ENTRY("S235"),
    /* 236 */ SENSOR_INFO_ENTRY("S236"),
    /* 237 */ SENSOR_INFO_ENTRY("S237"),
    /* 238 */ SENSOR_INFO_ENTRY("S238"),
    /* 239 */ SENSOR_INFO_ENTRY("S239"),
    /* 240 */ SENSOR_INFO_ENTRY("S240"),
    /* 241 */ SENSOR_INFO_ENTRY("S241"),
    /* 242 */ SENSOR_INFO_ENTRY("S242"),
    /* 243 */ SENSOR_INFO_ENTRY("S243"),
    /* 244 */ SENSOR_INFO_ENTRY("S244"),
    /* 245 */ SENSOR_INFO_ENTRY("S245"),
    /* 246 */ SENSOR_INFO_ENTRY("S246"),
    /* 247 */ SENSOR_INFO_ENTRY("S247"),
    /* 248 */ SENSOR_INFO_ENTRY("S248"),
    /* 249 */ SENSOR_INFO_ENTRY("S249"),
    /* 250 */ SENSOR_INFO_ENTRY("S250"),
    /* 251 */ SENSOR_INFO_ENTRY("S251"),
    /* 252 */ SENSOR_INFO_ENTRY("S252"),
    /* 253 */ SENSOR_INFO_ENTRY("S253"),
    /* 254 */ SENSOR_INFO_ENTRY("S254"),
#endif /* 0 */    

    /*sentinel*/ SENSOR_INFO_SENTINEL
};

#endif /* DECLARE_SENSOR_VARIABLES */
