/*
** Copyright (C) 2004-2005 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
**
** Use of the SILK system and related source code is subject to the terms
** of the following licenses:
**
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
**
** NO WARRANTY
**
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE,
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT,
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES.
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE
** DELIVERABLES UNDER THIS LICENSE.
**
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie
** Mellon University, its trustees, officers, employees, and agents from
** all claims or demands made against them (and any related losses,
** expenses, or attorney's fees) arising out of, or relating to Licensee's
** and/or its sub licensees' negligent use or willful misuse of or
** negligent conduct or willful misconduct regarding the Software,
** facilities, or other rights or assistance granted by Carnegie Mellon
** University under this License, including, but not limited to, any
** claims of product liability, personal injury, death, damage to
** property, or violation of any laws or regulations.
**
** Carnegie Mellon University Software Engineering Institute authored
** documents are sponsored by the U.S. Department of Defense under
** Contract F19628-00-C-0003. Carnegie Mellon University retains
** copyrights in all material produced under this contract. The U.S.
** Government retains a non-exclusive, royalty-free license to publish or
** reproduce these documents, or allow others to do so, for U.S.
** Government purposes only pursuant to the copyright license under the
** contract clause at 252.227.7013.
**
** @OPENSOURCE_HEADER_END@
*/
#ifndef _LIBRW_PRIV_H
#define _LIBRW_PRIV_H

#include "silk.h"

RCSIDENTVAR(rcsID_LIBRW_PRIV_H, "$SiLK: /src/rw/librw/librw_priv.h,v 1.18 2005/10/13 14:44:28 thomasm Exp $");

/*
**  librw_priv.h
**
**  For sharing of functions within librw (to avoid tons of extern's).
**
**  THESE FUNCTIONS ARE FOR INTERNAL USE BY LIBRW.
**
*/


#include "utils.h"
#include "rwpack.h"
#include "bits.h"
#include "filebuf.h"
#include "profile.h"


/* macros to swap the bytes in place */
#define SWAP_DATA32(d) *((uint32_t*)(d)) = BSWAP32(*(uint32_t*)(d))
#define SWAP_DATA16(d) *((uint16_t*)(d)) = BSWAP16(*(uint16_t*)(d))


/* macros to get sTime, eTime, and elapsed as seconds */
#define RWREC_GET_STIME_SEC(rwRP) \
    ((rwRP)->sTime + (((rwRP)->sTime_msec >= 500) ? 1 : 0))

#define RWREC_GET_ELAPSED_SEC(rwRP) \
    ((rwRP)->elapsed + (((rwRP)->elapsed_msec >= 500) ? 1 : 0))

/* #define RWREC_GET_ETIME_SEC(rwRP) \
 *    ((rwRP)->sTime + (rwRP)->elapsed + (((rwRP)->elapsed_msec >= 500) ? 1 : 0))
 */


/* Web classification utilities */

/* FIXME -- do these really live here or somewhere more public? */

/*
 *  isweb = WEBP(p)
 *      Return a true value if port 'p' is a "web" port; false otherwise
 */
#define WEBP(p) ((p) == 80 || (p) == 443 || (p) == 8080)

/*
 *  encoding = WEBPORT(p)
 *
 *      Encode the port 'p' into a value suitable for storing in the
 *      wPort field of an FT_RWWWW record.
 */
#define WEBPORT(p)                              \
    (((p) == 80)                                \
     ? 0                                        \
     : (((p) == 443)                            \
        ? 1                                     \
        : (((p) == 8080)                        \
           ? 2                                  \
           : 3)))

/*
 *  decoding = EXPAND_WEBPORT(p)
 *
 *    Decode the port 'p' from the value stored in the wPort field in
 *    an FT_RWWWW record.
 */
#define EXPAND_WEBPORT(p)                               \
    (((p) == 0)                                         \
     ? 80                                               \
     : (((p) == 1)                                      \
        ? 443                                           \
        : (((p) == 2)                                   \
           ? 8080                                       \
           : 0)))

/*
 *  isweb = IS_WEB_PORT(p)
 *
 *      Alias for WEBP
 */
#define IS_WEB_PORT(p) WEBP(p)

/*
 *  isweb = IS_WEB(r)
 *
 *      Return a true value if the flow 'r' represents a web flow;
 *      return false otherwise.  'r' is an rwrec, NOT an rwrec*.
 */
#define IS_WEB(r)                               \
    (((r).prot == 6) && (IS_WEB_PORT((r).srcport) || IS_WEB_PORT((r).dstport)))


/*
**  status = _rwAugmentedPrepareRead(rwIOS)
**  status = _rwAugroutingPrepareRead(rwIOS)
**  status = _rwAugwebPrepareRead(rwIOS)
**  status = _rwFilterPrepareRead(rwIOS)
**  status = _rwGenericPrepareRead(rwIOS)
**  status = _rwNotroutedPrepareRead(rwIOS)
**  status = _rwRoutedPrepareRead(rwIOS)
**  status = _rwSplitPrepareRead(rwIOS)
**  status = _rwWwwPrepareRead(rwIOS)
**
**  DO NOT CALL DIRECTLY.  FOR INTERNAL LIBRW USE
**
**  One of these functions should be called after a file has been
**  opened for reading or appending and the genericHeader has been
**  read.  The function does whatever is required to prepare the file
**  for reading records of the specified type: it checks that the
**  version is valid for this type of file, sets record and header
**  lengths, it sets reading and writing function pointers
**  appropriately for the file's type and version, it reads the
**  remainder of the header, and it prints the entire header to the
**  copyInputFD if required.
**
**  Returns LIBRW_OK on success; otherwise prints an error and returns
**  error code on failure: bad version, unable to read header.
*/
int _augmentedioPrepareRead(rwIOStruct *rwIOS);
int _augroutingioPrepareRead(rwIOStruct *rwIOS);
int _augwebioPrepareRead(rwIOStruct *rwIOS);
int _filterioPrepareRead(rwIOStruct *rwIOS);
int _genericioPrepareRead(rwIOStruct *rwIOS);
int _notroutedioPrepareRead(rwIOStruct *rwIOS);
int _routedioPrepareRead(rwIOStruct *rwIOS);
int _splitioPrepareRead(rwIOStruct *rwIOS);
int _wwwioPrepareRead(rwIOStruct *rwIOS);

int _augmentedioPrepareWrite(rwIOStruct *rwIOS);
int _augroutingioPrepareWrite(rwIOStruct *rwIOS);
int _augwebioPrepareWrite(rwIOStruct *rwIOS);
int _filterioPrepareWrite(rwIOStruct *rwIOS);
int _genericioPrepareWrite(rwIOStruct *rwIOS);
int _notroutedioPrepareWrite(rwIOStruct *rwIOS);
int _routedioPrepareWrite(rwIOStruct *rwIOS);
int _splitioPrepareWrite(rwIOStruct *rwIOS);
int _wwwioPrepareWrite(rwIOStruct *rwIOS);


#if 0
/*
**  status = _rwGenericPrepareWrite(rwIOS);
**
**  status = _rwNotroutedPrepareWrite(rwIOS, start_time)
**  status = _rwRoutedPrepareWrite(rwIOS, start_time)
**  status = _rwSplitPrepareWrite(rwIOS, start_time)
**  status = _rwWwwPrepareWrite(rwIOS, start_time)
**
**  DO NOT CALL DIRECTLY.  FOR INTERNAL LIBRW USE
**
**  One of these functions should be called after a file has been
**  opened for writing.  The function does whatever is required to
**  prepare the file for writing records of the specified type: it
**  checks that the version is valid for this type of file, sets
**  record and header lengths, it sets reading and writing function
**  pointers appropriately for the file's type and version, and it
**  writes the entire header to the stream.
**
**  For the functions that take a start_time, that is the epoch time
**  to write into the file's header.
**
**  Returns LIBRW_OK on success; otherwise prints an error and returns
**  error code on failure: bad version, unable to read header.
*/
int _rwGenericPrepareWrite(rwIOStruct **rwIOS);
int _rwNotroutedPrepareWrite(rwIOStruct **rwIOS, uint32_t start_time);
int _rwRoutedPrepareWrite(rwIOStruct **rwIOS, uint32_t start_time);
int _rwSplitPrepareWrite(rwIOStruct **rwIOS, uint32_t start_time);
int _rwWwwPrepareWrite(rwIOStruct **rwIOS, uint32_t start_time);
#endif /* 0 */


int _readFilenameToFlowType(const char *filename);
/*
 *    Return the flowtype_t of the file based on the the prefix of
 *    'filename'.  "Flow type" represents the class and type of the
 *    file.  Returns SILK_INVALID_FLOW_TYPE if the file does not have
 *    a prefix or if the prefix is not recognized.
 */


uint8_t _readFilenameToFileFormat(const char *filename);


/* *** rwheaders.c *** */

/*
**  handled in rwpack.h
**int filterWriteHeaderV1(filterHeaderV1Ptr FHPtr, FILE *outF);
**int filterAddFInfoToHeaderV1(filterHeaderV1Ptr FHPtr, int argc, char **argv);
**void filterPrintHeaderV1(filterHeaderV1Ptr fHPtr, FILE *outF);
**int filterReadHeaderV1(filterHeaderV1Ptr FHPtr, FILE *inF, int swapFlag);
*/
//int _librwWriteFilterHeaderV1(rwIOStruct **rwIOS);


int _headersAppendArgv(rwIOStruct *rwIOS, int argc, char * const *argv);
int _headersAppendFromFile(rwIOStruct *rwIOs, const rwIOStruct *rwIOS_in);
int _headersCreateV1(rwIOStruct *rwIOS);
int _headersReadV1(rwIOStruct *rwIOS, int fildes);
int _headersWriteV1(rwIOStruct *rwIOS, int fildes);


/* *** rwread.c *** */

extern struct profile prof1;


/*
** _errorReturn:
**      an internal utility function to cleanup the malloc'd
**      stuff from a rwRdr and return a NULL.
** Input: rwIOStructPtr
** Output: (rwIOStructPtr) NULL
*/
rwIOStructPtr _errorReturn(rwIOStructPtr rwIOSPtr);


/*
 *  _readReadRecord:
 *      Reads a single record into buf.
 *  Input:
 *      void *buf: the buffer to read into (sizeof(rwIOSPtr->recLen))
 *      rwIOStructPtr rwIOSPtr: pointer to reader struct
 *  Output:
 *      0/1 sucess/failure
 *  Side Effects:
 *      buf is populated
 *      rwIOSPtr->eofFlag can be set
 */
int _readReadRecord(void *buf, rwIOStructPtr rwIOSPtr);


int _packedfileheaderCreateV0(rwIOStruct *rwIOS);
int _packedfileheaderReadV0(rwIOStruct *rwIOS);
int _packedfileheaderWrite(rwIOStruct *rwIOS);
int _packedfileheaderWriteInit(
    rwIOStruct             *rwIOS,
    const rwGenericRec_V3  *rwRP);
int _packedfileheaderSetSTime(
    rwIOStruct     *rwIOS,
    uint32_t        start_time,
    int             round_to_hour);



void _readUnpackBytesPackets(
    rwGenericRec_V3    *rwRP,
    uint32_t            bpp,
    uint32_t            pkts,
    uint32_t            pflag);
/*
 *    Fills in 'bytes', 'packets', and 'bpp' fields of the rwRec
 *    pointed to by 'rwrec'.
 *
 *    The parameters 'bpp', 'pkts', and 'pflag' should be the values
 *    as stored in the packed file format; i.e., they should be the
 *    values that _writePackBytesPackets() created; they are expected
 *    to be in native byte order.
 *
 *    Specifically, 'pkts' is either the packet count or the packet
 *    count divided by the PKTS_DIVISOR if 'pflag' is non-zero.  'bpp'
 *    is the bytes-per-packet ratio given by a 14 bit value and a 6
 *    bit fractional part.
 *
 *    This function does no error checking.
 */


void _readUnpackSbbPef(
    rwGenericRec_V3    *rwRP,
    uint32_t            file_start_time,
    const uint32_t     *sbb,
    const uint32_t     *pef);
/*
 *    Unpacks the 'pef' and 'sbb' fields from V1 and V2 of the SPLIT,
 *    WWW, ROUTED, and NOTROUTED packed file formats and computes the
 *    elapsed, pkts, bytes, and sTime fields in the rwGeneric record
 *    pointer rwRP.
 *
 *    rwIOS is an rwIOStruct* to the file containing the
 *    record--required for file's start time.  sbb and pef are in
 *    native byte order.
 */


void _readUnpackTimeBytesPktsFlags(
    rwGenericRec_V3    *rwRP,
    uint32_t            file_start_time,
    const uint32_t     *pkts_stime,
    const uint32_t     *bbe,
    const uint32_t     *msec_flags);
/*
 *    Unpacks the 'pkts_stime', 'bbe', and 'msec_flags' fields from V3
 *    of the SPLIT, WWW, ROUTED, and NOTROUTED packed file formats and
 *    computes the elapsed, pkts, bytes, and sTime fields in the
 *    rwGeneric record pointed to by rwRP.
 *
 *    rwIOS is an rwIOStruct* to the file containing the
 *    record--required for file's start time.  sbb and pef are in
 *    native byte order.
 */


void _readUnpackProtoFlags(
    rwGenericRec_V3    *rwRP,
    uint8_t             is_tcp,
    uint8_t             prot_flags,
    uint8_t             tcp_state,
    uint8_t             rest_flags);


/*
**  publicly defined in rwpack.h
**
**void rwCloseFile(rwIOStructPtr rwIOSPtr);
**rwIOStructPtr rwOpenFile(const char *fPath, FILE *copyInputFD);
**void rwPrintHeader(rwIOStructPtr rwIOSPtr, FILE *fp);
**size_t librwGetRecordSize(genericHeaderPtr gHdrPtr);
**int librwComputeHeaderPadding(genericHeaderPtr gHdrPtr);
*/




/* *** rwsid.c *** */

/*
** getSIDFromFName:
**
**      Get the sensor-name portion of the file name and convert to an
**      id between 0..SENSOR_INVALID_SID-1.  The macro
**      SENSOR_INVALID_SID is returned if the sensor name cannot be
**      parsed from the file name.
**
** Inputs:
**      char *fPath; int stdinFlag (if 1 do not print error message)
**
*/
sensorID_t getSIDFromFName(const char *fPath, int stdinFlag);
/*
**  publicly defined in rwpack.h
**
**const char *sensorIdToName(sensorID_t id);
**sensorID_t sensorNameToId(const char *name);
*/


/*
**  length = _augmentedioGetRecLen(version);
**  length = _augroutingioGetRecLen(version);
**  length = _filterioGetRecLen(version);
**  length = _genericioGetRecLen(version);
**  length = _notroutedioGetRecLen(version);
**  length = _routedioGetRecLen(version);
**  length = _splitioGetRecLen(version);
**  length = _wwwioGetRecLen(version);
**
**    Return the on-disk length in bytes of records of the specified
**    type and vresion; or return 0 if the specified version is not
**    defined for the given type.
*/
size_t _augmentedioGetRecLen(fileVersion_t);
size_t _augroutingioGetRecLen(fileVersion_t);
size_t _augwebioGetRecLen(fileVersion_t);
size_t _filterioGetRecLen(fileVersion_t);
size_t _genericioGetRecLen(fileVersion_t);
size_t _notroutedioGetRecLen(fileVersion_t);
size_t _routedioGetRecLen(fileVersion_t);
size_t _splitioGetRecLen(fileVersion_t);
size_t _wwwioGetRecLen(fileVersion_t);


/* *** rwwrite.c *** */

int _writePackBytesPackets(
    uint32_t               *bpp_out,
    uint32_t               *pkts_out,
    uint32_t               *pflag_out,
    const rwGenericRec_V3  *rwRP);
/*
 *    Uses fields from the rwRec pointed to by 'rwrec' to compute the
 *    bytes-per-packet ('bpp'), packets ('pkts'), and
 *    packets-multiplier ('pflag') field required by the packed file
 *    formats FILTER, SPLIT, WWW, ROUTED, and NOTROUTED.
 *
 *    The parameters 'bpp', 'pkts', and 'pflag' will be the values to
 *    store in the packed file format; i.e., they will be the values
 *    that _readUnpackBytesPackets() can read; they will be in native
 *    byte order.
 *
 *    Specifically, 'pkts' is either the packet count or the packet
 *    count divided by the PKTS_DIVISOR when 'pflag' is non-zero.
 *    'bpp' is the bytes-per-packet ratio given by a 14 bit value and
 *    a 6 bit fractional part.
 *
 *    This function returns LIBRW_OK on success, or the following to
 *    indicate an error: LIBRW_ERR_PKTS_ZERO-the 'pkts' field on rwrec
 *    is 0; LIBRW_ERR_PKTS_OVRFLO-the 'pkts' value is too large to
 *    store in the packed file format.
 */


void _writePackProtoFlags(
    uint8_t                *is_tcp_out,
    uint8_t                *prot_flags_out,
    uint8_t                *tcp_state_out,
    uint8_t                *rest_flags_out,
    const rwGenericRec_V3  *rwRP);
/*
 *    Uses fields from the rwRec pointed to by 'rwrec' to compute the
 *    values pointed to by these variables:
 *
 *        is_tcp_out - 1 if the flow is TCP (proto==6); 0 otherwise
 *
 *        prot_flags_out - protocol when is_tcp==0; bitwise OR of TCP
 *        flags on ALL packages when is_tcp==1 and tcp_state!=0; TCP
 *        flags on FIRST packet when is_tcp==1 and tcp_state!=0
 *
 *        tcp_state_out - value of tcp_state field on the rwrec
 *
 *        rest_flags_out - the flags reported by the flow collector
 *        when is_tcp==0 (even though there are no flags to report);
 *        empty when is_tcp==1 and tcp_state==0; bitwise OR of TCP
 *        flags on all but the first packet when is_tcp==1 and
 *        tcp_state!=0.
 *
 *    The output variables prot_flags, tcp_state, and rest_flags will
 *    be the values to store in the packed file format; is_tcp can be
 *    stored in a single bit.  The values can be read by the
 *    _readUnpackProtoFlags() function.
 *
 *    This function should never fail, and thus has no return value.
 */


int _writePackSbbPef(
    uint32_t               *sbb_out,
    uint32_t               *pef_out,
    const rwGenericRec_V3  *rwRP,    
    uint32_t                file_start_time);
/*
 *    Uses fields from the rwRec pointed to by 'rwrec' to compute the
 *    'sbb' and 'pef' fields used when packing SPLIT, WWW, ROUTED, and
 *    NOTROUTED V1 and V2 files.  'file_start_time' is the time value
 *    stored in the header--record times are offset from that time.
 *
 *    Uses the sTime, elapsed, pkts, bytes, sTime_msec, and
 *    elapsed_msec fields in the rwrec to compute these values.  If
 *    the record has millisec values for sTime and/or elapsed, the
 *    sTime and elapsed fields are rounded if required.
 *
 *    sbb and pef are returned in native byte order.
 *
 *    Returns 0 on success or non-zero on these failures: rwRP's sTime
 *    is earlier than rwIOS's sTime or is too large; elapsed time
 *    is too large; packets field is zero or too large.
 */


int _writePackTimeBytesPktsFlags(
    uint32_t               *pkts_stime_out,
    uint32_t               *bbe_out,
    uint32_t               *msec_flags_out,
    const rwGenericRec_V3  *rwRP,
    uint32_t                file_start_time);



int _writeRecordWrite(rwIOStruct *rwIOS, const uint8_t *data);
/*
 *  Write the record at 'data' to the file pointed at by 'rwIOS'.
 *  Returns 0 if ok; otherwise returns non-zero.
 *
 */

int _readSkipRecords(rwIOStruct *rwIOS, int skip_count);


#endif /* _LIBRW_PRIV_H */
