/*
** Copyright (C) 2001-2005 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
**
** Use of the SILK system and related source code is subject to the terms
** of the following licenses:
**
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
**
** NO WARRANTY
**
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE,
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT,
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES.
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE
** DELIVERABLES UNDER THIS LICENSE.
**
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie
** Mellon University, its trustees, officers, employees, and agents from
** all claims or demands made against them (and any related losses,
** expenses, or attorney's fees) arising out of, or relating to Licensee's
** and/or its sub licensees' negligent use or willful misuse of or
** negligent conduct or willful misconduct regarding the Software,
** facilities, or other rights or assistance granted by Carnegie Mellon
** University under this License, including, but not limited to, any
** claims of product liability, personal injury, death, damage to
** property, or violation of any laws or regulations.
**
** Carnegie Mellon University Software Engineering Institute authored
** documents are sponsored by the U.S. Department of Defense under
** Contract F19628-00-C-0003. Carnegie Mellon University retains
** copyrights in all material produced under this contract. The U.S.
** Government retains a non-exclusive, royalty-free license to publish or
** reproduce these documents, or allow others to do so, for U.S.
** Government purposes only pursuant to the copyright license under the
** contract clause at 252.227.7013.
**
** @OPENSOURCE_HEADER_END@
*/

/*
**  mapsid.c
**    dump a map of sid integers to names from silk_site.h
*/

#include "silk.h"

RCSIDENT("$SiLK: /src/rw/librw/mapsid.c,v 1.14 2005/02/28 18:35:51 thomasm Exp $");

#define DECLARE_SENSORID_VARIABLES
#include "silk_site.h"
#include "utils.h"

/* where to send output */
#define OUT_FH stdout


static int mapName(char *name)
{
    uint32_t i;

    upper(name);
    for (i = 0; i < numSensors; i++ ) {
        if (strcmp(name, sensorInfo[i].sensorName) == 0) {
            if (sensorInfo[i].numClasses == 0) {
                skAppPrintErr("Sensor name '%s' not invalid: it has no data",
                              name);
                return 1;
            } else {
                fprintf(OUT_FH, "%s -> %5u\n", sensorInfo[i].sensorName, i);
                return 0;
            }
        }
    }

    skAppPrintErr("name '%s' is not a valid sensor name", name);
    return 1;
}


static void printByNameOrNumber(char *sensor)
{
    int err;
    uint32_t sid;

    /* try to parse as a number */
    err = skStringParseUint32(&sid, sensor, 0, numSensors-1);
    if (err == -1 || err == -2) {
        skAppPrintErr("No name or number was provided.");
    } else if (err == -4) {
        skAppPrintErr(("number '%s' is not a valid sensor number:"
                       " too large to parse"),
                      sensor);
    } else if (err == -12) {
        skAppPrintErr("number '%s' is not a valid sensor number: out of range",
                      sensor);
    } else if (err == 0) {
        /* got a clean parse */
        if (sensorInfo[sid].numClasses == 0) {
            skAppPrintErr("Sensor number '%s' is invalid: it has no data",
                          sensor);
        } else {
            fprintf(OUT_FH, "%5u -> %s\n",
                    sid, sensorInfo[sid].sensorName);
        }
    } else {
        /* didn't get a clean parse. try to treat as a name */
        mapName(sensor);
    }
}


static void printAllSensors(void)
{
    uint32_t count;
    uint32_t i;

    count = 0;
    for (i  = 0; i < numSensors; i++ ) {
        /* only count sensors that are associated with classes */
        if (sensorInfo[i].numClasses > 0) {
            count++;
            fprintf(OUT_FH, "%5d -> %s\n", i, sensorInfo[i].sensorName);
        }
    }

    fprintf(OUT_FH, "Total sensors %u\n", count);
}


int main(int argc, char **argv)
{
    int i;

    skAppRegister(argv[0]);

    if (argc > 1) {
        for (i = 1; i < argc; i++) {
            printByNameOrNumber(argv[i]);
        }
    } else {
        /* no args. dump all */
        printAllSensors();
    }

    return 0;
}
