/*-------------------------------------------------------------------------
 *  profile.c: Simple profiler.
 *
 *  Copyright (C) 2004 by Accelerated Servers,
 *  			  Todd K McAnally <todd@acceleratedservers.com>
 *  			  Charles Galpin  <charles@acceleratedservers.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $SiLK: profile.c,v 1.2 2004/05/12 17:07:15 thomasm Exp $
 *-----------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include <profile.h>

void init_profile(struct profile *prof, char * name) {
	strncpy(prof->name, name, sizeof(prof->name) - 1 );
	prof->name[sizeof(prof->name)-1] = '\0';

	prof->largest = 0;
	prof->smallest = 0;
	prof->times_called = 0;
	prof->cur_starttime = 0;
	prof->cur_stoptime = 0;
	prof->total_time = 0;
	memset(&prof->total_tms, sizeof(struct tms), 0);
	memset(&prof->cur_starttms, sizeof(struct tms), 0);
	memset(&prof->cur_stoptms, sizeof(struct tms), 0);
}

void start_profile (struct profile *prof) {
	prof->times_called++;
	prof->cur_starttime = times(&prof->cur_starttms);
}

void stop_profile (struct profile *prof) {
	clock_t elapsed;

	prof->cur_stoptime = times(&prof->cur_stoptms);
	elapsed = prof->cur_stoptime - prof->cur_starttime;

	if (elapsed > 0) {
		if ((prof->smallest == 0) || (prof->smallest > elapsed)) {
			prof->smallest = elapsed;
		}
		if (prof->largest < elapsed) {
			prof->largest = elapsed;
		}
	}

	prof->total_time += elapsed;
	prof->total_tms.tms_utime += 
		(prof->cur_stoptms.tms_utime - prof->cur_starttms.tms_utime);
	prof->total_tms.tms_stime += 
		(prof->cur_stoptms.tms_stime - prof->cur_starttms.tms_stime);
	prof->total_tms.tms_cutime += 
		(prof->cur_stoptms.tms_cutime - prof->cur_starttms.tms_cutime);
	prof->total_tms.tms_cstime += 
		(prof->cur_stoptms.tms_cstime - prof->cur_starttms.tms_cstime);
}

void print_profile (FILE *out, struct profile *prof) {
	fprintf(out, "PROFILE: %s\n", prof->name);
	fprintf(out, "	times called %lu\n", prof->times_called);
	fprintf(out, "	total time %llu\n", prof->total_time);
	/*
	fprintf(out, "	largest time %llu\n", prof->largest);
	fprintf(out, "	smallest time %llu\n", prof->smallest);
	*/
	fprintf(out, "	utime %lu\n", prof->total_tms.tms_utime);
	fprintf(out, "	stime %lu\n", prof->total_tms.tms_stime);
	fprintf(out, "	cutime %lu\n", prof->total_tms.tms_cutime);
	fprintf(out, "	cstime %lu\n", prof->total_tms.tms_cstime);
}

