/*
** Copyright (C) 2005 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
** 
** Use of the SILK system and related source code is subject to the terms 
** of the following licenses:
** 
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
** 
** NO WARRANTY
** 
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
** DELIVERABLES UNDER THIS LICENSE.
** 
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
** Mellon University, its trustees, officers, employees, and agents from 
** all claims or demands made against them (and any related losses, 
** expenses, or attorney's fees) arising out of, or relating to Licensee's 
** and/or its sub licensees' negligent use or willful misuse of or 
** negligent conduct or willful misconduct regarding the Software, 
** facilities, or other rights or assistance granted by Carnegie Mellon 
** University under this License, including, but not limited to, any 
** claims of product liability, personal injury, death, damage to 
** property, or violation of any laws or regulations.
** 
** Carnegie Mellon University Software Engineering Institute authored 
** documents are sponsored by the U.S. Department of Defense under 
** Contract F19628-00-C-0003. Carnegie Mellon University retains 
** copyrights in all material produced under this contract. The U.S. 
** Government retains a non-exclusive, royalty-free license to publish or 
** reproduce these documents, or allow others to do so, for U.S. 
** Government purposes only pursuant to the copyright license under the 
** contract clause at 252.227.7013.
** 
** @OPENSOURCE_HEADER_END@
*/

/*
**  Functions to support printing a record as text.
**
*/


#include "silk.h"

RCSIDENT("$SiLK: /src/rw/librw/rwascii.c,v 1.4 2005/10/26 14:10:35 thomasm Exp $");

#include "librw_priv.h"
#include "skstringmap.h"

/* following is required for outFInfo[] */
#include "silk_site.h"


/* error message when out of memory */
#define ERRMSG_ALLOC "memory allocation failed at %s:%d", __FILE__, __LINE__


struct _rwAsciiStream {
    FILE           *as_out_stream;
    uint8_t         as_widths[RWREC_PRINTABLE_FIELD_COUNT];
    uint32_t       *as_field_list;
    uint32_t        as_field_count;
    uint8_t         as_initialized;
    char            as_delimiter;
    int8_t          as_not_columnar;
    int8_t          as_integer_ips;
    int8_t          as_zero_pad_ips;
    int8_t          as_no_titles;
    int8_t          as_epoch_time;
    int8_t          as_integer_sensors;
    int8_t          as_icmp_type_code;
    int8_t          as_no_newline;
};


/*
 * This struct holds the field names and their IDs.  The same names
 * are used for the column titles.
 *
 * Names that map to the same ID must be put together, with the name
 * that you want to use for the title first, then any aliases
 * afterward.

 * NOTE!! We are assuming that the stringmap code leaves things in the
 * order we insert them and doesn't re-arrange them.  Since that code
 * uses a linked-list, we are safe for now.
 */
static const sk_stringmap_entry_t field_map_entries[] = {
    {"sIP",          RWREC_FIELD_SIP},
    {"sip",          RWREC_FIELD_SIP},
    {"1",            RWREC_FIELD_SIP},
    {"dIP",          RWREC_FIELD_DIP},
    {"dip",          RWREC_FIELD_DIP},
    {"2",            RWREC_FIELD_DIP},
    {"sPort",        RWREC_FIELD_SPORT},
    {"sport",        RWREC_FIELD_SPORT},
    {"3",            RWREC_FIELD_SPORT},
    {"dPort",        RWREC_FIELD_DPORT},
    {"dport",        RWREC_FIELD_DPORT},
    {"4",            RWREC_FIELD_DPORT},
    {"protocol",     RWREC_FIELD_PROTO},
    {"5",            RWREC_FIELD_PROTO},
    {"packets",      RWREC_FIELD_PKTS},
    {"pkts",         RWREC_FIELD_PKTS},
    {"6",            RWREC_FIELD_PKTS},
    {"bytes",        RWREC_FIELD_BYTES},
    {"7",            RWREC_FIELD_BYTES},
    {"flags",        RWREC_FIELD_FLAGS},
    {"8",            RWREC_FIELD_FLAGS},
    {"sTime",        RWREC_FIELD_STIME},
    {"stime",        RWREC_FIELD_STIME},
    {"9",            RWREC_FIELD_STIME},
    {"dur",          RWREC_FIELD_ELAPSED},
    {"10",           RWREC_FIELD_ELAPSED},
    {"eTime",        RWREC_FIELD_ETIME},
    {"etime",        RWREC_FIELD_ETIME},
    {"11",           RWREC_FIELD_ETIME},
    {"sensor",       RWREC_FIELD_SID},
    {"12",           RWREC_FIELD_SID},
    {"in",           RWREC_FIELD_INPUT},
    {"13",           RWREC_FIELD_INPUT},
    {"out",          RWREC_FIELD_OUTPUT},
    {"14",           RWREC_FIELD_OUTPUT},
    {"nhIP",         RWREC_FIELD_NHIP},
    {"15",           RWREC_FIELD_NHIP},
    {"class",        RWREC_FIELD_FTYPE_CLASS},
    {"type",         RWREC_FIELD_FTYPE_TYPE},
    {"sTime+msec",   RWREC_FIELD_STIME_MSEC},
    {"stime+msec",   RWREC_FIELD_STIME_MSEC},
    {"eTime+msec",   RWREC_FIELD_ETIME_MSEC},
    {"etime+msec",   RWREC_FIELD_ETIME_MSEC},
    {"dur+msec",     RWREC_FIELD_ELAPSED_MSEC}
};

const size_t field_map_count = (sizeof(field_map_entries)
                                / sizeof(sk_stringmap_entry_t));


/*
**  rwAsciiSetWidths(ascii_stream);
**
**    Set the column widths for all the columns in the 'ascii_stream'
**    struct.  The column widths will depend on flags set by the user,
**    such as whether the output is columnar, or the form of sensors
**    (names or numbers).
*/
static void rwAsciiSetWidths(rwAsciiStream_t *stream)
{
    uint8_t i;
    size_t longest_class;
    size_t longest_type;
    size_t s;

    if (stream->as_not_columnar) {
        memset(stream->as_widths, 0,
               sizeof(uint8_t)*RWREC_PRINTABLE_FIELD_COUNT);
        return;
    }

    /* get width of longest class and type name */
    longest_class = strlen(outFInfo[0].className);
    longest_type  = strlen(outFInfo[0].dirPrefix);
    for (i = 1; i < NUM_FLOW_TYPES; ++i) {
        s = strlen(outFInfo[i].className);
        if (s > longest_class) {
            longest_class = s;
        }

        s = strlen(outFInfo[i].dirPrefix);
        if (s > longest_type) {
            longest_type = s;
        }
    }

    for (i = 0; i < RWREC_PRINTABLE_FIELD_COUNT; i++) {
        switch (i) {
          case RWREC_FIELD_SIP:
          case RWREC_FIELD_DIP:
          case RWREC_FIELD_NHIP:
            /* ip numbers */
            if (stream->as_integer_ips) {
                stream->as_widths[i] = 10;
            } else {
                stream->as_widths[i] = 15;
            }
            break;

          case RWREC_FIELD_SPORT:
          case RWREC_FIELD_DPORT:
            /* sport and dport */
            stream->as_widths[i] = 5;
            break;

          case RWREC_FIELD_PROTO:
            /* proto */
            stream->as_widths[i] = 3;
            break;

          case RWREC_FIELD_PKTS:
          case RWREC_FIELD_BYTES:
            /* packets, bytes */
            stream->as_widths[i] = 10;
            break;

          case RWREC_FIELD_FLAGS:
          case RWREC_FIELD_INIT_FLAGS:
          case RWREC_FIELD_REST_FLAGS:
          case RWREC_FIELD_TCP_STATE:
            /* tcp flags, init-flags, non-init-flags, tcp-state */
            stream->as_widths[i] = 8;
            break;

          case RWREC_FIELD_APPLICATION:
            stream->as_widths[i] = 5;
            break;

          case RWREC_FIELD_ELAPSED:
            /* elapsed/duration */
            stream->as_widths[i] = 5;
            break;

          case RWREC_FIELD_ELAPSED_MSEC:
            /* elapsed/duration with milliseconds */
            stream->as_widths[i] = 9;
            break;

          case RWREC_FIELD_STIME:
          case RWREC_FIELD_ETIME:
            /* sTime and end time */
            if (stream->as_epoch_time) {
                stream->as_widths[i] = 10;
            } else {
                stream->as_widths[i] = 19;
            }
            break;

          case RWREC_FIELD_STIME_MSEC:
          case RWREC_FIELD_ETIME_MSEC:
            /* sTime and end time with milliseconds */
            if (stream->as_epoch_time) {
                stream->as_widths[i] = 14;
            } else {
                stream->as_widths[i] = 23;
            }
            break;

          case RWREC_FIELD_SID:
            /* sensor */
            if (stream->as_integer_sensors) {
                if (SENSOR_INVALID_SID < 99999) {
                    stream->as_widths[i] = 5;
                } else {
                    stream->as_widths[i] = 10;
                }
            } else {
                stream->as_widths[i] = SENSOR_MAX_SENSOR_NAME_LEN;
            }
            break;

          case RWREC_FIELD_INPUT:
          case RWREC_FIELD_OUTPUT:
            /* input,output */
            stream->as_widths[i] = 5;
            break;

          case RWREC_FIELD_FTYPE_CLASS:
            /* flow-type class */
            stream->as_widths[i] = longest_class;
            break;

          case RWREC_FIELD_FTYPE_TYPE:
            /* flow-type type */
            stream->as_widths[i] = longest_type;
            break;

          default:
            skAppPrintErr("Option %u not handled in switch() at %s:%d",
                          i, __FILE__, __LINE__);
            abort();
        } /* switch */
    }
    return;
}
/* rwAsciiSetWidths */


/*
 *  rwAsciiSetDefaultFields(stream);
 *
 *    Configure the 'stream' to print all fields.
 */
static void rwAsciiSetDefaultFields(rwAsciiStream_t *stream)
{
    uint32_t i;

    stream->as_field_count = RWREC_PRINTABLE_FIELD_COUNT;

    stream->as_field_list = malloc(stream->as_field_count*sizeof(uint32_t));
    if (NULL == stream->as_field_list) {
        /* out of memory */
        skAppPrintErr(ERRMSG_ALLOC);
        exit(EXIT_FAILURE);
    }

    for (i = 0; i < stream->as_field_count; ++i) {
        stream->as_field_list[i] = i;
    }
}
/* rwAsciiSetDefaultFields */


/*
 *  rwAsciiVerifyIcmpColumns(stream);
 *
 *    If the ICMP type and code are to be written, verify that the
 *    sPort and dPort columns are present, adding them if required.
 */
static void rwAsciiVerifyIcmpColumns(rwAsciiStream_t *stream)
{
    uint32_t i;
    int have_sport = 0;
    int have_dport = 0;

    for (i = 0; i < stream->as_field_count; ++i) {
        if (stream->as_field_list[i] == RWREC_FIELD_SPORT) {
            have_sport = 1;
            if (have_dport) {
                return;
            }
        } else if (stream->as_field_list[i] == RWREC_FIELD_DPORT) {
            have_dport = 1;
            if (have_sport) {
                return;
            }
        }
    }

    /* if we make it here, sport and/or dport are needed */
    stream->as_field_list = realloc(stream->as_field_list,
                                    ((2 + stream->as_field_count)
                                     * sizeof(uint32_t)));
    if (NULL == stream->as_field_list) {
        /* out of memory */
        skAppPrintErr(ERRMSG_ALLOC);
        exit(EXIT_FAILURE);
    }

    if (!have_sport) {
        stream->as_field_list[stream->as_field_count]= RWREC_FIELD_SPORT;
        ++stream->as_field_count;
    }
    if (!have_dport) {
        stream->as_field_list[stream->as_field_count]= RWREC_FIELD_DPORT;
        ++stream->as_field_count;
    }
}
/* rwAsciiVerifyIcmpColumns */


/*
 *  rwAsciiPreparePrint(stream);
 *
 *    Do any final initialization prior to printing the column titles
 *    or the first row: Set the field list to the default list if the
 *    caller did not choose specific columns; if ICMP type and code
 *    output was requested, make certain the correct columns exist in
 *    the output; and set the width of the columns.
 */
static void rwAsciiPreparePrint(rwAsciiStream_t *stream)
{
    stream->as_initialized = 1;

    if (stream->as_field_count == 0) {
        rwAsciiSetDefaultFields(stream);
    }

    if (stream->as_icmp_type_code) {
        rwAsciiVerifyIcmpColumns(stream);
    }

    rwAsciiSetWidths(stream);
}
/* rwAsciiPreparePrint */


int rwAsciiFlush(rwAsciiStream_t *stream)
{
    return fflush(stream->as_out_stream);
}


void rwAsciiStreamDestroy(rwAsciiStream_t **stream)
{
    assert(stream);

    if (NULL == *stream) {
        return;
    }

    if ((*stream)->as_field_count > 0) {
        free((*stream)->as_field_list);
    }
    free(*stream);
    *stream = NULL;
}
/* rwAsciiStreamDestroy */


int rwAsciiStreamCreate(rwAsciiStream_t **stream)
{
    assert(stream);

    /* create the object */
    *stream = calloc(1, sizeof(rwAsciiStream_t));
    if (!*stream) {
        /* out of memory */
        skAppPrintErr(ERRMSG_ALLOC);
        return -1;
    }

    /* non-zero defaults */
    (*stream)->as_out_stream = stdout;
    (*stream)->as_delimiter = '|';

    return 0;
}


int rwAsciiSetFields(
    rwAsciiStream_t    *stream,
    const uint32_t     *field_list,
    uint32_t            field_count)
{
    uint8_t i;

    assert(stream);

    /* verify input */
    if ((field_count == 0) || (field_list == NULL)) {
        return -1;
    }

    for (i = 0; i < field_count; ++i) {
        if (field_list[i] >= RWREC_PRINTABLE_FIELD_COUNT) {
            /* bad value */
            skAppPrintErr("Value '%u' is not a value field id",
                          field_list[i]);
            return -1;
        }
    }

    /* set fields */
    stream->as_field_list = malloc(field_count * sizeof(uint32_t));
    if (NULL == stream->as_field_list) {
        /* out of memory */
        skAppPrintErr(ERRMSG_ALLOC);
        return -1;
    }

    memcpy(stream->as_field_list, field_list, field_count * sizeof(uint32_t));
    stream->as_field_count = field_count;

    return 0;
}
/* rwAsciiSetFields */


void rwAsciiSetOutputHandle(rwAsciiStream_t *stream, FILE *fh)
{
    assert(stream);
    if (fh == NULL) {
        stream->as_out_stream = stdout;
    } else {
        stream->as_out_stream = fh;
    }
}


void rwAsciiSetDelimiter(rwAsciiStream_t *stream, char delimiter)
{
    assert(stream);
    stream->as_delimiter = delimiter;
}


void rwAsciiSetNoColumns(rwAsciiStream_t *stream)
{
    assert(stream);
    stream->as_not_columnar = 1;
}


void rwAsciiSetIntegerIps(rwAsciiStream_t *stream)
{
    assert(stream);

    if (stream->as_zero_pad_ips) {
        skAppPrintErr("Printing IPs as integer overrides zero padding IPs");
        stream->as_zero_pad_ips = 0;
    }

    stream->as_integer_ips = 1;
}


void rwAsciiSetZeroPadIps(rwAsciiStream_t *stream)
{
    assert(stream);

    if (stream->as_integer_ips) {
        skAppPrintErr("Printing IPs as integer overrides zero padding IPs");
        return;
    }

    stream->as_zero_pad_ips = 1;
}


void rwAsciiSetNoTitles(rwAsciiStream_t *stream)
{
    assert(stream);
    stream->as_no_titles = 1;
}


void rwAsciiSetEpochTime(rwAsciiStream_t *stream)
{
    assert(stream);
    stream->as_epoch_time = 1;
}


void rwAsciiSetIntegerSensors(rwAsciiStream_t *stream)
{
    assert(stream);
    stream->as_integer_sensors = 1;
}


void rwAsciiSetIcmpTypeCode(rwAsciiStream_t *stream)
{
    assert(stream);
    stream->as_icmp_type_code = 1;
}


void rwAsciiSetNoNewline(rwAsciiStream_t *stream)
{
    assert(stream);
    stream->as_no_newline = 1;
}


void rwAsciiGetFieldName(
    char                       *buf, 
    size_t                      buf_len,
    rwrec_printable_fields_t    field_id)
{
    size_t i;

    assert(buf);
    assert(buf_len > 0);

    buf[0] = '\0';

    for (i = 0; i < field_map_count; ++i) {
        if (field_map_entries[i].id == field_id) {
            strncpy(buf, field_map_entries[i].name, buf_len-1);
            buf[buf_len-1] = '\0';
            break;
        }
    }
}
/* rwAsciiGetFieldName */


void rwAsciiPrintTitles(rwAsciiStream_t *stream)
{
    char buf[RWREC_PRINTABLE_MAX_NAME_LEN];
    uint32_t i;

    /* initialize */
    if (stream->as_initialized == 0) {
        rwAsciiPreparePrint(stream);
    }

    /* don't print titles if we are not supposed to or if we already
     * have */
    if (stream->as_no_titles) {
        return;
    }
    stream->as_no_titles = -1;

    for (i = 0; i < stream->as_field_count; ++i) {
        rwAsciiGetFieldName(buf, sizeof(buf), stream->as_field_list[i]);
        if (stream->as_not_columnar) {
            fprintf(stream->as_out_stream, "%s%c",
                    buf, stream->as_delimiter);
        } else {
            fprintf(stream->as_out_stream, "%*.*s%c",
                    stream->as_widths[stream->as_field_list[i]],
                    stream->as_widths[stream->as_field_list[i]],
                    buf, stream->as_delimiter);
        }
    } /* for */

    if ( !stream->as_no_newline) {
        fprintf(stream->as_out_stream, "\n");
    }

    return;
}
/* rwAsciiPrintTitles */


/*  Print 'rwrec' to the ascci stream 'stream'.  See header for details */
void rwAsciiPrintRec(
    rwAsciiStream_t    *stream,
    const rwRec        *rwrec)
{
    static char pText[128];
    uint32_t i;
    char *cp;

    assert(stream);
    assert(rwrec);

    /* initialize */
    if (stream->as_initialized == 0) {
        rwAsciiPreparePrint(stream);
    }

    /* print titles if we haven't */
    if (stream->as_no_titles == 0) {
        /* print titles */
        rwAsciiPrintTitles(stream);
    }

    for (i = 0; i < stream->as_field_count; ++i) {
        cp = pText;
        switch (stream->as_field_list[i]) {
          case RWREC_FIELD_SIP:
            if (stream->as_integer_ips) {
                sprintf(cp, "%u", rwrec->sIP.ipnum);
            } else if (stream->as_zero_pad_ips) {
                cp = num2dot0(rwrec->sIP.ipnum);
            } else {
                cp = num2dot(rwrec->sIP.ipnum);
            }
            break;

          case RWREC_FIELD_DIP:
            if (stream->as_integer_ips) {
                sprintf(cp, "%u", rwrec->dIP.ipnum);
            } else if (stream->as_zero_pad_ips) {
                cp = num2dot0(rwrec->dIP.ipnum);
            } else {
                cp = num2dot(rwrec->dIP.ipnum);
            }
            break;

          case RWREC_FIELD_NHIP:
            if (stream->as_integer_ips) {
                sprintf(cp, "%u", rwrec->nhIP.ipnum);
            } else if (stream->as_zero_pad_ips) {
                cp = num2dot0(rwrec->nhIP.ipnum);
            } else {
                cp = num2dot(rwrec->nhIP.ipnum);
            }
            break;

          case RWREC_FIELD_SPORT:
            /* sport.  If as_icmp_type_code is true and the protocol
             * is ICMP, write the ICMP type here */
            if (stream->as_icmp_type_code && rwrec->proto == 1) {
                /* The Cisco 7600s have a bug in the way they process
                 * the ICMP type and code, so that the code may be
                 * (type<<8|code) in the dPort or (code<<8|type) in
                 * the sPort.  To handle this, use the dPort if it is
                 * non-zero, else use the sPort. */
                if (rwrec->dPort > 0) {
                    /* ICMP type is in the most significant byte of dPort */
                    sprintf(cp, "%u", (rwrec->dPort >> 8) & 0xFF);
                } else {
                    /* ICMP type is in the least significant byte of sPort */
                    sprintf(cp, "%u", rwrec->sPort & 0xFF);
                }
            } else {
                sprintf(cp, "%u", rwrec->sPort);
            }
            break;

          case RWREC_FIELD_DPORT:
            /* dport.  If as_icmp_type_code is true and the protocol
             * is ICMP, write the ICMP code here */
            if (stream->as_icmp_type_code && rwrec->proto == 1) {
                /* Handle the Cisco 7600; see SPORT note above */
                if (rwrec->dPort > 0) {
                    /* ICMP code is in the least significant byte of dPort */
                    sprintf(cp, "%u", rwrec->dPort & 0xFF);
                } else {
                    /* ICMP code is in the most significant byte of sPort */
                    sprintf(cp, "%u", (rwrec->sPort >> 8) & 0xFF);
                }
            } else {
                sprintf(cp, "%u", rwrec->dPort);
            }
            break;

          case RWREC_FIELD_PROTO:
            sprintf(cp, "%u", rwrec->proto);
            break;

          case RWREC_FIELD_PKTS:
            sprintf(cp, "%u", rwrec->pkts);
            break;

          case RWREC_FIELD_BYTES:
            sprintf(cp, "%u", rwrec->bytes);
            break;

          case RWREC_FIELD_FLAGS:
            cp = tcpflags_string(rwrec->flags);
            break;

          case RWREC_FIELD_INIT_FLAGS:
            cp = tcpflags_string(rwrec->init_flags);
            break;

          case RWREC_FIELD_REST_FLAGS:
            cp = tcpflags_string(rwrec->rest_flags);
            break;

          case RWREC_FIELD_TCP_STATE:
            sprintf(cp, "%c%c%c     ",
                    ((rwrec->tcp_state & SK_TCPSTATE_TIMEOUT_KILLED)
                     ? 'T' : ' '),
                    ((rwrec->tcp_state & SK_TCPSTATE_TIMEOUT_STARTED)
                     ? 'C' : ' '),
                    ((rwrec->tcp_state & SK_TCPSTATE_FIN_FOLLOWED_NOT_ACK)
                     ? 'F' : ' '));
            break;

          case RWREC_FIELD_APPLICATION:
            sprintf(cp, "%u", rwrec->application);
            break;

          case RWREC_FIELD_ELAPSED:
            sprintf(cp, "%u", rwrec->elapsed);
            break;

          case RWREC_FIELD_ELAPSED_MSEC:
            sprintf(cp, "%u.%03u", rwrec->elapsed, rwrec->elapsed_msec);
            break;

          case RWREC_FIELD_STIME:
            if (stream->as_epoch_time) {
                sprintf(cp, "%u", rwrec->sTime);
            } else {
                cp = timestamp(rwrec->sTime);
            }
            break;

          case RWREC_FIELD_ETIME:
            if (stream->as_epoch_time) {
                sprintf(cp, "%u", rwrec->sTime + rwrec->elapsed);
            } else {
                cp = timestamp(rwrec->sTime + rwrec->elapsed);
            }
            break;

          case RWREC_FIELD_STIME_MSEC:
            if (stream->as_epoch_time) {
                sprintf(cp, "%u.%03u", rwrec->sTime, rwrec->sTime_msec);
            } else {
                sprintf(cp, "%s.%03u",
                        timestamp(rwrec->sTime), rwrec->sTime_msec);
            }
            break;

          case RWREC_FIELD_ETIME_MSEC:
            {
                uint32_t etime = rwrec->sTime + rwrec->elapsed;
                uint32_t etime_msec = rwrec->sTime_msec + rwrec->elapsed_msec;
                if (etime_msec >= 1000) {
                    ++etime;
                    etime_msec -= 1000;
                }
                if (stream->as_epoch_time) {
                    sprintf(cp, "%u.%03u", etime, etime_msec);
                } else {
                    sprintf(cp, "%s.%03u", timestamp(etime), etime_msec);
                }
            }
            break;

          case RWREC_FIELD_SID:
            /* sensor ID */
            if ( !stream->as_integer_sensors ) {
                cp = (char*)sensorIdToName(rwrec->sID);
            } else if (SENSOR_INVALID_SID == rwrec->sID) {
                cp = "-1";
            } else {
                sprintf(cp, "%u", rwrec->sID);
            }
            break;

          case RWREC_FIELD_INPUT:
            sprintf(cp, "%u", rwrec->input);
            break;

          case RWREC_FIELD_OUTPUT:
            /* output */
            sprintf(cp, "%u", rwrec->output);
            break;

          case RWREC_FIELD_FTYPE_CLASS:
            if (rwrec->flow_type < NUM_FLOW_TYPES) {
                sprintf(cp, "%s", outFInfo[rwrec->flow_type].className);
            } else {
                cp = "?";
            }
            break;

          case RWREC_FIELD_FTYPE_TYPE:
            if (rwrec->flow_type < NUM_FLOW_TYPES) {
                sprintf(cp, "%s", outFInfo[rwrec->flow_type].dirPrefix);
            } else {
                cp = "?";
            }
            break;

          default:
            skAppPrintErr("Option %u not handled in switch() at %s:%d",
                          stream->as_field_list[i], __FILE__, __LINE__);
            abort();
            break;
        } /* switch */

        if (stream->as_not_columnar) {
            fprintf(stream->as_out_stream, "%s%c",
                    cp, stream->as_delimiter);
        } else {
            fprintf(stream->as_out_stream, "%*s%c",
                    stream->as_widths[stream->as_field_list[i]],
                    cp, stream->as_delimiter);
        }
    } /* for */

    if ( !stream->as_no_newline) {
        fprintf(stdout, "\n");
    }

    return;
}
/* rwAsciiPrintRec */


int rwAsciiParseFieldList(
    rwrec_printable_fields_t  **field_ids,
    uint32_t                   *field_count,
    const char                 *field_string)
{
    const uint32_t max_field_count = 8 * RWREC_PRINTABLE_FIELD_COUNT;
    unsigned long n, range_start, i;
    const char *sp;
    char *ep;
    rwrec_printable_fields_t *out_array_list = NULL; /* returned array */
    uint32_t out_count = 0; /* returned count */
    uint32_t array_size;
    uint32_t range_length;
    const uint32_t min_value = 1;
    const uint32_t max_value = RWREC_PRINTABLE_FIELD_COUNT;
    /* the last thing we successfully parsed. NUMBER is a single number
     * or start of a range; RANGE is the upper limit of a range. */
    enum {COMMA, HYPHEN, NUMBER, RANGE} parse_state;

    /* check input */
    assert(field_ids);
    assert(field_count);
    assert(field_string);

    /* Create the array to hold the list of values; use 2 * number of
     * possible fields. */
    array_size = RWREC_PRINTABLE_FIELD_COUNT * 2;

    out_array_list = calloc(array_size, sizeof(rwrec_printable_fields_t));
    if (!out_array_list) {
        skAppPrintErr("Out of memory! out_array_list=calloc()");
        goto ERROR;
    }

    /* range_start is non-zero when the number we are about to parse
     * is the upper half of the range m-n */
    range_start = 0;
    parse_state = COMMA;
    sp = field_string;

    /* parse user input */
    while (*sp) {
        if (!isdigit((int)*sp)) {
            if (parse_state == HYPHEN) {
                /* digit must follow a hyphen */
                skAppPrintErr(("parse error: expecting digit after hyphen; "
                               "saw '%c' at position %u"),
                              *sp, (unsigned)1+(sp - field_string));
                goto ERROR;
            }
            if (*sp == ',') {
                /* Comma is legal anywhere except after a hyphen,
                 * which we handled above. (we allow double comma) */
                parse_state = COMMA;
                ++sp;
                continue;
            }
            if (parse_state == RANGE) {
                /* range must end with a comma */
                skAppPrintErr(("parse error: expecting comma after range; "
                               "saw '%c' at position %u"),
                               *sp, (unsigned)1+(sp - field_string));
                goto ERROR;
            }
            if (parse_state == COMMA) {
                /* a digit or a comma(handled above) must follow a comma */
                skAppPrintErr(("parse error: expecting digit after comma; "
                               "saw '%c' at position %u"),
                              *sp, (unsigned)1+(sp - field_string));
                goto ERROR;
            }
            /* if we are here, parse_state must be NUMBER */
            assert(parse_state == NUMBER);
            if (*sp == '-') {
                parse_state = HYPHEN;
                ++sp;
                continue;
            }
            skAppPrintErr(("parse error: expecting digit, comma, or hyphen; "
                           "saw '%c' at position %u"),
                          *sp, (unsigned)1+(sp - field_string));
            goto ERROR;
        }

        /* If we make it here, we are looking at a digit; parse_state
         * must be either COMMA or HYPHEN */
        assert(parse_state == COMMA || parse_state == HYPHEN);

        /* parse the number */
        errno = 0;
        n = strtoul(sp, &ep, 10);
        if (sp == ep) {
            skAppPrintErr("cannot parse '%s' as an integer", sp);
            goto ERROR;
        }
        if (n == ULONG_MAX && errno == ERANGE) {
            skAppPrintErr("the number '%s' overflows the parser", sp);
            goto ERROR;
        }
        if (n < min_value) {
            skAppPrintErr("value '%lu' is less than minimum of %u",
                          n, min_value);
            goto ERROR;
        }
        if ((max_value != 0) && (n >= max_value)) {
            skAppPrintErr("value '%lu' is above the maximum of %u",
                          n, max_value);
            goto ERROR;
        }

        /* parsed the number; move pointer to next token */
        sp = ep;

        /* determine if the number we just parsed is the end of a
         * range or a lone number---and potentially the start of a
         * range */
        if (parse_state == COMMA) {
            /* we were not parsing the upper half of a range.  treat
             * it as a range of 1 */
            parse_state = NUMBER;
            range_length = 1;
            range_start = n;
        } else {
            /* parse_state must be HYPHEN: we were parsing a range */
            parse_state = RANGE;

            if (n < range_start) {
                /* user entered 3-2 */
                skAppPrintErr("bad range: lower > upper: %lu-%lu",
                              range_start, n);
                goto ERROR;
            }

            if (range_start == n) {
                /* user entered 3-3, ignore second '3' */
                continue;
            }

            /* we added range_start on the previous iteration, so add
             * range_start+1 through n now; */
            range_length = n - range_start;
            range_start++;
        }

        /* check number of fields user gave */
        if ((out_count + range_length) > max_field_count) {
            skAppPrintErr("Too many fields provided. Only %u fields allowed",
                          max_field_count);
            goto ERROR;
        }

        /* check if we need to grow array?  If so, realloc the array

         * to double its size and memset the new section to 0. */
        while ((out_count + range_length) > array_size) {
            size_t old_size = array_size;
            array_size *= 2;
            if (array_size > max_field_count) {
                array_size = max_field_count;
            }
            out_array_list = realloc(out_array_list, array_size);
            if (!out_array_list) {
                skAppPrintErr("Out of memory! out_array_list=realloc()");
                goto ERROR;
            }
            memset((out_array_list+old_size), 0, array_size-old_size);
        }

        /* add the entries */
        for (i = range_start; i <= n; ++i) {
            out_array_list[out_count] = i;
            ++out_count;
        }
    } /* while(*sp) */

    *field_count = out_count;
    *field_ids = out_array_list;
    return 0;

  ERROR:
    if (field_ids) {
        free(field_ids);
    }
    *field_count = 0;
    return -1;
}
/* rwAsciiParseFieldList */


int rwAsciiFieldMapAddDefaultFields(sk_stringmap_t **field_map)
{
    assert(field_map);

    /* Create the map if necessary */
    if (NULL == *field_map) {
        if (SKSTRINGMAP_OK != skStringMapCreate(field_map)) {
            return -1;
        }
    }

    /* add entries */
    if (skStringMapAddIDArray(*field_map, field_map_count, field_map_entries)
        != SKSTRINGMAP_OK)
    {
        return -1;
    }

    return 0;
}


int rwAsciiFieldMapParseFields(
    uint32_t              **field_list,
    uint32_t               *field_count,
    const char             *input,
    const sk_stringmap_t   *field_map,
    rwascii_dupes_t         handle_dupes)
{
    int rv = 1; /* return value; assume failure */
    sk_vector_t *entries_vec = NULL;
    sk_stringmap_entry_t *map_entry;
    sk_stringmap_status_t err;
    uint32_t *list = NULL;
    size_t count;
    size_t i;
    uint32_t j;
    char *bad_token = NULL;

    assert(field_list);
    assert(field_count);

    /* initialize field_count */
    *field_count = 0;

    /* create vector to hold match results */
    entries_vec = skVectorNew(sizeof(sk_stringmap_entry_t*));
    if (entries_vec == NULL) {
        skAppPrintErr("allocation failure at %s:%d",
                      __FILE__, __LINE__);
        goto END;
    }

    /* attempt to match */
    err = skStringMapMatch(entries_vec, &bad_token, field_map, input);
    if (err != SKSTRINGMAP_OK) {
        switch (err) {
          case SKSTRINGMAP_PARSE_NO_MATCH:
            skAppPrintErr("No match was found for the value '%s'", bad_token);
            goto END;
          case SKSTRINGMAP_PARSE_AMBIGUOUS:
            skAppPrintErr("The value '%s' matches multiple keys", bad_token);
            goto END;
          case SKSTRINGMAP_PARSE_UNPARSABLE:
            skAppPrintErr("Unable to parse the value '%s'", bad_token);
            goto END;
          default:
            skAppPrintErr("Unexpected return value from parser (%d)",
                          err);
            goto END;
        }
    }

    /* put the results into the field_list[] array */
    count = skVectorGetCount(entries_vec);
    list = calloc(count, sizeof(uint32_t));
    if (!list) {
        skAppPrintErr("allocation failure at %s:%d",
                      __FILE__, __LINE__);
        goto END;
    }

    for (i = 0; i < count; ++i) {
        (void)skVectorGetValue(&map_entry, entries_vec, i);
        if (handle_dupes != RW_ASCII_DUPES_KEEP) {
            /* search for a duplicate */
            for (j = 0; j < *field_count; ++j) {
                if (list[j] != map_entry->id) {
                    continue;
                }
                /* found a dupe */
                switch (handle_dupes) {
                  case RW_ASCII_DUPES_ERROR:
                    skAppPrintErr("Error: multiple occurrences of %u value",
                                  map_entry->id);
                    goto END;

                  case RW_ASCII_DUPES_REMOVE_WARN:
                    skAppPrintErr("Ignoring multiple occurrences of %u value",
                                  map_entry->id);
                    /* FALLTHROUGH */

                  case RW_ASCII_DUPES_REMOVE_SILENT:
                    /* force us out of the for-loop */
                    goto NEXT_MAP_ENTRY;

                  case RW_ASCII_DUPES_KEEP:
                    /* shouldn't be in this loop at all */
                    assert(0);
                    abort();
                }
            }
        }

        /* add the entry */
        list[*field_count] = map_entry->id;
        ++*field_count;

      NEXT_MAP_ENTRY:
        /* NO-OP*/ ;
    }

    /* success */
    *field_list = list;
    rv = 0;

  END:
    if (rv != 0) {
        *field_count = 0;
        if (list != NULL) {
            free(list);
        }
    }
    if (bad_token != NULL) {
        free(bad_token);
    }
    if (entries_vec != NULL) {
        skVectorDestroy(entries_vec);
    }

    return rv;
}


void rwAsciiFieldMapPrintUsage(FILE *fh, sk_stringmap_t *field_map)
{
    sk_stringmap_entry_t *entry;
    sk_stringmap_entry_t *old_entry;
    sk_link_item_t *node;
    sk_link_err_t rv;
    int len;
    const char column_sep = ';';
    const char alias_sep = ',';
    const int wrap_col = 65;

    fprintf(fh,
            "\t(Semicolon separates unique columns."
            " Comma separates column aliases.\n"
            "\t Names can be abbreviated to shortest unique prefix.)\n");

    /* force start of new line */
    len = -1;

    /* previous value from map */
    old_entry = NULL;

    /* loop through all entries in the map */
    for (rv = skLinkGetHead(&node, field_map);
         rv == SKLINK_OK;
         rv = skLinkGetNext(&node, node))
    {
        skLinkGetData((void**)&entry, node);

        if ((old_entry != NULL) && (old_entry->id == entry->id)) {
            /* 'entry' is an alias for 'old_entry' */
            len += fprintf(fh, "%c", alias_sep);
        } else {
            if (len >= wrap_col) {
                /* line too long, break it and set len to new-line value */
                fprintf(fh, "%c\n", column_sep);
                len = -1;
            }
            if (len < 0) {
                /* staring a new line */
                len = fprintf(fh, "    ");
            } else {
                /* continuing current line */
                len += fprintf(fh, "%c ", column_sep);
            }

            /* DEBUG *//* len += fprintf(fh, "(%u)", entry->id); */
        }
        
        len += fprintf(fh, "%s", entry->name);
        old_entry = entry;
    }
    fprintf(fh, "\n");
}
