/*
** Copyright (C) 2004 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
** 
** Use of the SILK system and related source code is subject to the terms 
** of the following licenses:
** 
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
** 
** NO WARRANTY
** 
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
** DELIVERABLES UNDER THIS LICENSE.
** 
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
** Mellon University, its trustees, officers, employees, and agents from 
** all claims or demands made against them (and any related losses, 
** expenses, or attorney's fees) arising out of, or relating to Licensee's 
** and/or its sub licensees' negligent use or willful misuse of or 
** negligent conduct or willful misconduct regarding the Software, 
** facilities, or other rights or assistance granted by Carnegie Mellon 
** University under this License, including, but not limited to, any 
** claims of product liability, personal injury, death, damage to 
** property, or violation of any laws or regulations.
** 
** Carnegie Mellon University Software Engineering Institute authored 
** documents are sponsored by the U.S. Department of Defense under 
** Contract F19628-00-C-0003. Carnegie Mellon University retains 
** copyrights in all material produced under this contract. The U.S. 
** Government retains a non-exclusive, royalty-free license to publish or 
** reproduce these documents, or allow others to do so, for U.S. 
** Government purposes only pursuant to the copyright license under the 
** contract clause at 252.227.7013.
** 
** @OPENSOURCE_HEADER_END@
*/

/*
**  A simple date data type.
**
**  Sep 2004  Dave Svoboda
**
*/

#include "silk.h"

RCSIDENT("$SiLK: /src/rw/rwfilter/cfg_date.c,v 1.2 2004/11/09 16:15:32 thomasm Exp $");

#include "utils.h"
#include "cfg_config.h"


/** Date functions **/

/* We are assuming:
 *  dates employ the same ordering as integers
 *  min_int < minimum date, max_int > maximum date
 * I trust this renders us Y2K compliant :)
 **/
cfg_date_t date_min = { INT_MIN, UNSPECIFIED_HOUR };
cfg_date_t date_max = { INT_MAX, UNSPECIFIED_HOUR };


cfg_err_t
cfg_date_create(cfg_date_t** x, int year, int month, int day) {
    return cfg_date_create_hour( x, year, month, day, UNSPECIFIED_HOUR);
}


cfg_err_t
cfg_date_create_hour(cfg_date_t** x, int year, int month, int day, int hour)
{
    int32_t date = julianDate( year, month, day);
    return cfg_date_create_jd_hour( x, date, hour);
}


cfg_err_t
cfg_date_create_jd(cfg_date_t** x, int32_t date)
{
    return cfg_date_create_jd_hour( x, date, UNSPECIFIED_HOUR);
}


cfg_err_t
cfg_date_create_jd_hour(cfg_date_t** x, int32_t date, int hour)
{
    cfg_date_t* result;
    CFG_ALLOCATE( cfg_date_t, result, 1);
    result->date = date;
    result->hour = hour;
    *x = result;
    return CFG_OK;
}


cfg_err_t
cfg_date_clone(void** result,const void* x)
{
    cfg_date_t* p = CFG_CAST( cfg_date_t, x);
    return cfg_date_create_jd_hour( CFG_CAST_PTR( cfg_date_t, *result),
				    p->date, p->hour);
}


cfg_err_t
cfg_date_destroy(cfg_date_t* x)
{
    FREE( x);
    return CFG_OK;
}


int
cfg_date_compare(const cfg_date_t* d1,const cfg_date_t* d2) {
    /* We are assuming:
     *  dates employ the same ordering as integers
     *  min_int < minimum date, max_int > maximum date
     * I trust this renders us Y2K compliant :)
     **/
    return (d1->date < d2->date) ? -1 :
	(d1->date > d2->date) ? 1 :
	(d1->hour < d2->hour) ? -1 :
	(d1->hour > d2->hour) ? 1 :
	0;
}


cfg_err_t
cfg_date_print(FILE* out,const void* x, int UNUSED(indent)) {
    /* print format <year>/<month>/<day>[:<hour>] */
    cfg_date_t* d = CFG_CAST( cfg_date_t, x);
    int32_t year, month, day;
    gregorianDate( d->date, &year, &month, &day);
    CFG_FPRINTF(( out, "%.2d/%.2d/%.2d", year, month, day));
    if (d->hour != UNSPECIFIED_HOUR) {
	CFG_FPRINTF(( out, ":%.2d", d->hour));
    }
    return CFG_OK;
}

