/*
** Copyright (C) 2004-2005 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
** 
** Use of the SILK system and related source code is subject to the terms 
** of the following licenses:
** 
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
** 
** NO WARRANTY
** 
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
** DELIVERABLES UNDER THIS LICENSE.
** 
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
** Mellon University, its trustees, officers, employees, and agents from 
** all claims or demands made against them (and any related losses, 
** expenses, or attorney's fees) arising out of, or relating to Licensee's 
** and/or its sub licensees' negligent use or willful misuse of or 
** negligent conduct or willful misconduct regarding the Software, 
** facilities, or other rights or assistance granted by Carnegie Mellon 
** University under this License, including, but not limited to, any 
** claims of product liability, personal injury, death, damage to 
** property, or violation of any laws or regulations.
** 
** Carnegie Mellon University Software Engineering Institute authored 
** documents are sponsored by the U.S. Department of Defense under 
** Contract F19628-00-C-0003. Carnegie Mellon University retains 
** copyrights in all material produced under this contract. The U.S. 
** Government retains a non-exclusive, royalty-free license to publish or 
** reproduce these documents, or allow others to do so, for U.S. 
** Government purposes only pursuant to the copyright license under the 
** contract clause at 252.227.7013.
** 
** @OPENSOURCE_HEADER_END@
*/

/*
**  packutils-common.c
**
**  utilities for packing raw data
*/

#include "silk.h"

RCSIDENT("$SiLK: /src/rw/rwpack/packutils-common.c,v 1.4 2005/05/27 20:58:40 thomasm Exp $");

#include "utils.h"
#include "rwpack.h"
#include "log.h"


#define	DEBUG 0


/*
** parseIPinOptions:
**	parse ipnumbers in dotted decimal comma separated
** Input: char *, array in which to stash the numbers. Max allowed 4
** Output: 0 if failed; # of ip's if OK
** Side Effects:
**	the input array is modified to include the ip numbers
*/
int32_t parseIPinOptions(const char *data, uint32_t *ipArray)
{
    char ipbuf[32];
    const char *cp, *ep;
    int32_t i;
    uint32_t ip;

    cp = data;
    i = 0;
    while (*cp) {
        ep = strchr(cp, ',');
        if (ep == cp) {
            /* double comma */
            ++cp;
            continue;
        }
        if (!ep) {
            /* cp is already null terminated; set ep to last char in cp,
             * i.e. '\0' */
            ep = cp+strlen(cp);
        } else {
            /* copy (ep-cp) characters from data into ipbuf; set cp to point
             * at ipbuf; increment ep to start of next ip */
            if ((size_t)(ep-cp) >= sizeof(ipbuf)-1) {
                fprintf(stderr, "%s: ip address too long\n",
                        skAppName());
                return 0;
            }
            strncpy(ipbuf, cp, ep-cp);
            ipbuf[ep-cp] = '\0';
            cp = ipbuf;
            ++ep;
        }

        ip = dot2num(cp);
        if (ip == 0xFFFFFFFF) {
            fprintf(stderr, "invalid ip '%s'\n", cp);
            return 0;			/* failure */
        }
        if (i >= MAX_IP_NUMS) {
            fprintf(stderr, "too many isp ipnums\n");
            return 0;			/* failure */
        }
        ipArray[i] = ip;
        i++;
        cp = ep;
    }

#if	DEBUG
    {
        int32_t j;
        fprintf(stdout, "parseIPinOptions: %s ->", data);
        for (j = 0; j < i; j++) {
            fprintf(stdout, "%s ", num2dot(ipArray[j]));
        }
        fprintf(stdout, "\n");
    }
#endif

    return i;
}


/*
** parseIndexinOptions:
**	parse index in  comma separated list
** Input: char *, uint8_t array in which to set the relevant index to 1
** Output: 0 if failed. # of index numbers if ok
** Side Effects:
**	the input array is modified to include the ip numbers
*/
int32_t parseIndexinOptions(char *data, uint8_t array[], uint8_t n)
{
    char *cp;
    char *ep;
    unsigned long index;
    int count = 0;

    cp = data;
    while (*cp) {
        ep = strchr(cp, ',');
        if (ep) {
            *ep = '\0';
        }
        index = strtoul(cp, (char **)NULL, 10);
        if (index >= SK_SNMP_INDEX_LIMIT) {
            logMsg("Invalid index number %lu\n", index);
            return 0;			/* error */
        }
        array[index] = n;
        count++;
        if (ep) {
            *ep = ',';
            cp = ep+1;
        } else {
            break;
        }
    }
#if	DEBUG
    {
        int32_t j;
        fprintf(stdout, "parseIndexinOptions: %s ->", data);
        for (j = 0; j < SK_SNMP_INDEX_LIMIT; j++) {
            if (array[j]) {
                fprintf(stdout, "%d ",j);
            }
        }
        fprintf(stdout, "\n");
    }
#endif
    return count;
}
