/*
** Copyright (C) 2005 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
** 
** Use of the SILK system and related source code is subject to the terms 
** of the following licenses:
** 
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
** 
** NO WARRANTY
** 
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
** DELIVERABLES UNDER THIS LICENSE.
** 
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
** Mellon University, its trustees, officers, employees, and agents from 
** all claims or demands made against them (and any related losses, 
** expenses, or attorney's fees) arising out of, or relating to Licensee's 
** and/or its sub licensees' negligent use or willful misuse of or 
** negligent conduct or willful misconduct regarding the Software, 
** facilities, or other rights or assistance granted by Carnegie Mellon 
** University under this License, including, but not limited to, any 
** claims of product liability, personal injury, death, damage to 
** property, or violation of any laws or regulations.
** 
** Carnegie Mellon University Software Engineering Institute authored 
** documents are sponsored by the U.S. Department of Defense under 
** Contract F19628-00-C-0003. Carnegie Mellon University retains 
** copyrights in all material produced under this contract. The U.S. 
** Government retains a non-exclusive, royalty-free license to publish or 
** reproduce these documents, or allow others to do so, for U.S. 
** Government purposes only pursuant to the copyright license under the 
** contract clause at 252.227.7013.
** 
** @OPENSOURCE_HEADER_END@
*/

/*
**  Support file for probeconf.c
**
**  probeconf.c includes this C file.  Do NOT add this C file to the
**  Makefile directly.
**
*/


RCSIDENTVAR(rcsID_generic, "$SiLK: /src/rw/rwpack/probeconf-generic.c,v 1.2 2005/05/20 17:54:55 thomasm Exp $");


/*
 *  Verify probe by its class.
 *
 *  NOTE: This function is known as skProbeVerifyClass in the object
 *  file.
 */
int skProbeVerifyClassGeneric(probe_def_t *probe)
{ 
    /* make certain at least one input interface has been specified */
    if (probe->incoming_if_count == 0) {
        skAppPrintErr("at least one input interface is required");
        return -1;
    }

    /* return the result of the type-specific checks */
    switch (probe->probe_type) {
      case PROBE_ENUM_NETFLOW:
        return skProbeVerifyNetflow(probe);
      case PROBE_ENUM_INVALID:
        skAppPrintErr("probe's type is INVALID");
        break;
      default:
        skAppPrintErr("probe type %d not handled in switch at %s:%d",
                      probe->probe_type, __FILE__, __LINE__);
        break;
    }

    return -1;
}


/*
 *  Update the flow_type and sID fields on the 'rwrec'.
 *
 *  NOTE: This function is known as skProbeDetermineFlowtype in the
 *  object file.
 */
int skProbeDetermineFlowtypeGeneric(
    const probe_def_t  *probe,
    rwRec              *rwrec)
{
    sensorinfo_t sensor_info;
    const sensorinfo_t *s_info;

    s_info = probeConfSensorGetInfo(&sensor_info, probe->sensor_id);
    rwrec->sID = probe->sensor_id;

    if (BITMAP_GETBIT(probe->incoming_if_map, rwrec->input)) {
        /* Flow is coming in from the ISP */
        if (rwrec->output == probe->null_interface) {
            /* Deal with blocked flows (ACL, etc.) */
            return (rwrec->flow_type = RW_IN_NULL);
        } else if (IS_WEB(rwrec)) {
            return (rwrec->flow_type = RW_IN_WEB);
        } else {
            return (rwrec->flow_type = RW_IN);
        }
    } else {
        /* Flow is going out to the ISP */
        if (rwrec->output == probe->null_interface) {
            /* Deal with blocked flows (ACL, etc.) */
            return (rwrec->flow_type = RW_OUT_NULL);
        } else if (IS_WEB(rwrec)) {
            return (rwrec->flow_type = RW_OUT_WEB);
        } else {
            return (rwrec->flow_type = RW_OUT);
        }
    }

    return -1;
}
