%array
    /* force yylex to be an array; Solaris lex requires this first */
%{
/*
** Copyright (C) 2005 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
**
** Use of the SILK system and related source code is subject to the terms
** of the following licenses:
**
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
**
** NO WARRANTY
**
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE,
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT,
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES.
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE
** DELIVERABLES UNDER THIS LICENSE.
**
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie
** Mellon University, its trustees, officers, employees, and agents from
** all claims or demands made against them (and any related losses,
** expenses, or attorney's fees) arising out of, or relating to Licensee's
** and/or its sub licensees' negligent use or willful misuse of or
** negligent conduct or willful misconduct regarding the Software,
** facilities, or other rights or assistance granted by Carnegie Mellon
** University under this License, including, but not limited to, any
** claims of product liability, personal injury, death, damage to
** property, or violation of any laws or regulations.
**
** Carnegie Mellon University Software Engineering Institute authored
** documents are sponsored by the U.S. Department of Defense under
** Contract F19628-00-C-0003. Carnegie Mellon University retains
** copyrights in all material produced under this contract. The U.S.
** Government retains a non-exclusive, royalty-free license to publish or
** reproduce these documents, or allow others to do so, for U.S.
** Government purposes only pursuant to the copyright license under the
** contract clause at 252.227.7013.
**
** @OPENSOURCE_HEADER_END@
*/

/*
**  Tokenizer for probe configuration file.
**
*/


#include "silk.h"

RCSIDENT("$SiLK: /src/rw/rwpack/probeconfscan.l,v 1.5 2005/07/06 15:21:50 thomasm Exp $");

#include "probeconfscan.h"
#include "probeconfparse.h"


/* current line number */
int probescan_line;


char probescan_clause[128];

static int stash(int tok);

%}

    /* create a state that says we have read a keyword and are parsing a
     * clause */

%s CLAUSE

    /* basic character classes */

idchar					[-._/A-Za-z0-9]

notidchar				[^-._/A-Za-z0-9\n]

    /* an ID must have at least one non-number character; note that this
     * must also match hostnames and IP addresses */

id					[-._/A-Za-z0-9]+

    /* comment; ignore, and increment line count */

comment					(#.*\n)

    /* whitespace; we ignore it */

ws					[ \t\r]+

    /* end of line; increment line count */

nl					\n

%%

{ws}					;

{comment}				{ ++probescan_line; BEGIN(INITIAL); }

{nl}					{ ++probescan_line; BEGIN(INITIAL); }


<INITIAL>sensor{notidchar}		{ return stash(SENSOR_T); }

<INITIAL>class{notidchar}		|
<INITIAL>classes{notidchar}		{ return stash(CLASS_T); }


<INITIAL>sensor-probe{notidchar}	{ return stash(SENSOR_PROBE_T); }

<INITIAL>probe-name{notidchar}		{ return stash(PROBE_NAME_T); }

<INITIAL>null-interface{notidchar}	{ return stash(NULL_INTERFACE_T); }

<INITIAL>input-index{notidchar}		|
<INITIAL>input-indexes{notidchar}	{ return stash(INPUT_INDEX_T); }

<INITIAL>output-index{notidchar}	|
<INITIAL>output-indexes{notidchar}	{ return stash(OUTPUT_INDEX_T); }

<INITIAL>isp-ips?{notidchar}		{ return stash(ISP_IP_T); }

<INITIAL>probe-type{notidchar}		{ return stash(PROBE_TYPE_T); }

<INITIAL>priority{notidchar}		{ return stash(PRIORITY_T); }

<INITIAL>listen-as-host{notidchar}	{ return stash(LISTEN_AS_HOST_T); }

<INITIAL>listen-on-port{notidchar}	{ return stash(LISTEN_ON_PORT_T); }

<INITIAL>listen-on-unix-domain-socket{notidchar}   { return stash(LISTEN_ON_USOCKET_T); }

<INITIAL>read-from-file{notidchar}      { return stash(READ_FROM_FILE_T); }

<INITIAL>accept-from-host{notidchar}	{ return stash(ACCEPT_FROM_HOST_T); }

<CLAUSE>{id}				{ yylval.string = yytext; return ID; }

<CLAUSE>","				{ return COMMA; }

.					{ /* unknown */ return yytext[0]; }

%%

/*
 *  stash(int)
 *
 *      Stash the current keyword (using the yyleng and yytext
 *      globals), put us into a CLAUSE start-state, and return the
 *      value we were passed.
 */
static int stash(int tok)
{
    /* make certain the keyword fits in our buffer */
    assert(yyleng < ((int)sizeof(probescan_clause)));

    /* copy it in */
    strcpy(probescan_clause, yytext);

    /* remember we got an extra character, so terminate the keyword a
     * character earlier, and unput that character */
    probescan_clause[yyleng-1] = '\0';
    unput(yytext[yyleng-1]);

    /* we're starting a clause */
    BEGIN(CLAUSE);

    /* return whatever we were passed */
    return tok;
}


/*
 * We stop parsing at end-of-file.  Define our own yywrap so
 * we don't need to link against -ll.
 */
#ifndef yywrap
int yywrap() { return 1; }
#endif

/*
** Local variables:
** mode:c
** indent-tabs-mode:nil
** c-basic-offset:4
** End:
*/
