/*
** Copyright (C) 2004-2005 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
** 
** Use of the SILK system and related source code is subject to the terms 
** of the following licenses:
** 
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
** 
** NO WARRANTY
** 
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
** DELIVERABLES UNDER THIS LICENSE.
** 
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
** Mellon University, its trustees, officers, employees, and agents from 
** all claims or demands made against them (and any related losses, 
** expenses, or attorney's fees) arising out of, or relating to Licensee's 
** and/or its sub licensees' negligent use or willful misuse of or 
** negligent conduct or willful misconduct regarding the Software, 
** facilities, or other rights or assistance granted by Carnegie Mellon 
** University under this License, including, but not limited to, any 
** claims of product liability, personal injury, death, damage to 
** property, or violation of any laws or regulations.
** 
** Carnegie Mellon University Software Engineering Institute authored 
** documents are sponsored by the U.S. Department of Defense under 
** Contract F19628-00-C-0003. Carnegie Mellon University retains 
** copyrights in all material produced under this contract. The U.S. 
** Government retains a non-exclusive, royalty-free license to publish or 
** reproduce these documents, or allow others to do so, for U.S. 
** Government purposes only pursuant to the copyright license under the 
** contract clause at 252.227.7013.
** 
** @OPENSOURCE_HEADER_END@
*/
#ifndef _RWPACK_NEW_H
#define _RWPACK_NEW_H

#include "silk.h"

RCSIDENTVAR(rcsID_RWPACK_NEW_H, "$SiLK: /src/rw/rwpack/rwpack-new.h,v 1.10 2005/05/20 17:54:59 thomasm Exp $");


/*
**  rwpack-new.h
**
**  Packing rules and related utilities.
**
*/

#include "rwpack.h"
#include "silk_site.h"
#include "skvector.h"



/* Web classification utilities; 'p' is a port in native byte order;
 * 'r' is an RWGENERIC record in native byte order. */
#define WEBP(p) ((p) == 80 || (p) == 443 || (p) == 8080)
#define WEBPORT(p) (((p) == 80) ? 0 : (((p) == 443) ? 1 : (((p) == 8080) ? 2 : 3)))
#define IS_WEB_PORT(p) WEBP(p)
#define IS_WEB(r) ((r).proto == 6 && (IS_WEB_PORT((r).sPort) || IS_WEB_PORT((r).dPort)))


struct _deployment;
typedef struct _deployment deployment_t;


typedef struct streamInfo streamInfo_t;

/* Initialize internal packing variables.  Return 0 if ok, non-zero
 * otherwise. */
int skPackInitialize(void);

/* FIXME. Consider making these global utilities as opposed to packing
 * utilities */
/* Configuration read from the command line */
void skPackSetRootDir(const char *rootDir);
const char *skPackGetRootDir(void);

void skPackBuildStorageFileName(
    char       *fName,
    int         maxlen,
    uint32_t    sTime,
    const char *pathPrefix,
    const char *filePrefix,
    const char *sID);
/*
 *    Build a file name for storing the packed data.  'out_fName' is the
 *    buffer where the name should be copied; 'maxlen' is the buffer's
 *    length; 'sTime' is the start time (start of the hour) in UNIX
 *    epoch seconds; 'pathPrefix' is the prefix before the date part of
 *    the path; 'filePrefix' is the file prefix for the file; 'sID' is
 *    the sensor name.
 */


#endif /* _RWPACK_NEW_H */
