/*
** Copyright (C) 2005 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
** 
** Use of the SILK system and related source code is subject to the terms 
** of the following licenses:
** 
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
** 
** NO WARRANTY
** 
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
** DELIVERABLES UNDER THIS LICENSE.
** 
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
** Mellon University, its trustees, officers, employees, and agents from 
** all claims or demands made against them (and any related losses, 
** expenses, or attorney's fees) arising out of, or relating to Licensee's 
** and/or its sub licensees' negligent use or willful misuse of or 
** negligent conduct or willful misconduct regarding the Software, 
** facilities, or other rights or assistance granted by Carnegie Mellon 
** University under this License, including, but not limited to, any 
** claims of product liability, personal injury, death, damage to 
** property, or violation of any laws or regulations.
** 
** Carnegie Mellon University Software Engineering Institute authored 
** documents are sponsored by the U.S. Department of Defense under 
** Contract F19628-00-C-0003. Carnegie Mellon University retains 
** copyrights in all material produced under this contract. The U.S. 
** Government retains a non-exclusive, royalty-free license to publish or 
** reproduce these documents, or allow others to do so, for U.S. 
** Government purposes only pursuant to the copyright license under the 
** contract clause at 252.227.7013.
** 
** @OPENSOURCE_HEADER_END@
*/
#ifndef _RWPPACKETHEADERS_H
#define _RWPPACKETHEADERS_H

#include "silk.h"

RCSIDENTVAR(rcsID_RWPMATCH_H, "$SiLK: /src/rw/rwpmatch/rwppacketheaders.h,v 1.2 2005/11/10 19:15:32 leechr Exp $");


/*
**  rwppacketheaders.h
**
**  Headers for ethernet, IP, tcp, and udp packets.
**
*/

/* mask with the IP header flags/fragment offset field to get the
   fragment offset. */
#define IPHEADER_FO_MASK 0x1FFF



typedef struct _eth_header_t {
    uint8_t  ether_dhost[6]; /* destination eth addr	*/
    uint8_t  ether_shost[6]; /* source ether addr	*/
    uint16_t ether_type; /* packet type ID field	*/
} __attribute__ ((__packed__)) eth_header_t;

#ifndef ETHERTYPE_IP
#define ETHERTYPE_IP 0x0800
#endif

/* IPv4 header */
typedef struct _ip_header_t{
    uint8_t  ver_ihl; /* version (4 bits) + iphdr length in words (4 bits) */
    uint8_t  tos;
    uint16_t tlen; /* total length of hdr + payload*/
    uint16_t identification; /* Identification */
    uint16_t flags_fo; /* Flags (3 bits) + Fragment offset (13 bits) */
    uint8_t  ttl;
    uint8_t  proto;
    uint16_t crc;
    ipUnion saddr;
    ipUnion daddr;
    /* variable length options... */
} ip_header_t;


/* ICMP header */
typedef struct _icmp_header_t{
    uint8_t type;
    uint8_t code;
    uint16_t checksum;
} icmp_header_t;

/* TCP header */
typedef struct _tcp_header_t {
    uint16_t sport;
    uint16_t dport;
    uint32_t seqNum;
    uint32_t ackNum;
    uint16_t o_r_e; /* offset, reserved, ecn */
    uint16_t window;
    uint16_t checksum;
    uint16_t urgentPtr;
    /* Variable length options and padding here */
} tcp_header_t;

/* UDP header */
typedef struct _udp_header_t{
    uint16_t sport;
    uint16_t dport;
    uint16_t len;
    uint16_t crc;
} udp_header_t;



#endif /* _RWPPACKETHEADERS_H */
