/*
** Copyright (C) 2005 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
**
** Use of the SILK system and related source code is subject to the terms
** of the following licenses:
**
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
**
** NO WARRANTY
**
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE,
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT,
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES.
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE
** DELIVERABLES UNDER THIS LICENSE.
**
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie
** Mellon University, its trustees, officers, employees, and agents from
** all claims or demands made against them (and any related losses,
** expenses, or attorney's fees) arising out of, or relating to Licensee's
** and/or its sub licensees' negligent use or willful misuse of or
** negligent conduct or willful misconduct regarding the Software,
** facilities, or other rights or assistance granted by Carnegie Mellon
** University under this License, including, but not limited to, any
** claims of product liability, personal injury, death, damage to
** property, or violation of any laws or regulations.
**
** Carnegie Mellon University Software Engineering Institute authored
** documents are sponsored by the U.S. Department of Defense under
** Contract F19628-00-C-0003. Carnegie Mellon University retains
** copyrights in all material produced under this contract. The U.S.
** Government retains a non-exclusive, royalty-free license to publish or
** reproduce these documents, or allow others to do so, for U.S.
** Government purposes only pursuant to the copyright license under the
** contract clause at 252.227.7013.
**
** @OPENSOURCE_HEADER_END@
*/
#ifndef _PRINT_NETWORK_STRUCTURE_H
#define _PRINT_NETWORK_STRUCTURE_H

#include "silk.h"

RCSIDENTVAR(rcsID_PRINT_NETWORK_STRUCTURE_H, "$SiLK: /src/rw/rwset/print_network_structure.h,v 1.6 2005/10/27 16:30:15 leechr Exp $");

#include "bagtree.h"

#define OCTET_A( key ) ( key >> 24 )
#define OCTET_B( key ) ( ( key & 0x00FF0000) >> 16 )
#define OCTET_C( key ) ( ( key & 0x0000FF00) >> 8 )
#define OCTET_X( key ) ( ( key & 0xE0 ) )

#define PRINT_SUMMARY_DELIMITER " : "

#define MASK_A 0xFF000000
#define MASK_AB 0xFFFF0000
#define MASK_ABC 0xFFFFFF00
#define MASK_ABCX 0xFFFFFFE0

#define PRINT_HOSTCOUNT "%d"
#define PRINT_COUNT "%llu"

#define PRINT_T_SUMMARY "%llu hosts in %d /8s, %d /16s, %d /24s, %d /27s"
#define PRINT_A_SUMMARY "%d hosts in %d /16s, %d /24s, %d /27s"
#define PRINT_B_SUMMARY "%d hosts in %d /24s, %d /27s"
#define PRINT_C_SUMMARY "%d hosts in %d /27s"
#define PRINT_X_SUMMARY "%d hosts"

typedef struct _print_state {
    /* how many class A addresses have been seen */
    int seen_A;
    uint64_t count_A;
    /* how many class B addresses have been seen */
    int seen_B;
    uint64_t count_B;
    /* how many class C addresses have been seen */
    int seen_C;
    uint64_t count_C;
    /* how many /27 addresses have been seen */
    int seen_X;
    uint64_t count_X;
    /* how many hosts have been seen */
    uint64_t seen_H;
    uint64_t count_H;

    /* for the current class A, how many class B addresses have been seen */
    int class_A_seen_B;
    uint64_t class_A_count_B;
    /* for the current class A, how many class C addresses have been seen */
    int class_A_seen_C;
    uint64_t class_A_count_C;
    /* for the current class A, how many /27 addresses have been seen */
    int class_A_seen_X;
    uint64_t class_A_count_X;
    /* for the current class A, how many hosts have been seen */
    int class_A_seen_H;
    uint64_t class_A_count_H;

    /* for the current class B, how many class C addresses have been seen */
    int class_B_seen_C;
    uint64_t class_B_count_C;
    /* for the current class B, how many /27 addresses have been seen */
    int class_B_seen_X;
    uint64_t class_B_count_X;
    /* for the current class B, how many hosts have been seen */
    int class_B_seen_H;
    uint64_t class_B_count_H;

    /* for the current class C, how many /27 addresses have been seen */
    int class_C_seen_X;
    uint64_t class_C_count_X;
    /* for the current class C, how many hosts have been seen */
    int class_C_seen_H;
    uint64_t class_C_count_H;

    /* for the current /27, how many hosts have been seen */
    int class_X_seen_H;
    uint64_t class_X_count_H;
    
    skBag_key_t prev_key; /* the last key read so far, to detect 
                             boundary changes */
    int f_first_entry; /* is this the first key printed? */
    int f_last_entry; /* was the last key already printed? */    
} print_state_t;



#endif /* _PRINT_NETWORK_STRUCTURE_H */
