/*
** Copyright (C) 2001-2005 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
** 
** Use of the SILK system and related source code is subject to the terms 
** of the following licenses:
** 
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
** 
** NO WARRANTY
** 
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
** DELIVERABLES UNDER THIS LICENSE.
** 
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
** Mellon University, its trustees, officers, employees, and agents from 
** all claims or demands made against them (and any related losses, 
** expenses, or attorney's fees) arising out of, or relating to Licensee's 
** and/or its sub licensees' negligent use or willful misuse of or 
** negligent conduct or willful misconduct regarding the Software, 
** facilities, or other rights or assistance granted by Carnegie Mellon 
** University under this License, including, but not limited to, any 
** claims of product liability, personal injury, death, damage to 
** property, or violation of any laws or regulations.
** 
** Carnegie Mellon University Software Engineering Institute authored 
** documents are sponsored by the U.S. Department of Defense under 
** Contract F19628-00-C-0003. Carnegie Mellon University retains 
** copyrights in all material produced under this contract. The U.S. 
** Government retains a non-exclusive, royalty-free license to publish or 
** reproduce these documents, or allow others to do so, for U.S. 
** Government purposes only pursuant to the copyright license under the 
** contract clause at 252.227.7013.
** 
** @OPENSOURCE_HEADER_END@
*/

/*
**  rwtotalutils.c
**
**
*/

#include "silk.h"

RCSIDENT("$SiLK: /src/rw/rwtotal/rwtotalutils.c,v 1.18 2005/10/26 14:23:18 thomasm Exp $");

#include "rwpack.h"
#include "utils.h"
#include "iochecks.h"
#include "rwtotal.h"


/* LOCAL DEFINES AND TYPEDEFS */

/* where to write --help output */
#define USAGE_FH stdout


/* Values to use for appOptions and option_to_count_mode[].
 * option_to_count_mode[] depends on the order of IDs here; keep them
 * in sync!! */ 
typedef enum {
    RWTO_SIP_8=0, RWTO_SIP_16,  RWTO_SIP_24,
    RWTO_SIP_L8,  RWTO_SIP_L16,
    RWTO_DIP_8,   RWTO_DIP_16,  RWTO_DIP_24,
    RWTO_DIP_L8,  RWTO_DIP_L16,
    RWTO_SPORT,   RWTO_DPORT,
    RWTO_PROTO,
    RWTO_PACKETS,
    RWTO_BYTES,
    RWTO_DURATION,
    RWTO_ICMPCODE,
    /* above map to count-modes; below control output */
    RWTO_DELIMITER,
    RWTO_SKIP_ZEROES,
    RWTO_NO_TITLES,
    RWTO_PAGER,
    RWTO_PRINT_FILENAMES,
    RWTO_COPY_INPUT
} appOptionsEnum;


/* a mapping from appOptionsEnum to the countMode, totalRecs, &
 * destFlag. */
typedef struct {
    int         count_mode;
    uint32_t    total_recs;
    int         dest_flag;
} option_mode_t;


/* LOCAL FUNCTIONS */

static void appUsageLong(void);
static void appOptionsUsage(FILE *fh);
static int  appOptionsHandler(clientData cData, int opt_index, char *opt_arg);


/* LOCAL VARIABLES */

/* a mapping from appOptionsEnum to the countMode, totalRecs, &
 * destFlag.  This depends on ordering in appOptionsEnum!! */
static option_mode_t option_to_count_mode[] = {
    /* RWTO_SIP_8    */ {RWTO_COPT_ADD8,     (1 <<  8), 0},
    /* RWTO_SIP_16   */ {RWTO_COPT_ADD16,    (1 << 16), 0},
    /* RWTO_SIP_24   */ {RWTO_COPT_ADD24,    (1 << 24), 0},
    /* RWTO_SIP_L8   */ {RWTO_COPT_LADD8,    (1 <<  8), 0},
    /* RWTO_SIP_L16  */ {RWTO_COPT_LADD16,   (1 << 16), 0},

    /* RWTO_DIP_8    */ {RWTO_COPT_ADD8,     (1 <<  8), 1},
    /* RWTO_DIP_16   */ {RWTO_COPT_ADD16,    (1 << 16), 1},
    /* RWTO_DIP_24   */ {RWTO_COPT_ADD24,    (1 << 24), 1},
    /* RWTO_DIP_L8   */ {RWTO_COPT_LADD8,    (1 <<  8), 1},
    /* RWTO_DIP_L16  */ {RWTO_COPT_LADD16,   (1 << 16), 1},

    /* RWTO_SPORT    */ {RWTO_COPT_PORT,     (1 << 16), 0},
    /* RWTO_DPORT    */ {RWTO_COPT_PORT,     (1 << 16), 1},
    /* RWTO_PROTO    */ {RWTO_COPT_PROTO,    (1 <<  8), 0},
    /* RWTO_PACKETS  */ {RWTO_COPT_PACKETS,  (1 << 24), 0},
    /* RWTO_BYTES    */ {RWTO_COPT_BYTES,    (1 << 24), 0},
    /* RWTO_DURATION */ {RWTO_COPT_DURATION,      4096, 0},
    /* RWTO_ICMPCODE */ {RWTO_COPT_ICMP,     (1 << 16), 0},
};


/* OPTIONS SETUP */

static struct option appOptions[] = {
    {"sip-first-8",     NO_ARG,       0, RWTO_SIP_8},
    {"sip-first-16",    NO_ARG,       0, RWTO_SIP_16},
    {"sip-first-24",    NO_ARG,       0, RWTO_SIP_24},
    {"sip-last-8",      NO_ARG,       0, RWTO_SIP_L8},
    {"sip-last-16",     NO_ARG,       0, RWTO_SIP_L16},

    {"dip-first-8",     NO_ARG,       0, RWTO_DIP_8},
    {"dip-first-16",    NO_ARG,       0, RWTO_DIP_16},
    {"dip-first-24",    NO_ARG,       0, RWTO_DIP_24},
    {"dip-last-8",      NO_ARG,       0, RWTO_DIP_L8},
    {"dip-last-16",     NO_ARG,       0, RWTO_DIP_L16},

    {"sport",           NO_ARG,       0, RWTO_SPORT},
    {"dport",           NO_ARG,       0, RWTO_DPORT},
    {"proto",           NO_ARG,       0, RWTO_PROTO},
    {"packets",         NO_ARG,       0, RWTO_PACKETS},
    {"bytes",           NO_ARG,       0, RWTO_BYTES},
    {"duration",        NO_ARG,       0, RWTO_DURATION},
    {"icmp-code",       NO_ARG,       0, RWTO_ICMPCODE},

    {"delimiter",       REQUIRED_ARG, 0, RWTO_DELIMITER},
    {"skip-zeroes",     NO_ARG,       0, RWTO_SKIP_ZEROES},
    {"no-titles",       NO_ARG,       0, RWTO_NO_TITLES},
    {"pager",           REQUIRED_ARG, 0, RWTO_PAGER},
    {"print-filenames", NO_ARG,       0, RWTO_PRINT_FILENAMES},
    {"copy-input",      REQUIRED_ARG, 0, RWTO_COPY_INPUT},
    {0,0,0,0}                     /* sentinal entry */
};

static char *appHelp[] = {
    "Key on the first 8 bits of the source IP address",
    "Key on the first 16 bits of the source IP address",
    "Key on the first 24 bits of the source IP address",
    "Key on the last 8 bits of the source IP address",
    "Key on the last 16 bits of the source IP address",

    "Key on the first 8 bits of the destination IP address",
    "Key on the first 16 bits of the destination IP address",
    "Key on the first 24 bits of the destination IP address",
    "Key on the last 8 bits of the destination IP address",
    "Key on the last 16 bits of the  destination  IP address",

    "Key on the source port",
    "Key on the destination port",
    "Key on the protocol",
    "Key on the number of packets",
    "Key on the number of bytes",
    "Key on duration",
    ("Key on icmp type and code (DOES NOT check to see\n"
     "\t if the record is ICMP)\n"
     "\noptions:"),

    "Delimiter between columns. Def. '|'",
    "Do not print bins that have no flows. Def. Print all",
    "Do not print column titles. Def. Print titles",
    "Program to invoke to page output. Def. $SILK_PAGER or $PAGER",
    "Print names of input files as they are opened. Def. No",
    "Copy all input SiLK Flows to given pipe or file. Def. No",
    (char *)NULL
};



/*
 * appUsage:
 *      print usage information to stderr and exit with code 1
 * Arguments: None
 * Returns: None
 * Side Effects: exits application.
 */
static void appUsage(void)
{
    fprintf(stderr, "Use `%s --help' for usage\n", skAppName());
    exit(EXIT_FAILURE);
}


/*
 * appUsageLong:
 *      print usage information to USAGE_FH and exits with code 1.
 *      passed to optionsSetup() to print usage when --help option
 *      given.
 * Arguments: None
 * Returns: None
 * Side Effects: exits application.
 */
static void appUsageLong(void)
{
#define USAGE_MSG                                                             \
    ("%s <key> [options] [files]\n"                                           \
     "\tSummarize SiLK Flow records by a specified key and print byte,\n"     \
     "\tpacket, and flow counts for flows matching the key.  When no files\n" \
     "\tgiven on command line, flows are read from STDIN.\n")
    fprintf(USAGE_FH, USAGE_MSG, skAppName());
    appOptionsUsage(USAGE_FH);
    exit(EXIT_FAILURE);
}


/*
 * appOptionsUsage:
 *      print help for options for this app to the specified file handle.
 * Arguments: None.
 * Returns: None.
 * Side Effects: None.
 * NOTE:
 *  do NOT add non-option usage here (i.e., required and other optional args)
 */

static void appOptionsUsage(FILE *fh)
{
    int i;

    fprintf(fh, "\npossible keys:\n");
    for (i = 0; appOptions[i].name; ++i) {
        fprintf(fh, "--%s %s. %s\n", appOptions[i].name,
                SK_OPTION_HAS_ARG(appOptions[i]), appHelp[i]);
    }
}


/*
 * appTeardown:
 *      teardown all used modules and all application stuff.
 * Arguments: None
 * Returns: None
 * Side Effects:
 *      All modules are torn down. Then application teardown takes place.
 *      Static variable teardownFlag is set.
 * NOTE: This must be idempotent using static teardownFlag.
 */
void appTeardown(void)
{
    static uint8_t teardownFlag = 0;

    if (teardownFlag) {
        return;
    }
    teardownFlag = 1;

    if (ioISP->inputCopyFD) {
        rwCloseFile(ioISP->inputCopyFD);
        ioISP->inputCopyFD = NULL;
    }

    iochecksTeardown(ioISP);
    optionsTeardown();

    return;
}


/*
 * appSetup
 *      do all the setup for this application include setting up required
 *       modules etc.
 * Arguments:
 *      argc, argv
 * Returns: None.
 * Side Effects:
 *      exits with code 1 if anything does not work.
 */
void appSetup(int argc, char **argv)
{
    /* register application */
    skAppRegister(argv[0]);

    /* initialize variables */
    countMode = RWTO_NONE;
    skip_zeroes_flag = 0;

    /* check that we have the same number of options entries and help*/
    assert((sizeof(appHelp)/sizeof(char *)) ==
           (sizeof(appOptions)/sizeof(struct option)));

    /* Create input/output checker */
    ioISP = iochecksSetup(0, 0, argc, argv);

    /* initialize options module */
    if (optionsSetup(appUsageLong)) {
        skAppPrintErr("unable to setup options module");
        exit(EXIT_FAILURE);
    }

    /* register our options and handler */
    if (optionsRegister(appOptions, (optHandler)appOptionsHandler, NULL)) {
        skAppPrintErr("unable to register application options");
        exit(EXIT_FAILURE);
    }

    /* parse options */
    ioISP->firstFile = optionsParse(argc, argv);
    if (ioISP->firstFile < 0) {
        appUsage();         /* never returns */
    }

    /* make certain we have something to count */
    if(countMode == RWTO_NONE) {
        skAppPrintErr("No key specified,\n"
                      "\t Please choose a summarization key.\n");
        appUsage();
    }

    /* Use STDIN as an input stream if it is not a TTY; make certain
     * we have some input and we are either reading from STDIN or
     * using files listed the command line, but not both. */
    if (iochecksAcceptFromStdin(ioISP) || iochecksInputs(ioISP, 0)) {
        appUsage();
    }

    if (atexit(appTeardown) < 0) {
        skAppPrintErr("unable to register appTeardown() with atexit()");
        appTeardown();
        exit(EXIT_FAILURE);
    }

    (void)umask((mode_t) 0002);

    return; /* OK */
}



/*
 * appOptionsHandler:
 *      called for each option the app has registered.
 * Arguments:
 *      clientData cData: ignored here.
 *      int index: index into appOptions of the specific option
 *      char *optarg: the argument; 0 if no argument was required/given.
 * Returns:
 *      0 if OK. 1 else.
 * Side Effects:
 *      Relevant options are set.
 */
static int appOptionsHandler(
    clientData  UNUSED(cData),
    int         opt_index,
    char       *opt_arg)
{
    /* handle the switches that set the key/count-mode */
    if (opt_index < (int)(sizeof(option_to_count_mode)/sizeof(option_mode_t))){
        if (countMode != RWTO_NONE) {
            skAppPrintErr("Only one summarization key may be specified");
            return 1;
        }

        countMode = option_to_count_mode[opt_index].count_mode;
        totalRecs = option_to_count_mode[opt_index].total_recs;
        destFlag  = option_to_count_mode[opt_index].dest_flag;
        return 0;
    }

    switch (opt_index) {
      case RWTO_DELIMITER:
        delimiter = opt_arg[0];
        break;

      case RWTO_SKIP_ZEROES:
        skip_zeroes_flag = 1;
        break;

      case RWTO_NO_TITLES:
        no_titles_flag = 1;
        break;

      case RWTO_PAGER:
        pager = opt_arg;
        break;

      case RWTO_PRINT_FILENAMES:
        print_filenames_flag = 1;
        break;

      case RWTO_COPY_INPUT:
        if (iochecksAllDestinations(ioISP, opt_arg)) {
            return 1;
        }
        break;

      default:
        skAppPrintErr("Unhandled option %d in options handler at %s:%d",
                      opt_index, __FILE__, __LINE__);
        exit(EXIT_FAILURE);
    } /* switch */

    return 0;                     /* OK */
}
