#!/usr/bin/perl -w

# Copyright (C) 2004-2005 by Carnegie Mellon University.
#
# @OPENSOURCE_HEADER_START@
# 
# Use of the SILK system and related source code is subject to the terms 
# of the following licenses:
# 
# GNU Public License (GPL) Rights pursuant to Version 2, June 1991
# Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
# 
# NO WARRANTY
# 
# ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
# PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
# PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
# "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
# KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
# LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
# MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
# OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
# SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
# TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
# WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
# LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
# CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
# CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
# DELIVERABLES UNDER THIS LICENSE.
# 
# Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
# Mellon University, its trustees, officers, employees, and agents from 
# all claims or demands made against them (and any related losses, 
# expenses, or attorney's fees) arising out of, or relating to Licensee's 
# and/or its sub licensees' negligent use or willful misuse of or 
# negligent conduct or willful misconduct regarding the Software, 
# facilities, or other rights or assistance granted by Carnegie Mellon 
# University under this License, including, but not limited to, any 
# claims of product liability, personal injury, death, damage to 
# property, or violation of any laws or regulations.
# 
# Carnegie Mellon University Software Engineering Institute authored 
# documents are sponsored by the U.S. Department of Defense under 
# Contract F19628-00-C-0003. Carnegie Mellon University retains 
# copyrights in all material produced under this contract. The U.S. 
# Government retains a non-exclusive, royalty-free license to publish or 
# reproduce these documents, or allow others to do so, for U.S. 
# Government purposes only pursuant to the copyright license under the 
# contract clause at 252.227.7013.
# 
# @OPENSOURCE_HEADER_END@

# geoip-to-silk.pl
#
# John McClary Prevost
# December 6th, 2004
#
# RCSIDENT("$SiLK: /src/util/ccfilter_new/geoip-to-silk.pl,v 1.3 2005/09/13 18:24:30 thomasm Exp $");
#
# Generate a silk-style prefixmap file from an encoded geoip data file.

my $COUNTRY_BEGIN = 16776960;

my $header = pack 'C8', 0xDE, 0xAD, 0xBE, 0xEF, 0x00, 0x25, 0x01, 0x00;
my $records = pack 'V', 0;

print $header;
print $records;

my $in;

my $count = 0;

my @countries = qw(
-- ap eu ad ae af ag ai al am an ao aq ar as at au aw az ba bb bd be
bf bg bh bi bj bm bn bo br bs bt bv bw by bz ca cc cd cf cg ch ci ck
cl cm cn co cr cu cv cx cy cz de dj dk dm do dz ec ee eg eh er es et
fi fj fk fm fo fr fx ga gb gd ge gf gh gi gl gm gn gp gq gr gs gt gu
gw gy hk hm hn hr ht hu id ie il in io iq ir is it jm jo jp ke kg kh
ki km kn kp kr kw ky kz la lb lc li lk lr ls lt lu lv ly ma mc md mg
mh mk ml mm mn mo mp mq mr ms mt mu mv mw mx my mz na nc ne nf ng ni
nl no np nr nu nz om pa pe pf pg ph pk pl pm pn pr ps pt pw py qa re
ro ru rw sa sb sc sd se sg sh si sj sk sl sm sn so sr st sv sy sz tc
td tf tg th tj tk tm tn to tp tr tt tv tw tz ua ug um us uy uz va vc
ve vg vi vn vu wf ws ye yt yu za zm zr zw a1 a2 o1
);

while ( read(STDIN, $in, 6) == 6 ) {
    my ($l0, $l1, $l2, $r0, $r1, $r2) = unpack 'C6', $in;
    if ( (defined $l0) && (defined $l1) && (defined $l2) ||
	 (defined $r0) && (defined $r1) && (defined $r2) ) {
	my $left = ($l2 << 16) + ($l1 << 8) + $l0;
	my $right = ($r2 << 16) + ($r1 << 8) + $r0;
	if ( $left >= $COUNTRY_BEGIN ) {
	    $left -= $COUNTRY_BEGIN;
	    my $cc = $countries[$left];
	    $left = 0x80000000 + ord(substr($cc,0,1)) * 256 + ord(substr($cc,1,1));
	}
	if ( $right >= $COUNTRY_BEGIN ) {
	    $right -= $COUNTRY_BEGIN;
	    my $cc = $countries[$right];
	    $right = 0x80000000 + ord(substr($cc,0,1)) * 256 + ord(substr($cc,1,1));
	}
	my $out = pack 'VV', $left, $right;
	print $out;
	$count++;
    }
}

seek STDOUT, 8, 0;

$records = pack 'V', $count;
print $records;
